
AddComponentPostInit('temperature',function(self)
	local old_getinsu = self.GetInsulation
	function self:GetInsulation(...)
		local winter,summer = old_getinsu(self,...)
		winter = self.cold_resistance_override or winter
		summer = self.hot_resistance_override or summer
		return winter,summer
	end
end)

AddComponentPostInit('inventory',function(self)
	local old_getwindproofness = self.GetWindproofness
	function self:GetWindproofness(slot)
		if slot then
			return old_getwindproofness(self, slot)
		elseif self.inst:HasTag('player') and self:Has("dust_resistant_pill",1) then
			return 1
		else
			return old_getwindproofness(self)
		end
	end
end)

AddComponentPostInit('builder', function(self)
	local old_can = self.CanBuild
	function self:CanBuild(recname,...)
    	if self.freebuildmode then
        	return true
    	end

    	local recipe = GetRecipe(recname)
    	if recipe and recipe.mk_testfn and not recipe.mk_testfn(self.inst) then
    		return false
    	end
    	return old_can(self,recname,...)
    end
end)

AddComponentPostInit('playercontroller',function(self)
	local checkaction = function()
		local _,action = self.inst.components.playeractionpicker:DoGetMouseActions()
		if not action or action.action == ACTIONS.LOOKAT or action.action == ACTIONS.WALKTO then
			return true 
		end
	end

	local old_onrightclick = self.OnRightClick
	function self:OnRightClick(down, ...)
		old_onrightclick(self, down, ...)
		local weapon = self.inst.components.combat:GetWeapon()
		if weapon and weapon.components.myth_weaponreticule and down then
			local canshowonthisaction = checkaction()
			weapon.components.myth_weaponreticule:OnRightClick(canshowonthisaction)
		end
	end

	local old_onleftclick = self.OnLeftClick
	function self:OnLeftClick(down, ...)
		old_onleftclick(self, down, ...)
		local weapon = self.inst.components.combat:GetWeapon()
		if weapon and weapon.components.myth_weaponreticule and down then
			weapon.components.myth_weaponreticule:HideReticule()
		end
	end
end)

local function DriverPostInit(c)
	local old_combine = c.CombineWithVehicle
	function c:CombineWithVehicle(...)
		old_combine(self,...)
		if self.vehicle.components.drivable.donothideonmounted then
			self.vehicle:Show()
		end
	end

	local old_save = c.OnSave
	function c:OnSave()
		local data, references = old_save(self)
		references = references or {}
		if self.vehicle and self.vehicle.entity:GetParent() then
			local dat, ref = self.vehicle:GetSaveRecord()
			if ref then
				for k,v in pairs(ref)do
					table.insert(references,v)
				end
			end
			if dat then
				data.follow_entity_data = dat
				data.follow_entity_data.GUID = self.vehicle.GUID
			end
		end
		return data,references
	end

	local old_load = c.OnLoad
	function c:OnLoad(...)
		old_load(self,...)
	end

	local old_loadpostpass = c.LoadPostPass
	function c:LoadPostPass(ents, data)
		if data.follow_entity_data then
			local vehicle = SpawnSaveRecord(data.follow_entity_data,ents)
			vehicle.components.drivable:OnMounted(self.inst)
			self:OnMount(vehicle)
			print("CALL LOAD POSTPASS!")
		end
		old_loadpostpass(self,ents,data)
    end
end

AddComponentPostInit('driver',DriverPostInit)


local function ResPostInit(c)
	local old_can = c.CanResurrect
	function c:CanResurrect(cause,...)
		if cause and ( cause == 'yamaraja_ghost' or cause == 'black_wc' or cause == 'white_wc') then
			STRINGS.UI.DEATHSCREEN.YOUAREDEAD = GLOBAL.MK_MOD_LANGUAGE_SETTING and 'Welcome to the nether world!' or '欢迎光临地府!'
			return false
		end
		return old_can(self,cause,...)
	end
end

AddComponentPostInit('resurrectable', ResPostInit)

local function HealthPostInit(self)
	function self:Myth_DoYamarajaBuff()
		if self.inst.components.yamarajabuff and self.inst.components.yamarajabuff.hasbuff then
			if cause == 'drowning' then  --寻找最近的传送点
				local closest_dist = 0
				for k,v in pairs(Ents) do
					if v.prefab == "beachresurrector" then
						local dist = v:GetDistanceSqToInst(self.inst)
						if not res or dist < closest_dist then
							res = v
							closest_dist = dist
						end
					end
					if not res then
						res = self.inst
					end
					self.inst.Transform:SetPosition(res:GetPosition():Get())
				end
			end

			local fx = SpawnPrefab('mk_halloween_firepuff_cold_'..math.random(3))
			fx.Transform:SetPosition(self.inst:GetPosition():Get())
			self.inst.components.health:SetPercent(1)
			self.inst.components.sanity:SetPercent(1)
			self.inst.components.hunger:SetPercent(1)
			self.inst.components.temperature:SetTemperature(TUNING.STARTING_TEMP)
			if self.inst.components.poisonable:IsPoisoned() then 
				self.inst.components.poisonable:Cure()
			end 
			self.inst.components.yamarajabuff.hasbuff = false

			return true
		end
	end

	local old_setval = self.SetVal
	function self:SetVal(val, cause, ...)
		if val <= 0 and self:Myth_DoYamarajaBuff() then
			return
		end
		old_setval(self, val, cause, ...)
	end
end

AddComponentPostInit("health",HealthPostInit)

local function ActivatePostInit(c)
	local old_DoActivate = c.DoActivate
	function c:DoActivate(doer,...)
		local mod = false
		for _,v in pairs({
			'shipwrecked_entrance',
			'shipwrecked_exit',
			'cave_entrance',
			'cave_exit',
			'porkland_entrance',
			'teleportato_base',
			'teleportato_sw',
			'teleportato_checkmate',
			'teleportato_sw_base',
			'adventure_portal',
			'volcano_exit',
		})
		do
			if self.inst.prefab == v then
				mod = true
				break
			end
		end
		if  doer and doer.components.driver and doer.components.driver.vehicle and (doer.components.driver.vehicle.prefab == 'mk_cloud' or doer.components.driver.vehicle.prefab == 'nz_wheel') then
			if mod then
				doer.components.talker:Say(GLOBAL.MK_MOD_LANGUAGE_SETTING and 'Let me put away somersault cloud first.' or '我得先把筋斗云收起来')
				return
			elseif self.inst.prefab == 'volcano' then
				doer.components.talker:Say(GLOBAL.MK_MOD_LANGUAGE_SETTING and 'Let me put away somersault cloud first.' or '因为筋斗云消失bug一直找不到修复方法\n所以干脆禁止乘坐筋斗云爬火山吧\n嗯,众所周知... 云靠近火山会蒸发...',10)
				return
			end
		end
		return old_DoActivate(self,doer,...)
	end	
end

AddComponentPostInit('activatable', ActivatePostInit)

AddPrefabPostInit('seeds', function (inst) --新作物 葫芦
	local old_pick = inst.components.plantable.product
	if old_pick then
		inst.components.plantable.product = function(inst)
			local old = old_pick(inst)
	
			local total_w = 0
			for k,v in pairs(VEGGIES) do
				total_w = total_w + (v.seed_weight or 1)
			end
	
			if math.random() < 1/total_w then
				return "gourd"
			else
				return old
			end
		end
	end
end)


local function hunter(self)
	local old_spawn = self.SpawnHuntedBeast
	function self:SpawnHuntedBeast()
		local rate = 0.25 --在生成座狼时, 有25%几率变成黑熊
		local old_pf = self.alternate_beast_prefab
		local old_type = type(old_pf)
		if math.random() < rate then
			self.alternate_beast_prefab = (old_type == 'string') and 'blackbear' or {'blackbear'}
		end
		local rc = old_spawn(self)
		self.alternate_beast_prefab = old_pf
		return rc
	end
end
AddComponentPostInit('hunter', hunter)

-----猴子/洞穴猴不会偷猴王东西
-----联机怎么办?我也不知道
local function MKnoSteal(inst)
	if not GetPlayer():HasTag('mythTag_monkeyfriend') then return end
	local old_push = inst.components.locomotor.PushAction
	inst.components.locomotor.PushAction = function (self, action, ...)
		if not (inst.components.follower and inst.components.follower.leader == GetPlayer()) then
			--当猴子不跟随玩家的情况下, 禁用拾取动作
			if action and action.action and 
				(action.action.id == 'PICK' or 
				action.action.id == 'PICKUP' or
				action.action.id == 'HARVEST') then
				return old_push(self, nil, ...)
			end
		end
		return old_push(self, action, ...)
	end
end
AddPrefabPostInit('monkey', MKnoSteal)
AddPrefabPostInit('primeape', MKnoSteal)

----阻止航海组件更新(位置/船损坏判定)
AddComponentPostInit('driver', function(self)
	local old_fn = self.OnUpdate
	function self:OnUpdate(...)
		if self.vehicle and (self.vehicle.prefab == 'mk_cloud' or self.vehicle.prefab == 'nz_wheel') then
			return
		end
		old_fn(self, ...)
	end
end)

----黑熊和熊大的仇恨机制
AddPrefabPostInit('blackbear', function(inst)
	local old_set = inst.components.combat.SetTarget
	inst.components.combat.SetTarget = function(self, target)
		if target and (target.prefab == 'blackbear' or target.prefab == 'bearger') then
			return 
		else
			return old_set(self, target)
		end
	end
end)

AddPrefabPostInit('bearger', function(inst)
	local old_set = inst.components.combat.SetTarget
	inst.components.combat.SetTarget = function(self, target)
		if target and target.prefab == 'blackbear' then
			return 
		else
			return old_set(self, target)
		end
	end
end)


---放桌子
AddPrefabPostInit('forest', function(inst)
	local function ValidatePos(x, z)
		local tx, ty = GetWorld().Map:GetTileCoordsAtPoint(x, 0, z)
		local actual_tile = GetWorld().Map:GetTile(tx, ty)
		return actual_tile ~= GROUND.IMPASSABLE
	end
	
	inst:DoTaskInTime(0.1, function(inst)
		for _,v in pairs(Ents)do
			if v.prefab == 'myth_rhino_desk' and v:IsValid() then
				return
			end
		end
		local desk = SpawnPrefab('myth_rhino_desk')
		local size = inst.Map:GetSize()
		for i = 1, 100 do
			local x = GetRandomMinMax(-size, size)*2
			local z = GetRandomMinMax(-size, size)*2

			if inst.GetIsOnLand and inst:GetIsOnLand(x, 0, z) or ValidatePos(x, z) then
				local canspawn = true
				local ents = TheSim:FindEntities(x, 0, z, 4, nil, {"INLIMBO", 'FX', "NOCLICK",})
				for _, v in ipairs(ents)do
					if v then
						canspawn = false
					end
				end
				if canspawn then
					desk.Transform:SetPosition(x, 0, z)
					print("Spawn desk at pos: ("..x..', '..z..' )')
					return
				end
			end
		end
	end)
end)

local function tributefn(inst)
	inst.components.myth_rhino_tribute = {
		inst = inst,
		CollectUseActions = function(self, doer, target, actions, right)
			if target and target.prefab == 'myth_rhino_desk' then
				table.insert(actions, ACTIONS.RHI_PLACEITEM)
			end
		end,
	}
end
AddPrefabPostInitAny(function(inst)
	if inst.prefab == 'peach' or inst.components.fuel then
		tributefn(inst)
	end
end)

--角色使用时间计时器
local function timer(inst)
	inst:AddComponent('skintimer')
end
AddPrefabPostInit("monkey_king", timer)
AddPrefabPostInit("neza", timer)
AddPrefabPostInit("white_bone", timer)