local function AddPlayerSgPostInit(fn)
    AddStategraphPostInit('wilson', fn)
    AddStategraphPostInit('wilsonboating', fn)
end

local function AddPlayerSg(sg)
    AddStategraphState('wilson', sg)
    AddStategraphState('wilsonboating', sg)
end

AddPlayerSg(State{
    name = 'wb_whip',
    tags = {"attack", "notalking", "abouttoattack", "busy"},
    onenter = function(inst)
        inst.sg.statemem.target = inst.components.combat.target
        inst.components.combat:StartAttack()
        inst.components.locomotor:Stop()
        inst.AnimState:PlayAnimation('whip_pre')
        inst.AnimState:PushAnimation('whip', false)
        inst.AnimState:PushAnimation('whip_lag', false)
        
        --inst.SoundEmitter:PlaySound('???')
        inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_weapon")
        
        if inst.components.combat.target then
            inst.components.combat:BattleCry()
            if inst.components.combat.target and inst.components.combat.target:IsValid() then
                inst:FacePoint(Point(inst.components.combat.target:GetPosition()))
            end
        end
    end,
    timeline = {
        TimeEvent(12*FRAMES, function(inst) 
            local weapon = inst.components.combat:GetWeapon()
            if not (weapon and weapon.prefab == 'wb_whip') then
                inst.sg:GoToState('idle')
                return
            end 
            local x, y, z = inst:GetPosition():Get()
            local ents = TheSim:FindEntities(x, 0, z, 10, {'combat'}, {'FX', 'INLIMBO', 'wb_skeleton'})
            local hit = false
            for _, v in pairs(ents)do
                if v ~= inst and inst.components.combat:CanTarget(v) and inst.components.combat:CanHitTarget(v)
                and not (v.components.follower and v.components.follower.leader == inst) then
                    inst.components.combat:DoAttack(v)
                    hit = true
                end
            end
            if hit then
                weapon.components.finiteuses:Use(1)
            end
        end),
        TimeEvent(20*FRAMES, function(inst)
            inst.sg:GoToState('idle')
        end),
    },
})

AddPlayerSg(State{
    name = 'mk_drink',
    tags = {'busy'},
    onenter = function(inst)
        inst.components.locomotor:Stop()
        --喝酒音效?
        --inst.SoundEmitter:PlaySound("dontstarve/wilson/eat", "eating")   
        inst.components.hunger:Pause()
        inst.AnimState:PlayAnimation("horn")
        inst.AnimState:Show("ARM_normal")
        inst.AnimState:OverrideSymbol("horn01", 'gourd_items', 'swap_wine')
    end,
    timeline={
        TimeEvent(22*FRAMES, function(inst)
            inst.SoundEmitter:PlaySound("dontstarve_DLC002/common/antivenom_whoosh")  
            inst:PerformBufferedAction() 
        end),
        
        TimeEvent(30*FRAMES, function(inst) 
            --inst.sg:RemoveStateTag("busy")
        end),
        
        TimeEvent(70*FRAMES, function(inst) 
            inst.SoundEmitter:KillSound("eating")    
        end),
        
    },     
    events={
        EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),
    },

    onexit = function(inst)
        --inst.SoundEmitter:KillSound("eating")    
        inst.components.hunger:Resume()
        if inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) then
            inst.AnimState:Show("ARM_carry") 
            inst.AnimState:Hide("ARM_normal")
        end
    end,
})

AddPlayerSg(State{
    name = 'wb_knife',
    tags = {"attack", "notalking", "abouttoattack", "busy"},
    onenter = function(inst)
        inst.AnimState:PlayAnimation("atk")  
        inst.SoundEmitter:PlaySound("dontstarve_DLC002/common/swordfish_sword")
        
        if inst.components.combat.target then
            inst.components.combat:BattleCry()
            if inst.components.combat.target and inst.components.combat.target:IsValid() then
                inst:FacePoint(Point(inst.components.combat.target.Transform:GetWorldPosition()))
            end
        end
        inst.sg.statemem.target = inst.components.combat.target
        --inst.components.combat:StartAttack()
        inst.components.locomotor:Stop()
    end,

    onupdate = function(inst)
        if inst.sg.timeinstate and inst.sg.timeinstate > 10/30 then
            inst.sg:RemoveStateTag('busy')
            inst.sg:RemoveStateTag('attack')
        end
    end,
    
    timeline=
    {
        TimeEvent(9*FRAMES, function(inst) 
            inst.components.combat:DoAttack(inst.sg.statemem.target) 
            inst.sg:RemoveStateTag("abouttoattack") 
        end),              
    },
    
    events=
    {
        EventHandler("animover", function(inst) inst.sg:GoToState("idle") end ),
    },
})

AddPlayerSg(State{
    name = 'wb_dofix',
    tags = {"doing", "busy"},

    onenter = function(inst)
        local item = inst:GetBufferedAction() and inst:GetBufferedAction().invobject
        if not (item and item.components.wb_bonehealer) then
            inst.sg:GoToState('idle')
        else
            inst.sg:GoToState('dolongaction', item.components.wb_bonehealer.value/20)
        end
    end,
})
local function BloodHealer(inst)
    if inst.components.healer and GetPlayer() and GetPlayer().prefab == 'white_bone' then
        local old = inst.components.healer.health
        inst.components.healer.health = (old or 0 )*2
    end
end
AddPrefabPostInit('mosquitosack', BloodHealer)
AddPrefabPostInit('mosquitosack_yellow', BloodHealer)

AddPlayerSgPostInit(function(sg)
    --禁用白骨治疗
    local old_handler = sg.actionhandlers[ACTIONS.HEAL].deststate
    sg.actionhandlers[ACTIONS.HEAL].deststate = function(inst, action)
        print(action)
        for k,v in pairs(action)do print(k,v) end
        if inst.prefab == 'white_bone' and (action.target == nil or action.target == inst) then
            local item = action and action.invobject
            if item and (item.prefab == 'mosquitosack' or item.prefab == 'mosquitosack_yellow') then
                return old_handler(inst, action)
            end
            inst.components.talker:Say(MK_MOD_LANGUAGE_SETTING and 'I am dead.' or '不过是一副白骨之躯')
            return nil
        end
        return old_handler(inst, action)
    end
    --莲花/一柱擎天
    local old_handler = sg.actionhandlers[ACTIONS.BUILD].deststate
    sg.actionhandlers[ACTIONS.BUILD].deststate = function(inst, action)
        if not inst.sg:HasStateTag('busy') then
            if action.recipe and action.recipe == 'lotus_flower' and action.doer and action.doer.prefab == 'neza' then
                return 'nz_makelilypad'
            elseif action.recipe and action.recipe == 'mk_jgb_rec' then
                return 'mk_summonjgb'
            else
                return old_handler(inst, action)
            end
        end
    end
    --鞭子攻击sg
    --骨刃快速攻击sg
    local old_handler = sg.events['doattack'].fn
    sg.events['doattack'].fn = function(inst, ...)
        if not inst.components.health:IsDead() and not inst.sg:HasStateTag("attack") then
            local weapon = inst.components.combat and inst.components.combat:GetWeapon()
            if inst.components.rider and inst.components.rider:IsRiding()then
                return old_handler(inst, ...)
            else
                if weapon and weapon.prefab == 'wb_whip' then
                    inst.sg:GoToState('wb_whip')
                    return
                elseif weapon and weapon.prefab == 'wb_knife' then
                    inst.sg:GoToState('wb_knife')
                    return
                else
                    return old_handler(inst, ...)
                end
            end
        end
    end
end)

-----搓花-----
AddPlayerSg(State{
    name = 'nz_makelilypad',
    tags = {"doing", "busy"},
    
    onenter = function(inst, timeout)
        inst.components.locomotor:Stop()
        inst.AnimState:PlayAnimation("emoteXL_bonesaw")
        inst.sg:SetTimeout(25*FRAMES)
    end, 

    timeline=
    {
        TimeEvent(5*FRAMES, function(inst)
            inst.sg:RemoveStateTag('busy')
        end),
        TimeEvent(24*FRAMES, function( inst )
            inst:PerformBufferedAction()
        end),
    },

    events=
    {
        EventHandler("animover", function(inst) inst.sg:GoToState("idle") end ),
    },        
    
})

---咸鱼突刺----------------------
AddStategraphState("wilson", 
    State{ 
        name = "mk_lunge_start",
        tags = { "aoe", "doing", "busy", "nointerrupt", "nomorph" },

        onenter = function(inst)
            inst.components.locomotor:Stop()
            local buffaction = inst:GetBufferedAction()
            if buffaction ~= nil then
                if buffaction.target and buffaction.target:IsValid() then
                    buffaction.pos = buffaction.target:GetPosition()
                end
                if buffaction.pos then
                    inst:ForceFacePoint(buffaction.pos:Get())
                end
            end
            inst.AnimState:PlayAnimation("lunge_pre")
        end,

        timeline =
        {
            TimeEvent(4 * FRAMES, function(inst)
                inst.SoundEmitter:PlaySound("monkey_sound/monkey_sound/twirl") --音效
            end),
            TimeEvent(8 * FRAMES, function(inst)
                inst.sg:GoToState('mk_lunge')
            end),
        },

        events =
        {
            EventHandler("unequip", function(inst)
                inst.sg:GoToState("idle")
            end),
        },
    }
)
AddStategraphState("wilson", 
    State{
        name = "mk_lunge",
        tags = { "aoe", "doing", "busy", "nomorph" },

        onenter = function(inst)
            local buffaction = inst:GetBufferedAction()
            local weapon = inst.components.combat:GetWeapon()
            if not (buffaction and weapon and weapon.DoLunge) then
                inst.sg:GoToState('idle')
            else
                if inst.components.hunger and inst.prefab == 'monkey_king' then
                    if inst:HasTag('skill_boost_mk') then
                        --inst.components.hunger:DoDelta(-5)
                        local hat = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HEAD)
                        if hat and hat.prefab == 'golden_hat_mk' then
                            hat.components.fueled:DoDelta(-24)
                        end
                    else
                        inst.components.hunger:DoDelta(-10)
                    end
                end
                weapon.components.myth_rechargeable:StartRecharging()
                inst.AnimState:PlayAnimation("lunge_pst")
                inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_weapon")
                --inst.SoundEmitter:PlaySound("monkey_sound/monkey_sound/fireball") --这个
                inst.Physics:SetMotorVelOverride(30,0,0)
                inst.components.locomotor:EnableGroundSpeedMultiplier(false)
                inst.components.bloomer:PushBloom("lunge", "shaders/anim.ksh", -2)
                inst.components.colouradder:PushColour("lunge", 1, 1, 0, 0)
                inst.sg.statemem.flash = 1

                local targetpos = buffaction.pos

                local pos = inst:GetPosition()
                if pos.x ~= targetpos.x or pos.z ~= targetpos.z then
                    inst:ForceFacePoint(targetpos:Get())
                end
				local angle = (inst.Transform:GetRotation() + 90) * DEGREES  
				local step = .75
				local offset = 0.25
				local dist = (10 + .5) * step + offset				
                weapon:DoLunge(inst, pos, targetpos)

                inst:PerformBufferedAction()
            end
        end,

        onupdate = function(inst)
            if inst.sg.statemem.flash > 0 then
                inst.sg.statemem.flash = math.max(0, inst.sg.statemem.flash - .1)
                inst.components.colouradder:PushColour("lunge", inst.sg.statemem.flash, inst.sg.statemem.flash, 0, 0)
            end
        end,

        timeline =
        {
            TimeEvent(7* FRAMES, function(inst)
                inst.Physics:ClearMotorVelOverride()
                inst.components.locomotor:EnableGroundSpeedMultiplier(true)
            end),
            TimeEvent(12 * FRAMES, function(inst)
                inst.components.bloomer:PopBloom("lunge")
            end),
        },

        events =
        {
            EventHandler("animover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
                end
            end),
        },

        onexit = function(inst)
            inst.components.bloomer:PopBloom("lunge")
            inst.components.colouradder:PopColour("lunge")
            inst.Physics:ClearMotorVelOverride()
            inst.components.locomotor:EnableGroundSpeedMultiplier(true)
        end,
    }
)
----跳劈
AddStategraphState('wilson',
    State{
        name = "mk_helmsplitter_pre",
        tags = { "helmsplitting", "doing", "busy", "nointerrupt", "nomorph" },

        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("atk_leap_pre")

            if inst.bufferedaction ~= nil and inst.bufferedaction.target ~= nil and inst.bufferedaction.target:IsValid() then
                inst.sg.statemem.target = inst.bufferedaction.target
                inst.components.combat:SetTarget(inst.sg.statemem.target)
                inst:ForceFacePoint(inst.sg.statemem.target.Transform:GetWorldPosition())
            end

            inst.sg:SetTimeout(8 * FRAMES)
            inst.components.health:SetInvincible(true)
        end,

        ontimeout = function(inst)
            inst.sg.statemem.helmsplitting = true
            inst.sg:GoToState("mk_helmsplitter", inst.sg.statemem.target)
        end,

        events =
        {
            EventHandler("animover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg.statemem.helmsplitting = true
                    inst.sg:GoToState("mk_helmsplitter", inst.sg.statemem.target)
                end
            end),
        },

        onexit = function(inst)
            if not inst.sg.statemem.helmsplitting then
                inst.components.combat:SetTarget(nil)
            end
            inst.components.health:SetInvincible(false)
        end,
    }
)
local function DoAreaAtk(inst)
    local angle = -inst.Transform:GetRotation()
    local pos = inst:GetPosition() + Vector3(math.cos(angle*DEGREES),0,math.sin(angle*DEGREES))*2
    local ents = TheSim:FindEntities(pos.x,pos.y,pos.z,3,{"combat"},{"INLIMBO"})
    for _,v in pairs(ents)do
        if v ~= inst and v.components.health and not v.components.health:IsDead() then
            if not v.components.follower or v.components.follower.leader ~= inst then
                v.components.combat:GetAttacked(inst,125,nil,'monkey_king_aoe')
                if v.sg and not v.components.health:IsDead() then
                    v.sg:GoToState('hit')
                end
            end
        end
    end
    inst.components.playercontroller:ShakeCamera(inst, "VERTICAL", 0.2, 0.03, 1, 30)
end

AddStategraphState('wilson',
    State
    {
        name = "mk_helmsplitter",
        tags = { "helmsplitting", "doing", "busy", "nointerrupt", "nomorph" },

        onenter = function(inst, target)
            inst.AnimState:SetDeltaTimeMultiplier(1.1)
            inst.components.health:SetInvincible(true)
            inst.components.locomotor:Stop()
            inst.Transform:SetEightFaced()
            inst.AnimState:PlayAnimation("atk_leap")
            inst.SoundEmitter:PlaySound("dontstarve/common/deathpoof")

            if target ~= nil and target:IsValid() then
                inst.sg.statemem.target = target
                inst:ForceFacePoint(target.Transform:GetWorldPosition())
            end

            inst.sg:SetTimeout(22 * FRAMES)
        end,

        timeline =
        {
            TimeEvent(9 * FRAMES, function(inst)
                inst.components.colouradder:PushColour("helmsplitter", .1, .1, 0, 0)
            end),
            TimeEvent(10 * FRAMES, function(inst)
                inst.components.colouradder:PushColour("helmsplitter", .2, .2, 0, 0)
            end),
            TimeEvent(11 * FRAMES, function(inst)
                inst.components.colouradder:PushColour("helmsplitter", .4, .4, 0, 0)
                inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_weapon")
                inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_whoosh")
            end),
            TimeEvent(12 * FRAMES, function(inst)
                inst.components.bloomer:PushBloom("helmsplitter", "shaders/anim.ksh", -2)
                inst.components.colouradder:PushColour("helmsplitter", 1, 1, 0, 0)
                inst.sg:RemoveStateTag("nointerrupt")
                --ShakeAllCameras(CAMERASHAKE.VERTICAL, .5, .015, .5, inst, 20)
                inst.sg.statemem.weapon = inst.components.combat:GetWeapon()
                --DoHelmSplit(inst)
                DoAreaAtk(inst)
            end),
            TimeEvent(13 * FRAMES, function(inst)
                inst.components.colouradder:PushColour("helmsplitter", .8, .8, 0, 0)
            end),
            TimeEvent(14 * FRAMES, function(inst)
                inst.components.colouradder:PushColour("helmsplitter", .6, .6, 0, 0)
            end),
            TimeEvent(15 * FRAMES, function(inst)
                inst.components.colouradder:PushColour("helmsplitter", .4, .4, 0, 0)
            end),
            TimeEvent(16 * FRAMES, function(inst)
                inst.components.colouradder:PushColour("helmsplitter", .2, .2, 0, 0)
            end),
            TimeEvent(17 * FRAMES, function(inst)
                inst.components.colouradder:PopColour("helmsplitter")
            end),
            TimeEvent(18 * FRAMES, function(inst)
                inst.components.bloomer:PopBloom("helmsplitter")
            end),
        },

        ontimeout = function(inst)
            inst.sg:GoToState("idle", true)
        end,

        events =
        {
            EventHandler("animover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
                end
            end),
        },

        onexit = function(inst)
            inst.AnimState:SetDeltaTimeMultiplier(1)
            inst.Transform:SetFourFaced()
            inst.components.bloomer:PopBloom("helmsplitter")
            inst.components.colouradder:PopColour("helmsplitter")
            inst.components.health:SetInvincible(false)
        end,
    }
)

---jbg
AddPlayerSg(State{
    name = "mk_summonjgb",
    tags ={"idle"},
    onenter = function(inst)
        inst.components.locomotor:Stop()
        inst.AnimState:PlayAnimation("research")
    end,

    timeline = 
        {
            TimeEvent(1*FRAMES, function(inst)inst:PerformBufferedAction() end)
            --TimeEvent( 8*FRAMES, function(inst) inst.SoundEmitter:PlaySound("dontstarve_DLC002/common/antivenom_whoosh") end),
            --TimeEvent(13*FRAMES, function(inst) inst.SoundEmitter:PlaySound("dontstarve_DLC002/common/heelclick") end),
            --TimeEvent(21*FRAMES, function(inst) inst.SoundEmitter:PlaySound("dontstarve_DLC002/common/heelclick") end),
        },

    events=
    {
        EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),
    },
})

----升天
AddStategraphState('wilson',

State{
        name = "nz_superjump_start",
        tags = { "aoe", "doing", "busy", "nointerrupt", "nomorph" },

        onenter = function(inst,pos)
            inst:AddTag('notarget')
            inst.components.hunger:DoDelta(-10)
            inst.components.locomotor:Stop()
            inst.components.health:SetInvincible(true)
            --inst.AnimState:PlayAnimation("superjump_pre")--@--
            inst.AnimState:PlayAnimation('nz36')
            inst.sg.statemem.pos = pos or inst:GetPosition()
            local weapon = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
            if weapon.components.myth_rechargeable then
                weapon.components.myth_rechargeable:StartRecharging()
            end
        end,

        events =
        {
            EventHandler("animover", function(inst)
                --@--inst.sg:GoToState("combat_superjump", {pos = inst.sg.statemem.pos})
                inst.sg:GoToState('nz_superjump',{pos=inst.sg.statemem.pos})
            end)
        },
        ---[[
        onexit = function(inst)
            inst:RemoveTag('notarget')
            inst.components.health:SetInvincible(false)
        end,
        --]]
    }
)

local function MakeShadow(parent)
    local inst = CreateEntity()
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.Transform:SetPosition(parent:GetPosition():Get())

    --parent.components.animstatemanager:SetEntityAnimState(inst)
    inst.AnimState:SetBank('wilson')
    inst.AnimState:SetBuild('neza')
    inst.AnimState:SetMultColour(1,1,1,.8)
    inst.AnimState:PlayAnimation('superjump_land')
    inst:ListenForEvent('animover',inst.Remove)
    --inst:DoTaskInTime(7*FRAMES,inst.Remove)
    return inst
end

AddStategraphState('wilson',

    State{
        name = "nz_superjump",
        tags = { "aoe", "doing", "busy", "noattack", "nomorph" },

        onenter = function(inst, data)
            inst.components.health:SetInvincible(true)
            --if data ~= nil and data.data ~= nil then
            if data then
                inst.sg.statemem.startingpos = data.startingpos
                inst.sg.statemem.isphysicstoggle = data.isphysicstoggle
                --data = data.data
                inst.sg.statemem.weapon = data.weapon
                --if inst.sg.statemem.startingpos ~= nil and
                --    data.targetpos ~= nil and
                --    data.weapon ~= nil and
                --    data.weapon.components.aoeweapon_leap ~= nil and
                --    inst.AnimState:IsCurrentAnimation("superjump") then
                    inst.AnimState:PlayAnimation("superjump_land")
                    inst.AnimState:SetMultColour(.4, .4, .4, .4)
                --    inst.sg.statemem.targetpos = data.targetpos
                    inst.sg.statemem.flash = 0
                    if not inst.sg.statemem.isphysicstoggle then
                        --ToggleOffPhysics(inst)
                    end
                    inst.Physics:Teleport(data.pos.x, 0, data.pos.z)
                    inst.components.health:SetInvincible(true)
                    inst.sg:SetTimeout(22 * FRAMES)
                    return
                --end
            end
            --Failed
            --inst.sg:GoToState("idle", true)
        end,

        onupdate = function(inst)
            if inst.sg.statemem.flash > 0 then
                inst.sg.statemem.flash = math.max(0, inst.sg.statemem.flash - .1)
                local c = math.min(1, inst.sg.statemem.flash)
                inst.components.colouradder:PushColour("superjump", c, c, 0, 0)
            end
        end,

        timeline =
        {
            TimeEvent(FRAMES, function(inst)
                inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_weapon")
                inst.AnimState:SetMultColour(.7, .7, .7, .7)
                inst.components.colouradder:PushColour("superjump", .1, .1, 0, 0)
            end),
            TimeEvent(2 * FRAMES, function(inst)
                inst.AnimState:SetMultColour(.9, .9, .9, .9)
                inst.components.colouradder:PushColour("superjump", .2, .2, 0, 0)
            end),
            TimeEvent(3 * FRAMES, function(inst)
                inst.AnimState:SetMultColour(1, 1, 1, 1)
                inst.components.colouradder:PushColour("superjump", .4, .4, 0, 0)
                inst.DynamicShadow:Enable(true)
            end),
            TimeEvent(4 * FRAMES, function(inst)
                inst.components.colouradder:PushColour("superjump", 1, 1, 0, 0)
                inst.components.bloomer:PushBloom("superjump", "shaders/anim.ksh", -2)
                --ToggleOnPhysics(inst)
                --ShakeAllCameras(CAMERASHAKE.VERTICAL, .7, .015, .8, inst, 20)
                inst.components.playercontroller:ShakeCamera(inst, "VERTICAL", 0.5, 0.03, 2, 30)
                inst.sg.statemem.flash = 1.3
                inst.sg:RemoveStateTag("noattack")
                inst.components.health:SetInvincible(false)
                --if inst.sg.statemem.weapon:IsValid() then
                    --inst.sg.statemem.weapon.components.aoeweapon_leap:DoLeap(inst, inst.sg.statemem.startingpos, inst.sg.statemem.targetpos)
                    --inst.sg.statemem.weapon = nil
                --end
                local pos = inst:GetPosition()
                local ents = TheSim:FindEntities(pos.x,pos.y,pos.z,4,{"combat"},{"INLIMBO"})
                for _,v in pairs(ents)do
                    if v ~= inst and v.components.health and not v.components.health:IsDead() then
                        if not v.components.follower or v.components.follower.leader ~= inst then
                            v.components.combat:GetAttacked(inst,70,nil,'nz_36_aoe')
                            if v.sg then
                                v.sg:GoToState('hit')
                            end
                        end
                    end
                end
            end),

            TimeEvent(8 * FRAMES,function(inst)
                local ent = MakeShadow(inst)
            end),

            TimeEvent(12*FRAMES,function(inst)
                inst.components.playercontroller:ShakeCamera(inst, "VERTICAL", 0.5, 0.03, 2, 30)
                local pos = inst:GetPosition()
                local ents = TheSim:FindEntities(pos.x,pos.y,pos.z,4,{"combat"},{"INLIMBO"})
                for _,v in pairs(ents)do
                    if v ~= inst and v.components.health and not v.components.health:IsDead() then
                        if not v.components.follower or v.components.follower.leader ~= inst then
                            v.components.combat:GetAttacked(inst,70,nil,'nz_36_aoe')
                            if v.sg then
                                v.sg:GoToState('hit')
                            end
                        end
                    end
                end
            end),

            TimeEvent(16* FRAMES,function(inst)
                local ent = MakeShadow(inst)
            end),

            TimeEvent(19*FRAMES,function(inst)
                inst.components.playercontroller:ShakeCamera(inst, "VERTICAL", 0.5, 0.03, 2, 30)
                local pos = inst:GetPosition()
                local ents = TheSim:FindEntities(pos.x,pos.y,pos.z,4,{"combat"},{"INLIMBO"})
                for _,v in pairs(ents)do
                    if v ~= inst and v.components.health and not v.components.health:IsDead() then
                        if not v.components.follower or v.components.follower.leader ~= inst then
                            v.components.combat:GetAttacked(inst,70,nil,'nz_36_aoe')
                            if v.sg then
                                v.sg:GoToState('hit')
                            end
                        end
                    end
                end
            end),

            TimeEvent(8 * FRAMES, function(inst)
                inst.components.bloomer:PopBloom("superjump")
            end),
            TimeEvent(19 * FRAMES, PlayFootstep),
        },

        ontimeout = function(inst)
            inst.sg:GoToState("idle", true)
        end,

        events =
        {
            EventHandler("animover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
                end
            end),
        },

        onexit = function(inst)
            if inst.sg.statemem.isphysicstoggle then
                ToggleOnPhysics(inst)
            end
            inst.AnimState:SetMultColour(1, 1, 1, 1)
            inst.DynamicShadow:Enable(true)
            inst.components.health:SetInvincible(false)
            inst.components.bloomer:PopBloom("superjump")
            inst.components.colouradder:PopColour("superjump")
            if inst.sg.statemem.weapon ~= nil and inst.sg.statemem.weapon:IsValid() then
                inst.sg.statemem.weapon:PushEvent("superjumpcancelled", inst)
            end
        end,
    }
)

--变身!
local function DoTalkSound(inst)
    if inst.talksoundoverride ~= nil then
        inst.SoundEmitter:PlaySound(inst.talksoundoverride, "talk")
        return true
    elseif not inst.prefab == 'wes' then
        inst.SoundEmitter:PlaySound((inst.talker_path_override or "dontstarve/characters/")..(inst.soundsname or inst.prefab).."/talk_LP", "talk")
        return true
    end
end

AddStategraphState('wilson',
    State{
        name = "mkhide",
        tags = {"idle", "hiding"},
        onenter = function(inst)
            inst:AddTag("mk_miao")
            inst.AnimState:OverrideSymbol("swap_hat", "mk_miao", "swap_hat")
            inst.AnimState:Show("HAT")
            inst.AnimState:Show("HAIR_HAT")
            inst.AnimState:Hide("HAIR_NOHAT")
            inst.AnimState:Hide("HAIR")
            inst.AnimState:Hide("HEAD")
            inst.AnimState:Show("HEAD_HAIR")

            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("hide")
            inst.SoundEmitter:PlaySound("dontstarve/movement/foley/hidebush")
            inst:AddTag("notarget")
        end,
        
        onexit = function(inst)
            inst:RemoveTag("notarget")
            
            if inst.sg.statemem.normal then
                return
            end
            inst:RemoveTag("mk_miao")
            inst.AnimState:ClearOverrideSymbol("swap_hat")
            inst.AnimState:Hide("HAT")
            inst.AnimState:Hide("HAIR_HAT")
            inst.AnimState:Show("HAIR_NOHAT")
            inst.AnimState:Show("HAIR")
            inst.AnimState:Show("HEAD")
            inst.AnimState:Hide("HEAD_HAIR")

            local hat = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HEAD)
            if hat then
                inst.components.inventory:Unequip(hat)
                inst.components.inventory:Equip(hat)
            end
        end,
        
        events=
        {
            EventHandler("animover", function(inst) 
                inst.sg.statemem.normal = 1
                inst.sg:GoToState("mkhide_hide")
            end),
        },
    }
)

AddStategraphState('wilson',
     State{
        name = "mkhide_hide",
        tags = {"idle", "hiding"},
        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.AnimState:OverrideSymbol("swap_hat", "mk_miao", "swap_hat")
            inst.AnimState:PlayAnimation("hide_idle", true)
            inst:AddTag("notarget")
            inst:AddTag("mk_miao")
            inst.AnimState:Show("HAT")
            inst.AnimState:Show("HAIR_HAT")
            inst.AnimState:Hide("HAIR_NOHAT")
            inst.AnimState:Hide("HAIR")
            inst.AnimState:Hide("HEAD")
            inst.AnimState:Show("HEAD_HAIR")
        end,
        
        onexit = function(inst)
            inst:RemoveTag("notarget")
            inst:RemoveTag("mk_miao")
            inst.AnimState:ClearOverrideSymbol("swap_hat")
            inst.AnimState:Hide("HAT")
            inst.AnimState:Hide("HAIR_HAT")
            inst.AnimState:Show("HAIR_NOHAT")
            inst.AnimState:Show("HAIR")
            inst.AnimState:Show("HEAD")
            inst.AnimState:Hide("HEAD_HAIR")

            local hat = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HEAD)
            if hat then
                inst.components.inventory:Unequip(hat)
                inst.components.inventory:Equip(hat)
            end
        end, 
    }
)

