
local offsets = { Vector3(0, 0, 0),}
for i = 1,6 do
    table.insert(offsets,Vector3(math.cos(i*6.283/6),0,math.sin(i*6.283/6)))
end

local Cloud = Class(function(self, inst)
    self.inst = inst
    self.clouds = {}
    self.tails = {}

    inst:ListenForEvent('exitlimbo',function()self:OnExitLimbo()end)
    inst:ListenForEvent('enterlimbo',function()self:OnEnterLimbo()end)

    inst:DoTaskInTime(0,function()self:CreateClouds() end)
    self.tailtask = self.inst:DoPeriodicTask(0.1,function()self:CreateTails()end)
    self.highlighttask = self.inst:DoPeriodicTask(0, function() self:SyncHighlight()end)
end)

function Cloud:CreateClouds()
    if self.inst:HasTag('INLIMBO') then
        self:OnEnterLimbo()
        return
    end
    for k,v in pairs(offsets)do
        local fx = SpawnPrefab('mk_cloud_child')
        local s = math.mod(k,2) == 0 and 1 or .8
        fx.AnimState:SetTime(2*math.random())
        fx.AnimState:SetFinalOffset(-3)
        fx.AnimState:SetDeltaTimeMultiplier(GetRandomMinMax(.9,1.1))
        fx.Transform:SetPosition((v*0.7):Get())
        fx.Transform:SetScale(s,s,s)
        fx.entity:SetParent(self.inst.entity)
        fx:ScaleFn(nil,nil,1.1)

        fx.cloud = self.inst
        self.clouds[fx] = true
    end
end

function Cloud:RemoveClouds()
    for k in pairs(self.clouds)do
        if k:IsValid() then 
            k.entity:SetParent(nil)
            k:Remove() 
        end
    end
end

function Cloud:OnRemoveEntity()
    for k,v in pairs(self.clouds)do
        if k:IsValid() then
            k.entity:SetParent(nil)
            k.AnimState:PlayAnimation('anim_fade')
            k:ListenForEvent("animover",k.Remove)
        end
    end
end

local function GetRandomPosAfterPlayer() --单机版随便乱写
    local pos = GetPlayer():GetPosition()
    local angle = (GetPlayer().Transform:GetRotation()-180)*DEGREES
    local r = .4
    return math.cos(angle)*r+pos.x, 0, -math.sin(angle)*r+pos.z
end

function Cloud:CreateTails()
    if self.inst:HasTag("INLIMBO") then
        self:OnEnterLimbo()
        return
    end
    if self.inst.components.drivable.driver ~= GetPlayer()  then
        return
    end
    if GetPlayer().sg.currentstate.name == 'jumponboatstart' then
        return
    end
    if GetPlayer().Physics:GetMotorVel() < 1 then
        return
    end
    local speed = GetPlayer().Physics:GetMotorVel()
    local fx = SpawnPrefab('mk_cloud_tail')
    fx.Transform:SetPosition(GetRandomPosAfterPlayer())
    fx.Transform:SetRotation(self.inst.Transform:GetRotation())
    fx.Physics:SetMotorVel(speed *GetRandomMinMax(.6,.6), 0, 0)
    self.tails[fx] = true
    
    for k in pairs(self.tails)do
        if not k:IsValid() then
            self.tails[k] = nil
        else
            if GetPlayer().Physics:GetMotorVel() > k.Physics:GetMotorVel() and self.inst:IsValid() then
                k:ForceFacePoint(self.inst:GetPosition():Get())
            end
        end
    end
end  

function Cloud:SplashCloudFx()
    for i = 1,10 do
        local fx = SpawnPrefab('mk_cloud_tail')
        local s = GetRandomMinMax(.6,1)
        fx.AnimState:PlayAnimation('anim_fade')
        fx.AnimState:SetDeltaTimeMultiplier(3)
        fx:ListenForEvent('animover',fx.Remove)
        fx:RemoveComponent('colourtweener')

        fx.Physics:SetMotorVel(2,GetRandomMinMax(1,3),0)
        fx.Physics:SetMass(0)
        fx.Transform:SetRotation(math.random(360))
        fx.Transform:SetScale(s,s,s)
        fx.Transform:SetPosition(self.inst:GetPosition():Get())
    end
end  

function Cloud:OnMounted(doer)
    self:SplashCloudFx()
end

function Cloud:OnDismounted()

end

function Cloud:OnEnterLimbo()
    if self.tailtask then
        self.tailtask:Cancel()
        self.tailtask = nil
    end
    self:RemoveClouds()
end

function Cloud:OnExitLimbo()
    if not self.tailtask then
        self.tailtask = self.inst:DoPeriodicTask(0.1,function()self:CreateTails() end)
    end
    self:CreateClouds()
end

function Cloud:StartTween(...)
    for k,v in pairs(self.clouds)do
        if not k.components.colourtweener then
            k:AddComponent('colourtweener')
        end
        k.components.colourtweener:StartTween(...)
    end
end

function Cloud:SyncHighlight()
    if self.inst:IsValid() then
        local l = {self.inst.AnimState:GetAddColour()}
        for k in pairs(self.clouds)do
            if k:IsValid() then k.AnimState:SetAddColour(unpack(l)) end
        end
    end
end

return Cloud