local function GetMusic()
    return GetPlayer().components.dynamicmusic
end

local function GetPath(name)
    if name == 'yamaraja' then
        return 'mythwords_bgm/combat/yamaraja2', .5
    end
    if name == 'cloud' then
        return 'mythwords_bgm/cloud/cloud', 1
    end
    if name == 'rhino' then
        return 'mythwords_bgm/combat/rhino', 1
    end
end

local DynamicMusic = Class(function(self, inst)
    self.inst = inst
    
    self.enabled = true
    
    self.playing_danger = false

    self.last_dismountcloud_time = -1000

    self.inst:ListenForEvent( "attacked", function(inst, dat)
        if self.enabled and dat and dat.attacker then
            if dat.attacker:HasTag('yamaraja') then
                self:OnStartDanger('yamaraja')
            elseif dat.attacker:HasTag('mythTag_rhino') then
                self:OnStartDanger('rhino')
            end
        else
            self:OnExtendDanger()
        end
    end )  
    self.inst:ListenForEvent( "doattack", function(inst, dat)
		if self.enabled and dat and dat.target then
            if dat.target:HasTag('yamaraja') then
			     self:OnStartDanger('yamaraja')
             elseif dat.target:HasTag('mythTag_rhino')then
                self:OnStartDanger('rhino')
            end
        else
            self:OnExtendDanger('yamaraja')
		end
	end )  
    inst:ListenForEvent( "resurrect", function(inst)
        self:StopPlayingDanger()
    end)

    inst:ListenForEvent("mountboat", function(it, data)
        if data and data.boat and data.boat.prefab == "mk_cloud" then
            self:OnMountCloud()
        end
    end)
    inst:ListenForEvent("dismountboat", function(it, data)
        self:StopPlayingBoating()
    end)

    self.SoundEmitter = CreateEntity().entity:AddSoundEmitter()
end)

function DynamicMusic:Enable()
    self.enabled = true
end

function DynamicMusic:Disable()
    self.enabled = false
    self:StopPlayingDanger()
end

function DynamicMusic:OnStartDanger(name)
    if not self.enabled then return end
    self.inst:StartUpdatingComponent(self)
    self.danger_timeout = 10
    if self.danger_name ~= name then --切换bgm
        self.SoundEmitter:KillSound('mythBGM_combat')
        self.danger_name = name
    end
    if not self.playing_danger and not self.SoundEmitter:PlayingSound("erupt") then
        local soundpath, vol = GetPath(name)
        if not soundpath then
            return
        end
        self.SoundEmitter:PlaySound(soundpath, "mythBGM_combat", vol or 1)
        self:SuppressDynamicMusic()     
        self.playing_danger = true
    end
end

function DynamicMusic:OnExtendDanger()
    if self.playing_danger then
        self.danger_timeout = 10
    end
end

function DynamicMusic:StopPlayingDanger()
    self.SoundEmitter:KillSound("mythBGM_combat")
    self.playing_danger = false
    --self.inst:StopUpdatingComponent(self)
end

function DynamicMusic:SuppressDynamicMusic()

    local cmp = GetMusic()
    if cmp then
        for _,v in pairs({'StopPlayingBusy','StopPlayingTone','StopPlayingSurfing','StopPlayingBoating'})do
            if cmp[v] then
                local fn = cmp[v]
                fn(cmp)
            end
        end
    end
end

function DynamicMusic:OnMountCloud()
    if not self.enabled then return end

    local soundpath, vol = GetPath('cloud')
    if not self.SoundEmitter:PlayingSound('mythBGM_cloud') 
        or GetTime() - self.last_dismountcloud_time >= 120 
        then
        self.SoundEmitter:KillSound('mythBGM_cloud')
        if MK_MOD_CLOUD_BGM then
            self.SoundEmitter:PlaySound(soundpath, 'mythBGM_cloud', vol or 1)
        end
    end
    --风声 
    if not self.SoundEmitter:PlayingSound('mythBGM_windsound') then
        self.SoundEmitter:PlaySound("dontstarve_DLC002/rain/islandwindAMB", "mythBGM_windsound")
    end

    self.on_cloud = true
    self.inst:StartUpdatingComponent(self)
end

function DynamicMusic:StopPlayingBoating()
    self.on_cloud = false
    self.last_dismountcloud_time = GetTime()
    self.SoundEmitter:SetParameter('mythBGM_cloud', 'intensity', 0)
    self.SoundEmitter:KillSound('mythBGM_windsound')
end

local function _UpdateDanger(self, dt)
    if self.danger_timeout and self.danger_timeout > 0 then
        self:SuppressDynamicMusic()
        GetMusic():StopPlayingDanger()
        self.danger_timeout = self.danger_timeout - dt
        if self.danger_timeout <= 0 then
            self:StopPlayingDanger()
            return true
        end
    else
        return true
    end
end

local function _UpdateCloud(self, dt)
    if self.on_cloud then
        self:SuppressDynamicMusic()
        local speed = self.inst.Physics:GetMotorVel()
        local intensity = Remap(speed, 0, 10, 0.5, 1)
        if self.danger_timeout and self.danger_timeout > 0 or GetMusic().playing_danger then
            intensity = 0
        end
        self.SoundEmitter:SetParameter('mythBGM_cloud', 'intensity', intensity)

        local windvolume = Remap(speed, 0, 10, 0.2, 0.8)
        self.SoundEmitter:SetParameter('mythBGM_windsound', 'intensity', windvolume)
    else
        return true
    end
end

function DynamicMusic:OnUpdate(dt)
    --print('up')
    local stop = _UpdateDanger(self, dt) and _UpdateCloud(self, dt)
    if stop then
        self.inst:StopUpdatingComponent(self)
    end
end

return DynamicMusic

