
local BuffManager = Class(function(self, inst)
    self.inst = inst
    self.buffs = {}
    inst:ListenForEvent('death',function()self:RemoveAllBuffs()end)
end)

local buff_data = 
{
    fly_pill = 
    {
        time=480,
    },

    bloodthirsty_pill = 
    {
        time=480,
        on = function(inst) return end,
        off = function(inst) return end,
    },

    condensed_pill = 
    {
        time=480,
        on = function(inst) inst.components.combat:AddDamageModifier('mk_condensed_pill',1) end,
        off = function(inst) inst.components.combat:AddDamageModifier('mk_condensed_pill',0) end,
    },

    detoxic_pill = 
    {
        time=240,
        on = function(inst) inst.components.poisonable:Cure(nil, true, 240) end,
        off = function() end,
    },

    armor_pill = 
    {
        time=480,
        on = function(inst) inst.components.health.mk_absorb = 0.8 end,
        off = function(inst) inst.components.health.mk_absorb = 0 end,
    },
}

function BuffManager:AddBuff(name, time)
    if not buff_data[name] then
        print('Invalid buff prefab ['..name..']')
        return
    end
    if not self.buffs[name] then
        self.buffs[name] = {}
    end
    local thisbuff = self.buffs[name]
    if thisbuff.timer then
        thisbuff.timer:Cancel()
    end
    time = time or buff_data[name].time
    thisbuff.timer = self.inst:DoTaskInTime(time, function()self:RemoveBuff(name)end)
    thisbuff.desttime = GetTime() + time
    local onfn = buff_data[name].on 
    if onfn then onfn(self.inst) end
end

function BuffManager:HasBuff(name)
    return self.buffs[name] and self.buffs[name].timer and self.buffs[name].desttime
end

function BuffManager:RemoveBuff(name)
    if not buff_data[name] then
        print('Invalid buff prefab ['..name..']')
        return
    end
    self.buffs[name].timer = nil
    self.buffs[name].desttime = nil
    local offfn = buff_data[name].off 
    if offfn then offfn(self.inst) end
end

function BuffManager:RemoveAllBuffs()
    for k,v in pairs(self.buffs)do
        if v.timer then
            self:RemoveBuff(k)
        end
    end
end

function BuffManager:GetAllBuffInfos() ---ui调用
    local infos = {}
    for k, v in pairs(self.buffs)do
        if v.timer then
            table.insert(infos, {name = k, time = v.desttime-GetTime()})
        end
    end
    table.sort(infos, function(x, y) return x.time > y.time end)
    return infos
end

function BuffManager:OnSave()
    local buffs = {}
    for name,v in pairs(self.buffs)do
        if v.timer then
            buffs[name] = v.desttime - GetTime()
        end
    end

    return buffs
end

function BuffManager:OnLoad(data)
    if data ~= nil  then
        for k, v in pairs(data) do
            self:AddBuff(k,v)
        end
    end
end
--[[
function BuffManager:GetDebugString()
	local str = "Num Buffs: " .. tostring(GetTableSize(self.debuffs))
	
    for k, v in pairs(self.debuffs) do
		str = str .. "\n  " .. tostring(v.inst.prefab)
	end
		
	return str
end
--]]
return BuffManager
