
local Rechargeable = Class(function(self, inst)
    self.inst = inst
    self.recharge = -2
    self.rechargetime = 255
	self.recharging = false

	self.onstartrecharging = function(inst) 
		if inst.components.myth_weaponreticule then
			inst.components.myth_weaponreticule:SetCanSpell(false)
		end
	end

	self.onstoprecharging = function(inst)
		if inst.components.myth_weaponreticule then
			inst.components.myth_weaponreticule:SetCanSpell(true)
		end
	end
end)

function Rechargeable:SetRechargeTime(time)
	self.rechargetime = time  --totaltime
	self.recharge = 0
end

function Rechargeable:StartRecharging()
	self.recharging = true
	
	self.inst:PushEvent("rechargechange", {percent = self:GetPercent()} )
	self.inst:StartUpdatingComponent(self)

	if self.onstartrecharging then
		self.onstartrecharging(self.inst)
	end
end

function Rechargeable:StopRecharging()
	self.inst:StopUpdatingComponent(self) -- to stop
	
	self.recharging = false
	self.recharge  = 0
	
	self.inst:PushEvent("rechargechange", {percent = self:GetPercent()} )

	if self.onstoprecharging then
		self.onstoprecharging(self.inst)
	end
end

function Rechargeable:OnUpdate(dt)

	self.recharge = self.recharge + dt
	self.inst:PushEvent("rechargechange", {percent = self:GetPercent()} )
	
	if self.recharge >= self.rechargetime then
	
		self:StopRecharging()
	end
end

function Rechargeable:GetPercent()
	if self.recharging == true  then
		return math.min(1, self.recharge / self.rechargetime)
	else
		return 1
	end
end

function Rechargeable:GetDebugString()

	if self.recharging == true then
		return	string.format("percent: %2.2f ", self:GetPercent())
	else
		return "recharged"
	end
end

function Rechargeable:GetRechargeTime()
	return self.recharging == true  and  self.rechargetime or  0
end

function Rechargeable:OnSave()
    return {
        time = self.recharge,
    }
end

function Rechargeable:OnLoad(data)
    if data ~= nil and data.time ~= nil and data.time ~= 0 then
        self.recharge = data.time
        self:StartRecharging()
    end
end

return Rechargeable