local cd = 120
local dur = 120
local radius = 32

local Fog = Class(function(self, inst)
	self.inst = inst
	self.left = 0
	
	self.last_use_time = -cd

end)

function Fog:Start()
	self.inst:AddTag('wb_fog')
	self.inst.components.locomotor:AddSpeedModifier_Additive('wb_fog', 1.5)
	self.inst:StartUpdatingComponent(self)
	self.last_use_time = GetTime()
	self.left = dur

	if self.inst.HUD then
		self.inst.HUD.white_bonefogover:StartFog()
	end
end

function Fog:Stop()
	self.inst:RemoveTag('wb_fog')
	self.inst.components.locomotor:RemoveSpeedModifier_Additive('wb_fog')
	self.inst:StopUpdatingComponent(self)

	if self.inst.HUD then
		self.inst.HUD.white_bonefogover:StopFog()
	end
end

function Fog:CanUse()
	return not self.inst:HasTag('wb_fog') and GetTime() - self.last_use_time > cd
end

function Fog:OnUpdate(dt)
	if self.left > 0 then
		self.left = self.left - dt
		self:DoFogEffect(dt)
		if self.left <= 0 then self:Stop() end
	end
end

function Fog:OnSave()
	return {cd = cd-(GetTime()-self.last_use_time)}
end

function Fog:OnLoad(data)
	if data and data.cd then
		self.last_use_time = data.cd - cd
	end
end

function Fog:CollectSceneActions(doer, actions, right)
	local inst = self.inst
    if right and self:CanUse() and
        not (inst.components.rider and inst.components.rider:IsRiding()) 
        --not inst.components.health:IsDead() then
        and not inst.sg:HasStateTag('wb_2ghost') and not inst._isghost then
        table.insert(actions, ACTIONS.WB_FOG)
    end
end

function Fog:DoFogEffect(dt)
	local x,y,z = self.inst:GetPosition():Get()
	local ents = TheSim:FindEntities(x,y,z, 32, { "combat" }, {"INLIMBO", 'player'})
	for _, v in ipairs(ents) do	
		if v.components.combat and v.components.combat.target ~= nil
			and v.components.combat.target == self.inst then

			v.hitwhite_bone_time = (v.hitwhite_bone_time or 0) + dt
			if v.hitwhite_bone_time >= math.random(6,10) then
				v.hitwhite_bone_time = nil
				v.components.combat:GiveUp()
			end
		end
	end
end


return Fog


