local WbSpawner = Class(function(self, inst)
	self.inst = inst

	self.target = nil
	self.cooldown = 70* 480 --一年时间
	self.canspawn = false
	self.defeated = false

	self.inst:DoTaskInTime(1, function()self:Pick() end)

	self.pigs = {}

	self.wb_shadow = nil

	self.inst:DoPeriodicTask(5, function() self:CoolDown(5) end)

	self.inst:DoPeriodicTask(1, function() self:DoBlood() end)

	--self.inst:ListenForEvent('mkEvent_fireeye', function(_, data) self:OnFireEye(data) end, GetPlayer())
end)

local function _isenabled()
	local sm = GetSeasonManager()
	if sm and sm.IsTemperateSeason and sm:IsTemperateSeason() then
		return true
	end
end

local function _onpigdeath(inst)
	MK_MOD_UTIL.MakeFxAt(inst:GetPosition(), 'statue_ruins_fx', 'white_bone_changefx', 'transform_nightmare')
	
end

function WbSpawner:CoolDown(time)
	if self.cooldown > 0 then
		self.cooldown = self.cooldown - time
		if self.cooldown <= 0 then
			self.canspawn = true
			self:Pick()
		end
	end
end

function WbSpawner:OnSave()
	local targetGUID = self.target and self.target:IsValid() and self.target.GUID
	local data = {
		cooldown = self.cooldown,
		canspawn = self.canspawn,
		defeated = self.defeated,
		target = targetGUID,
	}
	local ref = { targetGUID }
	return data, ref
end

function WbSpawner:OnLoad(data)
	if data then
		self.cooldown = data.cooldown
		self.defeated = data.defeated
		self.canspawn = data.canspawn
	end
end

function WbSpawner:LoadPostPass(newents, data)
	if data.target then
		local target = newents[data.target]
		if target then
			self:Track(target.entity)
		end
	end
end

function WbSpawner:AddPig(pig)
	if type(pig) == 'table' and pig.IsValid and pig:IsValid() then
		self.pigs[pig] = true
	end
end

function WbSpawner:Pick() --随机挑选一位幸运猪人
	local total = 0
	local picks = {}
	for k in pairs(self.pigs)do
		if not k:IsValid() then
			self.pigs[k] = nil
		else
			total = total + 1
			table.insert(picks, k)
		end
	end
	if total == 0 then
		self.inst:DoTaskInTime(10, function()self:Pick()end)
	elseif total == 1 then
		self:Track(picks[1])
	else
		self:Track(picks[math.random(total)])
	end
end

function WbSpawner:Track(target)
	if not self.canspawn then return end
	if not _isenabled() then return end
	if target and target:IsValid() then
		if self.target then
			self.target:RemoveTag('wb_pig')
		end
		self.target = target
		target:AddTag('wb_pig')
		if not (self.wb_shadow and self.wb_shadow:IsValid()) then
			self.wb_shadow = SpawnPrefab('wb_shadow')
			--
		end
	else
		self.target = nil
	end
end

function WbSpawner:OnFireEye(val)
	--处理火眼金睛视觉效果
	if not (self.target and self.target:IsValid()) then
		self.target = nil
	else
		if val then
			self.inst:StartUpdatingComponent(self)
		else
			self.inst:StopUpdatingComponent(self)
			if self.target and self.target:IsValid() then
				self.target.AnimState:SetMultColour(1,1,1,1)
			end
			if self.wb_shadow and self.wb_shadow:IsValid() then
				self.wb_shadow.AnimState:SetMultColour(1,1,1,0)
			end
		end
	end
end

function WbSpawner:DoBlood() --将白骨精附近生物的血量转移到白骨精身上
	if self.target and self.target:IsValid() and not self.target.components.health:IsDead() then
		if self.canspawn and _isenabled() then
			local x,y,z = self.target:GetPosition():Get()
			for _,v in pairs(TheSim:FindEntities(x,y,z, 32, nil, {'FX','INLIMBO', 'player'}))do
				if v ~= self.target and v.components.health and not v.components.health:IsDead() then
					v.components.health:DoDelta(-1, true, 'wb_boss')
					self.target.components.health:DoDelta(1, true, 'wb_boss')
				end
			end
		end
	else
		self.target = nil
	end
end

local alpha_base = 0.5
local alpha_speed = 0.5
function WbSpawner:OnUpdate(dt)
	local alpha_plus = 0.3* math.sin(GetTime()*alpha_speed)
	if self.target and self.target:IsValid() then
		self.target.AnimState:SetMultColour(1,1,1,alpha_base+alpha_plus)
	end
	if self.wb_shadow and self.wb_shadow:IsValid() then
		self.wb_shadow.AnimState:SetMultColour(1,1,1,alpha_base-alpha_plus)
	end
end


return  WbSpawner


