local assets =
{
	Asset("ANIM", "anim/bananafan.zip"),
}

if MK_MOD_LANGUAGE_SETTING then
    STRINGS.NAMES.BANANAFAN = "palm-leaf fan"
    STRINGS.CHARACTERS.GENERIC.DESCRIBE.BANANAFAN = "The precious fan"
else
    STRINGS.NAMES.BANANAFAN = "芭蕉扇"
    STRINGS.CHARACTERS.GENERIC.DESCRIBE.BANANAFAN = "这宝扇非同一般"
    STRINGS.CHARACTERS.MONKEY_KING.DESCRIBE.BANANAFAN = '好借好还~'
end

local function OnUse(inst, target)
	local coolingAmount = TUNING.FAN_COOLING
	if target.components.temperature then
		if target.components.temperature:GetCurrent() + coolingAmount <= 0 then
			coolingAmount = -target.components.temperature:GetCurrent() + 5
		end
		target.components.temperature:SetTemperature(target.components.temperature:GetCurrent() + coolingAmount)
		--target.components.temperature:DoDelta(coolingAmount)
	end
	local pos = target:GetPosition()
	local ents = TheSim:FindEntities(pos.x, pos.y, pos.z, 30, nil, {"FX", "NOCLICK","DECOR","INLIMBO"}, {"smolder", "fire", "alchmy_fur"})
	for i,v in pairs(ents) do
		if v.components.burnable then 
			-- Extinguish smoldering/fire and reset the propagator to a heat of .2
			v.components.burnable:Extinguish(true, 0) 
		end
		if v.prefab == 'alchmy_fur' then
			v.components.stewer_fur:LongUpdate(98765)
		end
	end

	local ground = GetWorld()
	if ground.components.seasonmanager then
        ground.components.seasonmanager:ForcePrecip()
    end
end

local function CanUse(inst, target)
	return target.components.temperature and target.components.temperature.current > 0
end

local function fn()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
	MakeInventoryPhysics(inst)
	MakeInventoryFloatable(inst, "ilde", "idle")
	MakeBlowInHurricane(inst, TUNING.WINDBLOWN_SCALE_MIN.LIGHT, TUNING.WINDBLOWN_SCALE_MAX.LIGHT)

	inst.AnimState:SetBank('bananafan')
	inst.AnimState:SetBuild('bananafan')

	inst.animinfo = 'bananafan'

	inst.AnimState:PlayAnimation("idle")

	inst:AddTag("fan")

    inst:AddComponent("inspectable")
    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.atlasname = 'images/monkey_king_item.xml'

    inst:AddComponent("fan")
    inst.components.fan:SetOnUseFn(OnUse)

    inst:AddComponent("finiteuses")
    inst.components.finiteuses:SetMaxUses(3)
    inst.components.finiteuses:SetUses(3)
    inst.components.finiteuses:SetOnFinished(inst.Remove)
    inst.components.finiteuses:SetConsumption(ACTIONS.FAN, 1)

	return inst
end

return Prefab("bananafan", fn, assets)