require "brains/blackbearbrain"
require "stategraphs/SGblackbear"

assets =
{
    Asset("ANIM", "anim/blackbear.zip")
}

local BLACKWIND_CD = function(inst) 
   return inst.components.health:GetPercent() < .2 and 7 or 12
end

local POUND_CD = function(inst)
    return 25
end

local function shouldPound(inst)
    return GetTime() - inst.last_pound_time > POUND_CD(inst) and not inst:shouldBW()
end

local function shouldBW(inst)
    return inst.components.health:GetPercent() < .5 and
        GetTime() - inst.last_bw_time > BLACKWIND_CD(inst)
end

local function onPound(inst)
    inst.last_pound_time = GetTime()
end

local function onBW(inst)
    inst.last_bw_time = GetTime()
end

SetSharedLootTable( 'blackbear',
{
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},
    {'meat',             1.00},

    --{'fly_pill',         1.75},
    --{'bloodthirsty_pill',1.75},
    --{'condensed_pill',   1.75},
    --{'armor_pill',       1.75},
    {'gourd_pill_blackbear', 1},

    {'cassock',          1.00},
    {'kam_lan_cassock_bp',1.00}, --此处应为斑斓袈裟蓝图
    {'kam_lan_cassock',  0.05},

})

local BASE_TAGS = {"structure"}
local SEE_STRUCTURE_DIST = 20

local TARGET_DIST = 7.5

local function LeaveWorld(inst)
    inst:Remove()
end

local function NearPlayerBase(inst)
    local pt = inst:GetPosition()
    local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, SEE_STRUCTURE_DIST, BASE_TAGS)
    if #ents >= 2 then
        inst.SeenBase = true
        return true
    end
end

local function CalcSanityAura(inst, observer) --噩梦光环
    if inst.components.combat.target then
        return -400/60
    end

    return -100/60
end

local function RetargetFn(inst)
    --if inst.components.sleeper and inst.components.sleeper:IsAsleep() then return end
    if inst.last_eat_time and (GetTime() - inst.last_eat_time) > TUNING.BEARGER_DISGRUNTLE_TIME then
        return FindEntity(inst, TARGET_DIST*5, function(guy) 
            return inst.components.combat:CanTarget(guy)
               and not guy:HasTag("prey")
               and not guy:HasTag("smallcreature")
        end)
    else
        return FindEntity(inst, TARGET_DIST, function(guy)
            return inst.components.combat:CanTarget(guy)
                and not guy:HasTag("prey")
                and not guy:HasTag("smallcreature")
        end)
    end
end

local function KeepTargetFn(inst, target)
    return inst.components.combat:CanTarget(target)
end

local function OnSave(inst, data)
    data.SeenBase = inst.SeenBase
    data.CanGroundPound = inst.CanGroundPound
    data.num_food_cherrypicked = inst.num_food_cherrypicked
    data.num_good_food_eaten = inst.num_good_food_eaten
    data.KilledPlayer = inst.KilledPlayer
    data.shouldGoAway = inst.shouldGoAway
end

local function OnLoad(inst, data)
    if data then
        inst.SeenBase = data.SeenBase
        inst.CanGroundPound = data.CanGroundPound
        inst.num_food_cherrypicked = data.num_food_cherrypicked or 0
        inst.num_good_food_eaten = data.num_good_food_eaten or 0
        inst.KilledPlayer = data.KilledPlayer or false
        inst.shouldGoAway = data.shouldGoAway or false
    end
end

local function OnAttacked(inst, data)
    inst.components.combat:SetTarget(data.attacker)
end

local function OnCollide(inst, other)
    if not inst.sg:HasStateTag("running") and not other:HasTag("tree") then return end
    
    local v1 = Vector3(inst.Physics:GetVelocity())
    if v1:LengthSq() < 1 then return end

    inst:DoTaskInTime(2*FRAMES, function()
        if other and other.components.workable and other.components.workable.workleft > 0 then
            SpawnPrefab("collapse_small").Transform:SetPosition(other:GetPosition():Get())
            other.components.lootdropper:SetLoot({})
            other.components.workable:Destroy(inst)
        end
    end)
end

local function WorkEntities(inst)
    local pt = inst:GetPosition()
    local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, 5)
    local heading_angle = -(inst.Transform:GetRotation())
    local dir = Vector3(math.cos(heading_angle*DEGREES),0, math.sin(heading_angle*DEGREES))

    for k,v in pairs(ents) do
        if v and v.components.workable and not v:HasTag("insect") then
            local hp = v:GetPosition()
            local offset = (hp - pt):GetNormalized()     
            local dot = offset:Dot(dir)
            if dot > .3 then
                v.components.workable:Destroy(inst)
            end
        end
    end
end

local function LaunchItem(inst, target, item)
    if item.Physics then

        local x, y, z = item:GetPosition():Get()
        y = .1
        item.Physics:Teleport(x,y,z)

        local hp = target:GetPosition()
        local pt = inst:GetPosition()
        local vel = (hp - pt):GetNormalized()     
        local speed = 5 + (math.random() * 2)
        local angle = math.atan2(vel.z, vel.x) + (math.random() * 20 - 10) * DEGREES
        item.Physics:SetVel(math.cos(angle) * speed, 10, math.sin(angle) * speed)

    end
end

local function OnHitOther(inst, data)  --普通攻击时使物体掉落
    local other = data.target
    if other and other.components.inventory then
        local item = other.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
        
        if not item or item:HasTag("cantdrop") or item:HasTag("mk_jgb") then return end  

        other.components.inventory:DropItem(item)
        LaunchItem(inst, data.target, item)
    end
end

local function spawnbarrier(inst,pt) --升起石墙
    local angle = 0
    local radius = 13
    local number = 32
    for i=1,number do        
        local offset = Vector3(radius * math.cos( angle ), 0, -radius * math.sin( angle ))
        local newpt = pt + offset
        local tile = GetWorld().Map:GetTileAtPoint(newpt.x, newpt.y, newpt.z)

        if tile ~= GROUND.IMPASSABLE and tile ~= GROUND.INVALID and not GetWorld().Map:IsWater(tile) then
            GetWorld():DoTaskInTime(math.random()*0.3, function()            
                local rock = SpawnPrefab(PrefabExists("rock_basalt") and "rock_basalt" or "rock"..math.random(1,2))
                rock.AnimState:PlayAnimation("emerge")
                inst.SoundEmitter:PlaySound("dontstarve_DLC003/creatures/boss/hulk_metal_robot/rock")
                rock.AnimState:PushAnimation("full")
                if rock.perfab ~= 'rock_basalt' then
                    rock.AnimState:PlayAnimation('full')
                end

                rock.Transform:SetPosition(newpt.x,newpt.y,newpt.z)

            end)
        end
        angle = angle + (PI*2/number)
    end
end

local function doPound(inst) --环形石墙+摧毁
    inst.components.groundpounder.damageRings = 4
    inst.components.groundpounder.destructionRings = 4
    inst.components.groundpounder.numRings = 4                
    GetPlayer().components.playercontroller:ShakeCamera(inst, "FULL", 0.7, 0.02, 2, 40)          
    inst.components.groundpounder:GroundPound()
    local pt = Vector3(inst.Transform:GetWorldPosition())
    GetWorld():DoTaskInTime(0.6,function() inst:spawnbarrier(pt) end)
end

local function SetStandState(inst, state)
    --"quad" or "bi" state
    inst.StandState = string.lower(state)
end

local function IsStandState(inst, state)
    return inst.StandState == string.lower(state)
end

local function OnKill(inst, data)
    if data and data.victim == GetPlayer() then
        inst.KilledPlayer = true
    end
end

local function OnPlayerAction(inst, player, data)
    local playerAction = data.action
    local selfAction = inst:GetBufferedAction()
    if not playerAction or not selfAction then return end --You're not doing anything so whatever.

    if playerAction.target == selfAction.target then -- We got a problem bud.
        
        inst.num_food_cherrypicked = inst.num_food_cherrypicked + 1
        if inst.num_food_cherrypicked < TUNING.BEARGER_STOLEN_TARGETS_FOR_AGRO then
            inst.sg:GoToState("targetstolen")
        else
            inst.num_food_cherrypicked = TUNING.BEARGER_STOLEN_TARGETS_FOR_AGRO - 1
            inst.components.combat:SuggestTarget(player)
        end
    end
end



local function fn(Sim)
    local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	local sound = inst.entity:AddSoundEmitter()
	local shadow = inst.entity:AddDynamicShadow()
	shadow:SetSize(6, 3.5)
    
    inst.Transform:SetFourFaced()
    
    local s = 1
    trans:SetScale(s,s,s)

	MakeCharacterPhysics(inst, 1000, 1.5)
    MakePoisonableCharacter(inst)
    inst.Physics:SetCollisionCallback(OnCollide)

    anim:SetBank("bearger")
    anim:SetBuild("blackbear")
    anim:PlayAnimation("idle_loop", true)
    
    ------------------------------------------

	inst:AddTag("epic")
    inst:AddTag("monster")
    inst:AddTag("hostile")
    inst:AddTag("bearger")
    inst:AddTag("scarytoprey")
    inst:AddTag("largecreature")

    inst:AddComponent("sanityaura")
    inst.components.sanityaura.aurafn = CalcSanityAura

    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(6000)--@--

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(150) --注意调整
    inst.components.combat:SetRange(TUNING.BEARGER_ATTACK_RANGE, TUNING.BEARGER_MELEE_RANGE)
    inst.components.combat:SetAreaDamage(6, 0.8) --
    inst.components.combat.hiteffectsymbol = "bearger_body"
    inst.components.combat:SetAttackPeriod(TUNING.BEARGER_ATTACK_PERIOD)
    inst.components.combat:SetRetargetFunction(3, RetargetFn)
    inst.components.combat:SetKeepTargetFunction(KeepTargetFn)
    inst.components.combat:SetHurtSound("dontstarve_DLC001/creatures/bearger/hurt")
    inst:ListenForEvent("killed", function(inst, data)
        if inst.components.combat and data and data.victim == inst.components.combat.target then
            inst.components.combat.target = nil
        end 
    end)

    ----

    inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetChanceLootTable("blackbear")

    ------------------------------------------

    inst:AddComponent("inspectable")
    inst.components.inspectable:RecordViews()

    ------------------------------------------

    inst:AddComponent("knownlocations")
    inst:AddComponent("thief")
    inst:AddComponent("inventory")

    inst:AddComponent("groundpounder")
    inst.components.groundpounder.destroyer = true
    inst.components.groundpounder.groundpoundfx = "groundpound_fx"

    inst:AddComponent("timer")
    inst:AddComponent("eater")
    inst.components.eater.foodprefs = {"MEAT", "VEGGIE", "GENERIC"}
    inst.components.eater.eatwholestack = true

    inst:ListenForEvent("attacked", OnAttacked)
    inst:ListenForEvent("onhitother", OnHitOther)

    MakeLargeBurnableCharacter(inst, "swap_fire")
    MakeHugeFreezableCharacter(inst, "bearger_body")

    SetStandState(inst, "QUAD")
    inst.SetStandState = SetStandState
    inst.IsStandState = IsStandState
    inst.SeenBase = false
    inst.NearPlayerBase = NearPlayerBase
    inst.WorkEntities = WorkEntities
    --inst.CanGroundPound = false
    inst.KilledPlayer = false
    inst.spawnbarrier = spawnbarrier

    inst.num_good_food_eaten = 0

    inst.num_food_cherrypicked = 0

    inst.last_pound_time = 0
    inst.last_bw_time = 0
    inst.shouldPound = shouldPound
    inst.shouldBW = shouldBW
    inst.onPound = onPound
    inst.onBW = onBW
    inst.doPound = doPound

    
    inst:ListenForEvent("actionsuccess", function(player, data) OnPlayerAction(inst, player, data) end, GetPlayer())

    inst:ListenForEvent("killed", OnKill)

    inst.OnSave = OnSave
    inst.OnLoad = OnLoad

    ------------------------------------------

    inst:AddComponent("locomotor")
    inst.components.locomotor.walkspeed = 3
    inst.components.locomotor.runspeed = 10
    inst.components.locomotor:SetShouldRun(true)

    ------------------------------------------

    inst:SetStateGraph("SGblackbear")
    local brain = require("brains/blackbearbrain")
    inst:SetBrain(brain)

    return inst
end

return Prefab( "prefabs/blackbear", fn, assets) 