local assets =
{
    Asset("ANIM", "anim/cloud_puff_soft.zip"),
	Asset("ANIM", "anim/mk_cloudfx.zip"),
}

local function setupcontainer(inst, slots, bank, build, inspectslots, inspectbank, inspectbuild, inspectboatbadgepos, inspectboatequiproot)
	inst:AddComponent("container")
	inst.components.container:SetNumSlots(#slots)
	inst.components.container.type = "boat"
	inst.components.container.side_align_tip = -500
	inst.components.container.canbeopened = false

	inst.components.container.widgetslotpos = slots
	inst.components.container.widgetanimbank = bank
	inst.components.container.widgetanimbuild = build
	inst.components.container.widgetboatbadgepos = Vector3(0, 40, 0)
	inst.components.container.widgetequipslotroot = Vector3(-80, 40, 0)

	local boatwidgetinfo = {}
	boatwidgetinfo.widgetslotpos = inspectslots
	boatwidgetinfo.widgetanimbank = inspectbank
	boatwidgetinfo.widgetanimbuild = inspectbuild
	boatwidgetinfo.widgetboatbadgepos = inspectboatbadgepos
	boatwidgetinfo.widgetpos = Vector3(200, 0, 0)
	boatwidgetinfo.widgetequipslotroot = inspectboatequiproot
	inst.components.container.boatwidgetinfo = boatwidgetinfo
end

local function OnMounted(inst,data)
	print('mkcloud > call OnMounted')
	inst.AnimState:SetMultColour(0,0,0,0)
	inst.Physics:SetActive(false)
	inst.components.mk_cloudfxspawner:OnMounted()
	inst.components.inventoryitem.canbepickedup = false
	
	local driver = data and data.driver
	if driver then
		
		driver:AddTag('mythTag_oncloud')
		inst.entity:SetParent(driver.entity) ---这种写法会导致云无法被保存,需要修改driver组件的存读函数
		inst.Transform:SetPosition(0,0,0)
		MK_MOD_UTIL.SetPhysicsCollision(driver, {'ground', 'wav', 'char'})

		driver.components.locomotor:AddSpeedModifier_Mult('mk_cloud',.8)
		inst:DoTaskInTime(0,function()		
			MK_MOD_UTIL.SetPhysicsCollision(driver, {'ground', 'wav', 'char'})
		end)

		if not inst.pos_task then
			inst.pos_task = inst:DoPeriodicTask(1,function()
				if inst.components.drivable.driver then
					inst.Transform:SetPosition(0,0,0)  --修正坐标
				else
					inst.pos_task:Cancel()
					inst.pos_task = nil
				end
			end)
		end
	end
end

local function OnDismounted(inst,data)
	print('mkcloud > call Dismounted')
	inst.entity:SetParent(nil)
	inst.Physics:SetActive(true)
	inst.components.mk_cloudfxspawner:OnDismounted()
	inst.components.inventoryitem.canbepickedup = true
	if data and data.driver then
		inst.Transform:SetPosition(data.driver:GetPosition():Get())
		data.driver:RemoveTag('mythTag_oncloud')
		data.driver.components.locomotor:AddSpeedModifier_Mult('mk_cloud', 0)
	end
	if inst.pos_task then
		inst.pos_task:Cancel()
		inst.pos_task = nil
	end
end

local function OnDropped(inst)
	inst.entity:SetParent(nil)
	inst.Physics:SetActive(true)
	inst.components.mk_cloudfxspawner:OnDismounted()
end

local function ondepletedfn(inst)
	local driver = inst.components.drivable.driver
	if driver then
		driver.components.driver:OnDismount(true)
	end
	inst:Remove()
end

local function onhealthdelta(inst,a,b)
	local function IsCross(c) return a < c and b >= c end
	local say = IsCross(.5) and 1 or IsCross(.25) and 2 or IsCross(.1) and 3

	local driver = inst.components.drivable.driver
	if driver and say then
		if driver.prefab == 'monkey_king' then
			local lines = MK_MOD_LANGUAGE_SETTING and 
			{
				'OoO','OoO','OoO',
			} or
			{
				'俺老孙的筋斗云开始晃了',
				'这云怕是载不起俺老孙了',
				'俺老孙该落地了',
			}
			driver.components.talker:Say(lines[say])
		else
			local lines = MK_MOD_LANGUAGE_SETTING and 
			{
				'I want to get back to the ground.',
				'I don\'t want to fall',
				'Get me out of here!',
			} or
			{
				'我想回到地面...',
				"我可不想掉下去!",
				'快放我下来!!',
			}
			driver.components.talker:Say(lines[say])
		end
	end
end

local function fn() --筋斗云本体
	local inst = CreateEntity()
	inst.entity:AddTransform()
    inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	inst.entity:AddFollower()

	inst.AnimState:SetBank("mk_cloudfx")
    inst.AnimState:SetBuild("mk_cloudfx")
    inst.AnimState:PlayAnimation('clicker')

    inst.AnimState:SetMultColour(0,0,0,0)
    inst.AnimState:SetFinalOffset(1)

    inst.Transform:SetScale(.5,.5,.5)

	MakeInventoryFloatable(inst, 'clicker', 'clicker')
	MakeInventoryPhysics(inst)

	setupcontainer(inst, {}, "boat_hud_raft", "boat_hud_raft", {}, "boat_inspect_raft", "boat_inspect_raft", {x=0,y=5}, {})
	inst.components.container.CollectSceneActions = function() return end

	inst:AddComponent('inventoryitem')
	inst.components.inventoryitem.atlasname = "images/monkey_king_item.xml"
	inst.components.inventoryitem.nobounce = true
	local actionfn = inst.components.inventoryitem.CollectSceneActions
	inst.components.inventoryitem.CollectSceneActions = function(self,doer,actions,right)
		if not right then actionfn(self,doer,actions) end
	end

	inst:AddComponent('inspectable')
    inst.components.inspectable.CollectSceneActions = function(self,doer, actions, right) 
        if doer.components.driver and right then 
            table.insert(actions, ACTIONS.MOUNT)
        end
    end

	local fueltime = GetPlayer().prefab == 'monkey_king' and 3*480 or 1*480 --真正的耐久度
	local rate = 100/480 --修正数值
	local deltap = 1/fueltime  --燃料组件的刷新频率是1/s
	inst:AddComponent('fueled')
	inst.components.fueled:InitializeFuelLevel(GetPlayer().prefab == 'monkey_king' and 3*100 or 1*100)
	inst.components.fueled.rate = rate
	inst.components.fueled:SetDepletedFn(ondepletedfn)
	inst.components.fueled.fueltype = "MK_CLOUD"
	inst.components.fueled.accepting = true

	inst:ListenForEvent('percentusedchange',function(inst) 
		local now = inst.components.fueled:GetPercent()
		inst:PushEvent('cloudhealthchange',{percent=now,oldpercent=now+deltap,maxhealth=inst.components.fueled.maxfuel,});
		onhealthdelta(inst,now,now+deltap)
	end)
	
	inst:AddComponent("drivable")
	inst.components.drivable.overridebuild = 'mk_cloudfx'
	inst.components.drivable.flotsambuild = 'mk_cloudfx'

	inst.components.drivable.prerunanimation = 'surf_pre'
	inst.components.drivable.runanimation = 'surf_loop'
	inst.components.drivable.postrunanimation = 'surf_pst'

	inst.components.drivable.sailloopanim = "surf_loop"
	inst.components.drivable.sailstartanim = "surf_pre"
	inst.components.drivable.sailstopanim = "surf_pst"

	inst.components.drivable.alwayssail = true
	inst.components.drivable.hitfx = 'mk_cloudpuff_small'

	inst.components.drivable.creaksound = ''
	inst.components.drivable.runspeed = GetPlayer().prefab == 'monkey_king' and .5 or 0
	inst.components.drivable.donothideonmounted = true
	inst.components.drivable.maprevealbonus = 3

	inst:ListenForEvent('mounted',OnMounted)
	inst:ListenForEvent('dismounted',OnDismounted)
	inst:ListenForEvent('ondropped',OnDropped)

	inst:AddComponent('mk_cloudfxspawner')

	return inst
end

local function child()
	local inst = CreateEntity()
	inst.entity:AddTransform()
    inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()

	inst.AnimState:SetBank("mk_cloudfx")
    inst.AnimState:SetBuild("mk_cloudfx")
    inst.AnimState:PlayAnimation('anim_loop', true)
    inst.AnimState:SetFinalOffset(1)

	inst.ScaleFn = function(inst, min, max, basic)
	 	max = max or 1.3
		min = min or .8
		
		local step = 0.06+0.03*math.random()
		local s = inst.Transform:GetScale()*(basic or 1)
		local tick = math.random(628)
		inst:DoPeriodicTask(0,function()
			tick = tick+1
			local scale = s*((max+min)/2+((max-min)/2)*math.sin(tick*step))
			inst.Transform:SetScale(scale,scale,scale)
			if tick >= 63832 then
				tick = 0
			end
		end)
	end

	inst:AddTag('FX')
	inst:AddTag('NOCLICK')
	inst.persists = false

	return inst
end

local function tail() --筋斗云尾气
	local inst = CreateEntity()
	inst.entity:AddTransform()
    inst.entity:AddAnimState()
	inst.entity:AddPhysics()

	inst.AnimState:SetBank("mk_cloudfx")
    inst.AnimState:SetBuild("mk_cloudfx")
    inst.AnimState:PlayAnimation('anim_fade')
    inst.AnimState:SetDeltaTimeMultiplier(3.5)
    inst.AnimState:SetMultColour(.8,.8,.8,.5)
    inst.AnimState:SetTime(.4)
    inst.AnimState:SetFinalOffset(-5)

    inst:AddTag("NOCLICK")
    inst:AddTag("FX")
    inst.persists = false

    inst:AddComponent('colourtweener')
    inst.components.colourtweener:StartTween({.8,.8,.8,0},.7, inst.Remove)

	return inst
end

return Prefab("mk_cloud", fn, assets),
Prefab("mk_cloud_child", child),
Prefab("mk_cloud_tail", tail)
