
local function ondeploy(inst, pt, deployer)
    local sapling = SpawnPrefab("peach_sapling")
    sapling:StartGrowing()
    sapling.Transform:SetPosition(pt:Get())
    sapling.SoundEmitter:PlaySound("dontstarve/wilson/plant_tree")

    inst:Remove()
end

local notags = {'NOBLOCK', 'player', 'FX'}
local function test_ground(inst, pt)
	local tiletype = GetGroundTypeAtPosition(pt)
	local ground_OK = tiletype ~= GROUND.ROCKY and tiletype ~= GROUND.ROAD and tiletype ~= GROUND.IMPASSABLE and
						tiletype ~= GROUND.UNDERROCK and tiletype ~= GROUND.WOODFLOOR and 
						tiletype ~= GROUND.CARPET and tiletype ~= GROUND.CHECKER and tiletype < GROUND.UNDERGROUND
	
	if ground_OK then
	    local ents = TheSim:FindEntities(pt.x,pt.y,pt.z, 4, nil, notags) -- or we could include a flag to the search?
		local min_spacing = inst.components.deployable.min_spacing or 2

	    for k, v in pairs(ents) do
			if v ~= inst and v:IsValid() and v.entity:IsVisible() and not v.components.placer and v.parent == nil then
				if distsq( Vector3(v.Transform:GetWorldPosition()), pt) < min_spacing*min_spacing then
					return false
				end
			end
		end
		
		return true

	end
	return false
	
end

local function makefood(name,preparedfood,data)
	
	local assets =
	{
		Asset("ANIM", "anim/"..name..".zip"),
	}
	if name == 'gourd_cooked' then
		assets = nil
	end

	local function fn()
		local inst = CreateEntity()

		inst.entity:AddTransform()
		inst.entity:AddAnimState()
		inst.entity:AddSoundEmitter()


		MakeInventoryPhysics(inst)
	    MakeInventoryFloatable(inst, 'idle', 'idle')

		inst.AnimState:SetBank(name)
		inst.AnimState:SetBuild(name)
		inst.AnimState:PlayAnimation("idle")
		
        inst:AddComponent("edible")
        inst.components.edible.healthvalue = data.health or 0
        inst.components.edible.hungervalue = data.hunger or 0
        inst.components.edible.foodtype = "VEGGIE"
        inst.components.edible.sanityvalue = data.sanity or 0
        inst.components.edible.temperaturedelta = data.temperature or 0
        inst.components.edible.temperatureduration = data.temperatureduration or 0
	

		if preparedfood then
			inst.components.edible.foodstate = "PREPARED" --修复沃利的食物判定
			inst:AddTag("preparedfood")
		end

		inst:AddComponent("tradable")

		inst:AddComponent("inspectable")
		
        inst:AddComponent("stackable")
        inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM
		
		inst:AddComponent("inventoryitem")
		inst.components.inventoryitem.imagename = name
		inst.components.inventoryitem.atlasname = "images/monkey_king_item.xml"
		
		inst:AddComponent("perishable")
		inst.components.perishable:SetPerishTime(data.perishtime)
		inst.components.perishable:StartPerishing()
		inst.components.perishable.onperishreplacement = "spoiled_food"
		
		if name == "peach" then
			inst:AddComponent("cookable")
			inst.components.cookable.product = name.."_cooked"
			
			inst:AddComponent("deployable")
			inst.components.deployable.ondeploy = ondeploy
	    	inst.components.deployable.test = test_ground
		end

		if name == "gourd" then
			inst:AddComponent("cookable")
			inst.components.cookable.product = name.."_cooked"
		end

		if name == "gourd_cooked" then
			inst.AnimState:SetBuild('gourd')
			inst.AnimState:SetBank('gourd')
			MakeInventoryFloatable(inst, 'cooked', 'cooked')
		end

		if name == 'peach_wine' then
			inst.mk_bufffn = {
				on = function(i) i.components.sanity.dapperness = i.components.sanity.dapperness+TUNING.DAPPERNESS_LARGE end,
				off= function(i) i.components.sanity.dapperness = i.components.sanity.dapperness-TUNING.DAPPERNESS_LARGE end,
			}
			inst.components.edible.oneaten = function(inst,eater)
				if eater.prefab == 'monkey_king' and not eater.components.health:IsDead() then
					eater.components.mk_timerbuff_manager:AddBuff(name,300)
				end
			end
			inst:AddComponent("wine")
		end

		if name == 'bigpeach' then
			inst:AddComponent("cookable")
			inst.components.cookable.product = "peach_cooked"

			inst.components.edible.oneaten = function(inst,eater)
				if eater.components.poisonable then
					eater.components.poisonable:Cure(inst)
				end
			end
		end

		return inst
	end
	return Prefab(name, fn, assets)
end
return  MakePlacer("peach_placer", "peach_sapling", "peach_sapling", "plant"),
		makefood("peach",false,{ health = 12, hunger = 12.5, sanity = 5 ,  perishtime = TUNING.WINTERHAT_PERISHTIME}), 
		makefood("gourd",false,{ health = 3, hunger = 37.5, perishtime = TUNING.AUTOSAVE_INTERVAL* 10}),
		makefood("gourd_cooked",false,{health = 8, hunger = 37.5, perishtime = TUNING.AUTOSAVE_INTERVAL*6}),
		makefood("bigpeach",false,{health = 1024, hunger = 2048, sanity = 4096, perishtime = TUNING.WINTERHAT_PERISHTIME}),
		makefood("peach_cooked",false,{ health = 6, hunger = 12.5, sanity = 0 ,  perishtime = TUNING.UMBRELLA_PERISHTIME }), 
		makefood("peach_banquet",true,{  health = 40, hunger = 37.5, sanity = 10 ,  perishtime = TUNING.WINTERHAT_PERISHTIME,temperaturedelta = TUNING.COLD_FOOD_BONUS_TEMP,temperatureduration = TUNING.FOOD_TEMP_BRIEF}),
		makefood("peach_wine",true,{ health = 3, hunger = 12.5, sanity = 30 ,  perishtime = TUNING.TRUNKVEST_PERISHTIME,temperaturedelta = TUNING.COLD_FOOD_BONUS_TEMP,temperatureduration = TUNING.FOOD_TEMP_BRIEF }),
		makefood("honey_pie",true,{ health = 0, hunger = 37.5, sanity = 5 ,  perishtime = TUNING.AUTOSAVE_INTERVAL* 25 }),
		makefood("gourd_soup",true,{ health = 20, hunger = 37.5, sanity = 15 ,  perishtime = TUNING.AUTOSAVE_INTERVAL* 15}),	
		makefood("gourd_omelet",true,{ health = 40, hunger = 75, sanity = 10 ,  perishtime = TUNING.AUTOSAVE_INTERVAL* 20}),	
		makefood("vegetarian_food",true,{ health = 0, hunger = 75, sanity = 0 ,  perishtime = TUNING.AUTOSAVE_INTERVAL* 10})