local assets =
{
    Asset("ANIM", "anim/mk_jgb.zip"),
}

local fxassets =
{
    Asset("ANIM", "anim/dst_myth/lavaarena_staff_smoke_fx_myth.zip"),
}

local prefabs =
{
    "mk_jgb_fx",
}
local function resetbuild(inst)
    if inst._skin == 'sea' then
        inst.AnimState:OverrideSymbol('drop', 'mk_jgb', 'drop_sea')
        inst.AnimState:OverrideSymbol('item', 'mk_jgb', 'item_sea')
        inst.components.inventoryitem:ChangeImageName('mk_jgb_sea')
    else
        inst.AnimState:ClearOverrideSymbol('drop')
        inst.AnimState:ClearOverrideSymbol('item')
        inst.components.inventoryitem:ChangeImageName('mk_jgb')
    end
end

local function onequip(inst, owner)
    if inst._skin == 'sea' then
        owner.AnimState:OverrideSymbol('swap_object', 'mk_jgb', 'swap_mk_jgb_sea')
    else
        owner.AnimState:OverrideSymbol("swap_object", "mk_jgb", "swap_mk_jgb")
    end
	owner.AnimState:Show("ARM_carry")
	owner.AnimState:Hide("ARM_normal")
end

local function onunequip(inst, owner)
	owner.AnimState:ClearOverrideSymbol("swap_object")
    owner.AnimState:Hide("ARM_carry")
    owner.AnimState:Show("ARM_normal")
end

local function TryLunge(inst, doer, pos)
	doer:PushEvent("mklunge", {targetpos = pos, weapon = inst})
    inst.components.myth_rechargeable:SetRechargeTime(15*(doer:HasTag('skill_boost_mk') and .85 or 1))
	--inst.components.myth_rechargeable:StartRecharging()
end

local function DoLunge(inst, doer, pos, targetpos)
    local numsteps = 8
    local x, y, z = doer.Transform:GetWorldPosition() 
    local angle = (doer.Transform:GetRotation() + 90) * DEGREES   
    local step = .75
    local offset = 1 - step 
    local ground = GetWorld().Map
    local tile = ground:GetTileAtPoint(x, y, z)
    local i = -1
    local noground = false
    local fx, dist, delay, x1, z1
    local targets = {}
    while i < numsteps do
        i = i + 1
        dist = i * step + offset 
        delay = math.max(0, i - 1)
        x1 = x + dist * math.sin(angle) 
        z1 = z + dist * math.cos(angle)
        if not (tile ~= GROUND.IMPASSABLE and tile ~= GROUND.INVALID ) then
            noground = true
        end
        fx = SpawnPrefab("mk_jgb_fx")
        fx.Transform:SetPosition(x1, 0, z1)
        fx:Trigger(delay * FRAMES, targets, doer, inst)

        if noground then break end
    end
    if i < numsteps then
        dist = (i + .5) * step + offset
        x1 = x + dist * math.sin(angle)
        z1 = z + dist * math.cos(angle)
    end
    return true
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddMiniMapEntity()
    inst.MiniMapEntity:SetIcon("mk_jgb.tex")

    MakeInventoryPhysics(inst)

    inst.AnimState:SetBank("swap_mk_jgb")
    inst.AnimState:SetBuild("mk_jgb")
    inst.AnimState:PlayAnimation("idle")

    inst._skin = 'none'
    inst:ListenForEvent('mythskin_change', function(_, data)
        if data == 'monkey_king_none' then
            inst._skin = 'none'
        elseif data == 'monkey_king_sea' then
            inst._skin = 'sea'
        end
        resetbuild(inst)
    end, GetPlayer())

    inst:AddTag("sharp")
    inst:AddTag("pointy")
    inst:AddTag("mk_jgb")

    inst:AddTag("aoeweapon_lunge")
    inst:AddTag("rechargeable")
    inst:AddTag("irreplaceable")

    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(59.5)
	inst.components.weapon:SetRange(1)

    inst:AddComponent("inspectable")

    inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.imagename = "mk_jgb"
	inst.components.inventoryitem.atlasname = "images/monkey_king_item.xml"
    local old_hitwater = inst.components.inventoryitem.OnHitWater
    inst.components.inventoryitem.OnHitWater = function(self) 
        SpawnPrefab('small_puff').Transform:SetPosition(inst:GetPosition():Get())
        inst:DoTaskInTime(0,function()GetPlayer().components.inventory:GiveItem(inst)end)
        old_hitwater(self)
    end

    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(onequip)
    inst.components.equippable:SetOnUnequip(onunequip)

	inst:AddComponent("myth_weaponreticule")
	inst.components.myth_weaponreticule:SetSpellFn(TryLunge)
    inst.DoLunge = DoLunge
	
	inst:AddComponent("myth_rechargeable")
	inst.components.myth_rechargeable:SetRechargeTime(15)

	inst:DoTaskInTime(0,function()
        if GetPlayer().prefab ~= 'monkey_king' then 
            inst:Remove()
            return
        end 
        for k,v in pairs(Ents) do
            if v.prefab == "mk_jgb" and v ~= inst and v:IsValid() then
                if v.spawn_by_sp then
                    v:DoTaskInTime(0, function() v.spawn_by_sp = nil end)
                else
                    v:Remove()
                end
            end
        end
    end)
    
    resetbuild(inst)
    inst:ListenForEvent('ondropped', resetbuild)
    return inst
end


local function DoDamage(inst, targets, doer, weapon)
    local damage = doer:HasTag('skill_boost_mk') and 150 or 100
	local x, y, z = inst.Transform:GetWorldPosition()
    if inst.AnimState ~= nil then
		inst.AnimState:PlayAnimation("idle")
        inst.AnimState:SetAddColour(1, 1, 0, 1)
		inst:Show()
		inst:DoTaskInTime(0.7 + 2 * FRAMES, inst.Remove)
    else
        inst:DoTaskInTime(2 * FRAMES, inst.Remove)
	end
    for i, v in ipairs(TheSim:FindEntities(x, 0, z, 4, nil, { "playerghost", "INLIMBO", "player","companion","wall", 'mythTag_hitbymkaoe' })) do
        if not targets[v] and v:IsValid() and not (v.components.health ~= nil and v.components.health:IsDead()) then
			if inst:IsNear(v, 2 + (v.Physics and v.Physics:GetRadius() or 0)) then
                if doer ~= nil and doer:IsValid() and doer.components.combat and doer.components.combat:CanTarget(v) then
                    targets[v] = true
					doer:PushEvent("onareaattackother", { target = v, weapon = weapon or nil, stimuli = "mk_jgb_skill" })
					v.components.combat:GetAttacked(doer, damage, weapon, "mk_jgb_skill")
                    --v:AddTag('mythTag_hitbymkaoe')
                    --v:DoTaskInTime(0, function() v:RemoveTag('mythTag_hitbymkaoe') end)
                end
            end
        end
    end
end

local function Trigger(inst, delay, targets, doer, weapon)
    if (delay or 0) > 0 then
        inst.task = inst:DoTaskInTime(delay, DoDamage, targets or {}, doer, weapon)
    else
        DoDamage(inst, targets or {}, doer, weapon)
    end
end

local function fxfn()
	local inst = CreateEntity()

	inst.entity:AddTransform()
	inst.entity:AddAnimState()

	inst:AddTag("FX")
	inst:AddTag("NOCLICK")

	inst.AnimState:SetBank("lavaarena_staff_smoke_fx")
	inst.AnimState:SetBuild("lavaarena_staff_smoke_fx_myth")
	--inst.AnimState:SetAddColour(1, 1, 0, 0)
	inst.AnimState:SetBloomEffectHandle("shaders/anim.ksh")
	inst:Hide()

	inst.persists = false

	inst.Trigger = Trigger  

	return inst
end

return Prefab("mk_jgb", fn, assets,prefabs),
		Prefab("mk_jgb_fx", fxfn, fxassets)