local function autoputoff(inst)
	if inst.components.burnable.autoputoff_task then
		inst.components.burnable.autoputoff:Cancel()
		inst.components.burnable.autoputoff = nil
	end
	inst.components.burnable.autoputoff_task = inst:DoTaskInTime(2,function()
		inst.components.burnable:Extinguish()
		inst.components.burnable.autoputoff_task = nil
	end)
end

local function HeatOn(inst,owner,vel)
	if owner.components.health then
	 	owner.components.health.fire_damage_scale = owner.components.health.fire_damage_scale +(vel and -1 or 1)
	end
	if owner.components.burnable then
		if vel then
			owner:ListenForEvent('onignite',autoputoff)
		else
			owner:RemoveEventCallback('onignite',autoputoff)
		end
	end
	if owner.components.temperature then
		owner.components.temperature.hot_resistance_override = vel and 10000 or nil
	end
end

local function ColdOn(inst,owner,vel)
	--print("DEBUG: cold resistant on")
	if owner.components.temperature then
		owner.components.temperature.cold_resistance_override = vel and 10000 or nil
	end

	if owner.components.inventoryitem then
		if vel then
			if not owner.components.waterproofer then
				owner:AddComponent("waterproofer")
				owner.components.waterproofer.temp = true
				owner.components.waterproofer:SetEffectiveness(0)
			end
		else
			if owner.components.waterproofer and owner.components.waterproofer.temp then
				owner:RemoveComponent("waterproofer")
			end
		end
	end
end

local function DustOn(inst,owner,vel)
	if owner:HasTag("player")then
		if vel then
			owner:AddTag('prevents_hayfever')
		else
			owner:RemoveTag('prevents_hayfever')
		end
	end
	--防风见component.lua
end

local function ThornOn(inst, owner, vel)
	if vel then
		owner:AddTag('bramble_resistant')
		owner:AddTag('armorcactus')
	else
		owner:RemoveTag('bramble_resistant')
		owner:RemoveTag('armorcactus')
	end
end

local bufffns = 
{
	heat_resistant_pill = { on = function(inst,owner,vel) HeatOn(inst,owner,true)end, off = function(inst,owner,vel) HeatOn(inst,owner,false)end},
	cold_resistant_pill = { on = function(inst,owner,vel) ColdOn(inst,owner,true)end, off = function(inst,owner,vel) ColdOn(inst,owner,false)end},
	dust_resistant_pill = { on = function(inst,owner,vel) DustOn(inst,owner,true)end, off = function(inst,owner,vel) DustOn(inst,owner,false)end},
	thorns_pill = {on=function(inst, owner) ThornOn(inst,owner,true) end, off=function(inst,owner) ThornOn(inst,owner,false) end},
}	

local assets = {Asset("ANIM", "anim/mk_pills.zip"),}

local function makeinvpill(name)
	local function applybuff(owner)
		if not owner then
			return
		end
		if not owner.components.mk_invbuff_manager then
			owner:AddComponent('mk_invbuff_manager')
		end
		owner.components.mk_invbuff_manager:Apply()
	end
	local function fn()
		local inst = CreateEntity()

		inst.entity:AddTransform()
		inst.entity:AddAnimState()
		inst.entity:AddSoundEmitter()

		inst:AddTag("irreplaceable")

		MakeInventoryPhysics(inst)
		MakeInventoryFloatable(inst, name, name)

		inst.AnimState:SetBank("mk_pills")
		inst.AnimState:SetBuild("mk_pills")
		inst.AnimState:PlayAnimation(name)

		inst:AddTag('mk_invbuff')
		inst.mk_bufffn = {on = bufffns[name].on,off = bufffns[name].off}
		
		inst:AddComponent("tradable")

		inst:AddComponent("inspectable")

		inst:AddComponent("inventoryitem")
		inst.components.inventoryitem.imagename = name
		inst.components.inventoryitem.atlasname = "images/monkey_king_item.xml"
		inst.components.inventoryitem.onputininventoryfn = function (inst,owner)
			applybuff(owner)
			applybuff(inst.components.inventoryitem:GetGrandOwner())
		end

		inst:ListenForEvent('onpickup', function (inst,data)
			applybuff(data and data.owner)
		end)

		if name == 'cold_resistant_pill' then
			inst:AddComponent("machine")
			inst.components.machine.CollectSceneActions = function() return end
			inst.components.machine.CollectInventoryActions = function() return end
			inst.components.machine.IsOn = function()
				return inst.components.inventoryitem:GetGrandOwner() and inst.components.inventoryitem:GetGrandOwner():HasTag('player')
			end
			inst:AddTag('blows_air')

			inst:AddComponent("waterproofer")
			inst.components.waterproofer:SetEffectiveness(0)

			inst:AddComponent("heater")
			inst.components.heater.carriedheatfn = function(inst,observer)
				local observer = inst.components.inventoryitem:GetGrandOwner()
				if observer and observer.components.temperature and observer.components.temperature.current <= 20 then
					return 100
				else
					return 0
				end
			end
		elseif name == 'heat_resistant_pill' then
			inst:AddComponent("heater")
			inst.components.heater.iscooler = true
			inst.components.heater.carriedheatfn = function(inst,observer)
				local observer = inst.components.inventoryitem:GetGrandOwner()
				if observer and observer.components.temperature and observer.components.temperature.current >= 50 then
					return -100
				else
					return 0
				end
			end
		elseif name == 'thorns_pill' then
			inst:AddComponent("fueled")
			inst.components.fueled:InitializeFuelLevel(15*480)
			inst.components.fueled:StartConsuming()
			inst.components.fueled:SetDepletedFn(inst.Remove)
			inst.components.fueled.updatefn = function(inst)
				if inst.components.inventoryitem.owner and inst.components.inventoryitem.owner.prefab == 'gourd_pill' then
					inst.components.fueled:StopConsuming()
				end
			end
			inst:ListenForEvent('ondropped', function(inst) inst.components.fueled:StartConsuming()end)
			inst:ListenForEvent('onputininventory', function(inst)
				if inst.components.inventoryitem.owner and inst.components.inventoryitem.owner.prefab == 'gourd_pill' then
					inst.components.fueled:StopConsuming()
				else
					inst.components.fueled:StartConsuming()
				end
			end)
		end

		return inst
	end
	return Prefab(name, fn, assets)
end

local function makeeatpill(name,eatfn)
	local function fn()
		local inst = CreateEntity()

		inst.entity:AddTransform()
		inst.entity:AddAnimState()
		inst.entity:AddSoundEmitter()

		MakeInventoryPhysics(inst)
		MakeInventoryFloatable(inst, name, name)

		inst:AddTag("irreplaceable")

		inst.AnimState:SetBank("mk_pills")
		inst.AnimState:SetBuild("mk_pills")
		inst.AnimState:PlayAnimation(name)
		
		inst:AddComponent("edible")
		inst.components.edible.hungervalue = 0
		if name == "bloodthirsty_pill" then
			inst.components.edible.healthvalue = -40
			inst.components.edible.sanityvalue = -50
		else
			inst.components.edible.healthvalue = 0
		end
		inst.components.edible.oneaten = function(inst,eater)
			if eater:HasTag('player') and not eater.components.health:IsDead() then
				eater.components.mk_timerbuff_manager:AddBuff(name)
			end
		end

		inst:AddComponent("tradable")

		inst:AddComponent("inspectable")

		inst:AddComponent('stackable')

		inst:AddComponent("inventoryitem")
		inst.components.inventoryitem.imagename = name
		inst.components.inventoryitem.atlasname = "images/monkey_king_item.xml"

		if name == "fly_pill" then
			inst:AddComponent("fuel")
			inst.components.fuel.fuelvalue = 480
			inst.components.fuel.fueltype = "MK_CLOUD"

			inst.components.edible.caffeinedelta = TUNING.CAFFEINE_FOOD_BONUS_SPEED
			inst.components.edible.caffeineduration = 500
		end

		return inst
	end
	return Prefab(name, fn, assets)
end
return  makeinvpill("heat_resistant_pill"), --避火
		makeinvpill("cold_resistant_pill"), --避水
		makeinvpill("dust_resistant_pill"), --避尘
		makeinvpill("thorns_pill"),   --反伤
		makeeatpill("fly_pill"), --腾云
		makeeatpill("bloodthirsty_pill"), --嗜血
		makeeatpill("condensed_pill"),--会心
		makeeatpill("detoxic_pill"), --解毒
		makeeatpill("armor_pill") --壮骨
		