
local MakePlayerCharacter = require "prefabs/player_common"
local easing = require "easing"
local cooking = require 'cooking'
local preparedfoods = require 'preparedfoods'

local assets = {
    Asset("SCRIPT", "scripts/prefabs/player_common.lua"),
    Asset("ANIM", "anim/monkey_king.zip"),

	Asset("ANIM", "anim/dst_myth/player_attack_leap_myth.zip"),

	Asset("SOUNDPACKAGE", "sound/mythwords_bgm.fev"),
    Asset("SOUND", "sound/mythwords_bgm.fsb"),

    Asset("IMAGE", "images/colour_cubes/beaver_vision_cc.tex"),
}
if softresolvefilepath('anim/walani.zip') and softresolvefilepath('anim/walani_paddle.zip') then
	table.insert(assets,Asset("ANIM", "anim/walani.zip"))
	table.insert(assets,Asset("ANIM", "anim/walani_paddle.zip"))
end

local prefabs = {
	"mk_jgb",
	"mk_striker",
	"peach",
	"mk_striker_fx"
}

local start_inv = {
	"mk_jgb","peach","peach","peach","peach","peach",
}
local function ignorefog(inst)
	inst:AddTag('venting')
end

local function CanEnableFireEye(inst,check)
	local sm = GetSeasonManager()
    if sm and sm:IsHurricaneStorm() and not inst.components.mk_invbuff_manager:HasBuff('dust_resistant_pill') then
    	if not check or check and inst.fireeye then
    		inst.components.talker:Say(STRINGS.MKINWIND)
    	end
    	return false
    end
    local hay = inst.components.hayfever
    if hay and hay.enabled and hay.nextsneeze <= 60 and not inst.components.mk_invbuff_manager:HasBuff('dust_resistant_pill') then
    	if not check or check and inst.fireeye then
    		inst.components.talker:Say(STRINGS.MKINSANDSTORM) 
    	end
    	return false
    end

	return true
end

local function SetBeaverVision(inst, enable)
	if enable and not CanEnableFireEye(inst) then
		return
	end
	if enable then

	GetWorld().components.colourcubemanager:SetOverrideColourCube(resolvefilepath "images/colour_cubes/beaver_vision_cc.tex")
	inst.Light:Enable(true)
	inst.fireeye = true
	inst:PushEvent('mkEvent_fireeye', true)
	if GetWorld().components.wb_spawner then
		GetWorld().components.wb_spawner:OnFireEye(true)
	end

	inst:DoTaskInTime(0,function()
  		if inst:HasTag("lightsource") then       
            inst:RemoveTag("lightsource")    
        end
    end) 

	local firefx = SpawnPrefab('mk_halloween_firepuff_'..math.random(3))
	firefx.Transform:SetPosition(inst:GetPosition():Get())
	--inst.components.hunger:AddBurnRateModifier("_mknightvision",inst:HasTag('fireeye_boost_mk') and 1 or 3)
	if not inst.fireeye_hunger_task then
		inst.fireeye_hunger_task = inst:DoPeriodicTask(1,function()
			inst.components.hunger:DoDelta(inst:HasTag('fireeye_boost_mk') and 0 or -2/3,true)
		end)
	end
	--inst.components.sanity.dapperness = -TUNING.DAPPERNESS_HUGE * 0.5
	
	inst:AddTag("venting")
	inst:ListenForEvent('unequip',ignorefog)

	else -------------------------------------

	GetWorld().components.colourcubemanager:SetOverrideColourCube(nil)
	inst.Light:Enable(false)
	inst.fireeye = false
	inst:PushEvent('mkEvent_fireeye', false)
	if GetWorld().components.wb_spawner then
		GetWorld().components.wb_spawner:OnFireEye(false)
	end

	--inst.components.hunger:RemoveBurnRateModifier("_mknightvision")
	if inst.fireeye_hunger_task then
		inst.fireeye_hunger_task:Cancel()
		inst.fireeye_hunger_task = nil
	end
	--inst.components.sanity.dapperness = 0

	if not inst.components.inventory:EquipHasTag('venting')then
		inst:RemoveTag("venting")
	end
	inst:RemoveEventCallback('unequip',ignorefog)

	end
end

local function CheckFireEye(inst) 
	if not CanEnableFireEye(inst,true) and inst.fireeye then
		SetBeaverVision(inst,false)
	end
end

local function lunge(inst,target)
	local x,y,z = target.Transform:GetWorldPosition()
	local r = 6
	local du = math.random(180)
	inst:StartThread(function()
        for k = 1, 4 do  
		local g = du + k* 90 
			local striker = SpawnPrefab("mk_striker")
			if striker then
			striker.Transform:SetPosition(x+ r*math.cos(2*math.pi/360*g),0,z+r*math.sin(2*math.pi/360*g))
			SpawnPrefab("mk_striker_fx").Transform:SetPosition(striker.Transform:GetWorldPosition())
			striker:ForceFacePoint(x, 0, z)
			if striker.Attack then
				striker:Attack(inst,target)
			end
            Sleep(.15)
        end
		end
    end)
end

local function onhitotherfn(inst, target, damage, stimuli)
	if not target:IsValid() then
		inst.mk_hitcounter = {}
		return
	end
	if stimuli == 'monkey_king_strike' or stimuli == 'monkey_king_aoe' or stimuli == 'mk_jgb_skill' then
		return
	end
	if not inst.components.combat:GetWeapon() then
		return
	end
	if inst.mk_hitcounter[target]then
		inst.mk_hitcounter[target]=inst.mk_hitcounter[target]+1
		if inst.mk_hitcounter[target] == 10 then
			lunge(inst,target)
			inst.mk_hitcounter[target] = 0
		end
	else
		inst.mk_hitcounter = nil
		inst.mk_hitcounter = {[target]=0}
	end
	----几率aoe
	if inst.components.rider and inst.components.rider:IsRiding() then
		return
	end
	local rate = inst:HasTag("mk_battle_flag_buff") and .05 or .01
	if math.random() < rate then
		inst.sg:GoToState("mk_helmsplitter_pre")
	end
end

local function OnAttacked(inst, data)
	if not data.attacker or data.attacker == 'primeape' then
		return
	end
	inst.components.combat:ShareTarget(data.attacker, 20,function(dude)
        return dude.prefab == 'primeape' and dude.components.follower.leader == inst
    end, 10)
end

local function DropWeapon(inst, data)
	if data and data.stimuli == 'monkey_king_strike' then
		return
	end
    local tool = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
    if tool and tool.prefab ~= "mk_jgb"  and math.random() < 0.5 then
    	inst.mk_hitcounter = nil
    	inst.mk_hitcounter = {}
        local projectile =
            data and data.weapon ~= nil and
            data.projectile == nil and
            (data.weapon.components.projectile ~= nil or data.weapon.components.complexprojectile ~= nil)

        if projectile then
        	do return end
            local num = data.weapon.components.stackable ~= nil and data.weapon.components.stackable:StackSize() or 1
            if num <= 1 then
               return
            end
            inst.components.inventory:Unequip(EQUIPSLOTS.HANDS)
            tool = data.weapon.components.stackable:Get(num - 1)
            tool.Transform:SetPosition(inst.Transform:GetWorldPosition())
           if tool.components.inventoryitem ~= nil then
                tool.components.inventoryitem:OnDropped()
            end
        else
            inst.components.inventory:Unequip(EQUIPSLOTS.HANDS)
            inst.components.inventory:DropItem(tool, true, true)
        end

        if tool.Physics ~= nil then
            local x, y, z = tool.Transform:GetWorldPosition()
            tool.Physics:Teleport(x, .3, z)
            local angle = (math.random() * 20 - 10) * DEGREES
            if data.target ~= nil and data.target:IsValid() then
                local x1, y1, z1 = inst.Transform:GetWorldPosition()
                x, y, z = data.target.Transform:GetWorldPosition()
                angle = angle + (
                    (x1 == x and z1 == z and math.random() * 2 * PI) or
                    (projectile and math.atan2(z - z1, x - x1)) or
                    math.atan2(z1 - z, x1 - x)
                )
            else
                angle = angle + math.random() * 2 * PI
            end
            local speed = projectile and 2 + math.random() or 3 + math.random() * 2
            tool.Physics:SetVel(math.cos(angle) * speed, 10, math.sin(angle) * speed)
        end
		inst.components.talker:Say(STRINGS.MKDROPWEAPON)
    end
end

local function sanityfn(inst)
    local x, y, z = inst.Transform:GetWorldPosition() 
    local delta = 0
	local moisture_delta = inst.components.moisture == nil and 0 or
		easing.inSine(inst.components.moisture.moisture, 0, TUNING.MOISTURE_SANITY_PENALTY_MAX, inst.components.moisture.moistureclamp.max)
	local max_rad = 10
    local ents = TheSim:FindEntities(x, y, z, max_rad, nil, nil , { "renewable", "burnt", "bush","tree"} )
    for i, v in ipairs(ents) do
        if v and  ( v.components.burnable and  v.components.burnable:IsBurning() or v:HasTag("burnt") ) then
            local sz = TUNING.SANITYAURA_TINY / max_rad
            local distsq = inst:GetDistanceSqToInst(v)
            delta = delta + 15/ (30*distsq)
        end
    end
    local fireeye_san = inst.fireeye and -0.5*TUNING.DAPPERNESS_HUGE or 0
    fireeye_san = fireeye_san * (inst:HasTag('skill_boost_mk') and 0 or 1)
	return moisture_delta - delta + fireeye_san
end

local function oneat(inst,food)
	local function IsFruit(item)
		local ingredients = cooking.ingredients
		local name = item.prefab or ""
		if item.components.edible and item.components.edible.foodtype == 'VEGGIE' then
			--return true
		end
		if ingredients[name] and ingredients[name].tags and ingredients[name].tags.fruit and ingredients[name].tags.fruit >=1 then
			return true
		end
	end
	if food and IsFruit(food) then
		inst.components.sanity:DoDelta(6)
	end
	if food and food.components.edible and food.components.edible.foodtype == "MEAT" then
		inst.components.talker:Say(STRINGS.MKONEATMEAT)
	end
end

local function gethungermultfn(com,food,value)
	if food.components.edible.foodtype == "MEAT" and value > 0 then
		if preparedfoods[food.prefab] then
			return 0.8
		else
			return 0.2
		end
	end
	return 1
end

local fn = function(inst) 
	inst.MiniMapEntity:SetIcon( "monkey_king.tex" )
	
	inst:AddTag("monkey_king")
	--inst:AddTag("monkey")
	inst.soundsname = "sfx"
	inst.talker_path_override = 'MK/'

	inst.entity:AddLight()
    inst.Light:Enable(false)
	inst.Light:SetRadius(5)
    inst.Light:SetFalloff(.5)
    inst.Light:SetIntensity(.6)
    inst.Light:SetColour(245/255,40/255,0/255)

	inst.fireeye = false

	inst.components.health:SetMaxHealth(175)
	inst.components.hunger:SetMax(150)
	inst.components.sanity:SetMax(125)
	
	inst.components.sanity.custom_rate_fn = sanityfn
	
    inst.components.combat.damagemultiplier = 1
	inst.components.combat.onhitotherfn = onhitotherfn
	inst.mk_hitcounter = {}
	
	inst.components.hunger.hungerrate = 1 * TUNING.WILSON_HUNGER_RATE

	inst.components.eater:SetOnEatFn(oneat)
	inst.components.eater.gethungermultfn = gethungermultfn

	inst.components.locomotor.runspeed = 6.5

	inst:AddComponent("monkeyfriend")

	inst:ListenForEvent("onhitother", DropWeapon)
	inst:ListenForEvent("attacked", OnAttacked)

	inst:ListenForEvent("death", CheckFireEye)
	inst:ListenForEvent("updatepollen",CheckFireEye)
	inst:ListenForEvent("hurricanestart",function()CheckFireEye(inst)end,GetWorld())
	inst:ListenForEvent("wind_rampup",function()CheckFireEye(inst)end,GetWorld())
	inst:ListenForEvent("windguststart",function()CheckFireEye(inst)end,GetWorld())

	local key = KEY_H
	TheInput:AddKeyDownHandler(key, function()
    	if not IsPaused() then
    		--and not TheInput:IsKeyDown(KEY_CTRL) and not TheInput:IsKeyDown(KEY_SHIFT) and not TheInput:IsKeyDown(KEY_ALT) then
    		if inst.TriggerFireEyeUI then
    			inst:TriggerFireEyeUI()
    		end
    	end 
    end)

    inst.TriggerFireEye = function(inst)
    	if inst.fireeye then
			SetBeaverVision(inst,false)
		elseif CanEnableFireEye(inst) then
			SetBeaverVision(inst,true)
		end
	end

    inst.components.inventory:GuaranteeItems({'mk_jgb'})

    MK_MOD_SKINS.handler(inst, 'monkey_king') --皮肤接口
end

return MakePlayerCharacter("monkey_king", prefabs, assets,fn, start_inv)
