local assets =
{
    Asset("ANIM", "anim/dst_myth/rhinodrill_anim.zip"),
    Asset("ANIM", "anim/dst_myth/rhinodrill_blue.zip"),
    Asset("ANIM", "anim/dst_myth/rhinodrill_red.zip"),
    Asset("ANIM", "anim/dst_myth/rhinodrill_yellow.zip"),
}
local L = MK_MOD_LANGUAGE_SETTING

STRINGS.NAMES.RHINO3_RED = L and "" or '辟暑大王' 
STRINGS.CHARACTERS.GENERIC.DESCRIBE.RHINO3_RED = L and "" or "" 

STRINGS.NAMES.RHINO3_BLUE = L and "" or '辟寒大王' 
STRINGS.CHARACTERS.GENERIC.DESCRIBE.RHINO3_BLUE = L and "" or "" 

STRINGS.NAMES.RHINO3_YELLOW = L and "" or '辟尘大王' 
STRINGS.CHARACTERS.GENERIC.DESCRIBE.RHINO3_YELLOW = L and "" or "" 

require 'stategraphs/SGrhino3'

local ATTACKED_BY_SPELL = {
    '兄弟, 你能不能看准点放?',
    '????????????',
    '猪队友啊啊啊',
    '喂, 你到底是哪一边的?',
    '别用法术了, 秋梨膏!',
}

local function OnAttackedBySpell(inst)
    if GetTime() - inst.last_spelled > 15 then
        inst.last_spelled = GetTime()
        inst.components.talker:Say(ATTACKED_BY_SPELL[math.random(#ATTACKED_BY_SPELL)])
    end
end

local function KeepTargetFn(inst, target)
    return inst.components.combat:CanTarget(target)
end

local function RetargetFn(inst, target)
    if inst.components.sleeper and inst.components.sleeper:IsAsleep() then return end
    if inst.stage == 1 then return end
    local r = inst.stage == 2 and 4 or inst.stage == 3 and 2 or 10
    return FindEntity(inst, r, function(guy)
        return inst.components.combat:CanTarget(guy) and not guy:HasTag('mythTag_rhino')
    end)
end

local function OnAttacked(inst, data)
    inst.components.combat.laststartattacktime = 0 
    if inst.last_spell_time == 0 then
        inst.last_spell_time = 10 + math.random(10)
    end
    if data and data.attacker then
        inst.components.combat:SetTarget(data.attacker)
        inst.components.combat:ShareTarget(data.attacker, 30, function(dude)
            return dude:HasTag("mythTag_rhino") and not dude.components.health:IsDead()
        end, 10)
    end
end

--local SPELL_INTERVAL = 20
local function GetSpellInterval(inst) return inst.prefab == 'rhino3_yellow' and 10 or 20 end
local MAX_SPELL_RUN_TIME = 10 

local function isSpellAvailable(inst)
    if inst.components.health:IsDead()then return false end
    if inst.sg:HasStateTag('dashing') then return false end
    if inst.components.combat.target
    and inst.components.combat.target:IsValid()
    and GetTime() - inst.last_spell_time > GetSpellInterval(inst)
        then
        return true
    end
end

local function ShouldSpellRun(inst)
    if isSpellAvailable(inst) then
        if inst:GetDistanceSqToInst(inst.components.combat.target) > 8* 8 then
            return false
        else
            inst.last_spell_runtime = GetTime()
            return true
        end
    end
end

local function shouldDoSpell(inst)
    if inst.components.health:IsDead()then return false end
    local target = inst.components.combat.target
    if not (target and target:IsValid()) then return false end
    if GetClosestInstWithTag('mythTag_rhinospell', target, 3.5) then return false end --防止法阵重叠
    return isSpellAvailable(inst)
    and (inst:GetDistanceSqToInst(target) > 5 * 5
    or GetTime() - inst.last_spell_runtime > MAX_SPELL_RUN_TIME)
end

local function SpellTask(inst)
    if inst.components.health:IsDead() then
        inst.spelltask:Cancel()
        inst.spelltask = nil
    end
    if inst:CanDoSpell() and not inst.sg:HasStateTag('spelling') then
        inst:PushEvent('dospell')
    end
end

local function ShouldLeave(inst)
    if inst.stage == 1 and inst.components.health:GetPercent() == 1 and GetClock():IsDay() then
        return true
    end
end

local DASH_INTERVAL = 25
local function CanDash(inst)
    return GetTime() - inst.last_dash_time > DASH_INTERVAL
end

local function SetStage(inst, num)
    num = num or inst.stage or 1
    num = num > 3 and 3 or num
    inst.stage = num
    inst.components.health.absorb = num == 2 and 0.1 or num == 3 and 0.25 or 0
    inst.components.combat:AddDamageModifier(num == 2 and 0.5 or num == 3 and 1 or 0)
    local s = num == 2 and 1.15 or num == 3 and 1.3 or 1
    inst.Transform:SetScale(s, s, s)
end

local function CommonRhino()
    local inst = CreateEntity()
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddDynamicShadow()

    inst.DynamicShadow:SetSize(2.5, 1.2)
    inst.Transform:SetSixFaced()
    inst.Transform:SetScale(0.85, 0.85, 1)

    MakeCharacterPhysics(inst, 400, 1)
    inst.physicsradiusoverride = 1

    inst.AnimState:SetBank("rhinodrill")

    inst.AnimState:PlayAnimation("idle_loop", true)

    inst:AddTag("monster")
    inst:AddTag("hostile")
    inst:AddTag("largecreature")
    inst:AddTag("mythTag_rhino")
    

    inst.last_spell_time = 0--GetTime() + math.random(10)
    inst.last_spell_runtime = 0
    inst.last_dash_time = GetTime() + math.random(10)
    inst.CanSpell = isSpellAvailable
    inst.ShouldSpellRun = ShouldSpellRun
    inst.CanDoSpell = shouldDoSpell
    inst.CanDash = CanDash
    inst.spelltask = inst:DoPeriodicTask(0.1, SpellTask)

    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(2250--[[1800]])
    inst.components.health.destroytime = 5

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(150)
    inst.components.combat:SetRange(3.4, 4.2)
    inst.components.combat.playerdamagepercent = .33333
    inst.components.combat.hiteffectsymbol = "body"
    inst.components.combat:SetAttackPeriod(3)
    inst.components.combat:SetRetargetFunction(3, RetargetFn)
    inst.components.combat:SetKeepTargetFunction(KeepTargetFn)
    --inst.components.combat:SetHurtSound("dontstarve_DLC001/creatures/bearger/hurt")
    ----
    inst:AddComponent("lootdropper")

    inst:AddComponent("inspectable")

    inst:AddComponent("knownlocations")

    inst:AddComponent("timer")

    inst:ListenForEvent("attacked", OnAttacked)

    MakeLargeBurnableCharacter(inst, "body")
    --MakeHugeFreezableCharacter(inst, "body")

    inst:AddComponent("sleeper")
    inst.components.sleeper:SetResistance(8)
    inst.components.sleeper.wearofftime = 5
    inst.components.sleeper.sleeptestfn = function() return false end

    inst:AddComponent("locomotor")
    inst.components.locomotor.walkspeed = 2
    inst.components.locomotor.runspeed = 7

    inst:AddComponent('talker')

    inst.SetStage = SetStage
    inst:SetStage(1)

    inst.OnSave = function(inst, data)
        data.stage = inst.stage
        data.desk = inst.desk and inst.desk.GUID 
        return {data.desk}
    end
    inst.OnLoad = function(inst, data) inst.stage = data.stage or 1; inst:SetStage(); end
    inst.OnLoadPostPass = function(inst, ents, data)
        if data and data.desk and ents[data.desk] then
            inst.desk = ents[data.desk].entity
            if inst.desk then
                inst.desk.rhinos[inst] = true
            end
        end
    end

    inst:SetStateGraph("SGrhino3")
    local brain = require("brains/rhino3brain")
    inst:SetBrain(brain)

    inst:ListenForEvent('mythEvent_attackedbyspell', OnAttackedBySpell)
    inst.last_spelled = 0

    inst:ListenForEvent('death', function(inst)
        local x,y,z = inst:GetPosition():Get()
        for _,v in ipairs(TheSim:FindEntities(x,y,z,30,nil,{"INLIMBO"}))do
            if v:HasTag('mythTag_rhino') and not v.components.health:IsDead() then
                v:SetStage((v.stage or 1) + 1)
            end
        end
    end)

    inst.ShouldLeave = ShouldLeave
    inst:ListenForEvent('entitysleep', function(inst)
        if inst:ShouldLeave() then
            inst:Remove()
        end
    end)

    return inst
end

local function RhinoBlue()
    local inst = CommonRhino()
    inst:AddTag('rhino3_blue')

    inst.AnimState:SetBuild('rhinodrill_blue')

    inst.spellprefab = 'myth_rhino_spell_ice'

    inst.components.lootdropper:SetLoot({
        "minotaurhorn",
        "bluegem", "bluegem", "bluegem", 
        "nitre", "nitre", "nitre", "nitre", "nitre", "nitre", 
    })
    inst:RemoveComponent('freezable')

    inst:ListenForEvent('onhitother', function(inst, data)
        local other = data.target
        if other and other.components.freezable then
            other.components.freezable:AddColdness(2)
            other.components.freezable:SpawnShatterFX()
        end
    end)

    return inst
end

local function RhinoRed()
    local inst = CommonRhino()
    inst:AddTag('rhino3_red')

    inst.AnimState:SetBuild('rhinodrill_red')

    inst.spellprefab = 'myth_rhino_spell_fire'

    inst.components.lootdropper:SetLoot({
        "minotaurhorn",
        "redgem", "redgem", "redgem", 
        "charcoal", "charcoal", "charcoal", "charcoal", "charcoal", "charcoal", 
    })
    inst:RemoveComponent('burnable')
    inst.components.health.vulnerabletoheatdamage = false

    inst:ListenForEvent('onhitother', function(inst, data)
        local other = data.target
        if other and other.components.burnable and not other.components.burnable:IsBurning() then
            other.components.burnable:Ignite()
        end
    end)

    MakeLargePropagator(inst)
    inst.components.propagator.decayrate = 0
    inst.components.propagator:StartSpreading()

    local light = inst.entity:AddLight()
    light:Enable(true)
    light:SetRadius(2)
    light:SetFalloff(0.5)
    light:SetIntensity(.75)
    light:SetColour(235/255,121/255,12/255)

    return inst
end

local function RhinoYellow()
    local inst = CommonRhino()
    inst:AddTag('rhino3_yellow')

    inst.AnimState:SetBuild('rhinodrill_yellow')

    inst.spellprefab = 'myth_rhino_spell_hole'

    inst.components.lootdropper:SetLoot({
        "minotaurhorn",
        "yellowgem", "yellowgem", "yellowgem", 
        "sand", "sand", "sand", "sand", "sand", "sand", 
    })

    return inst
end

-- local function sp()
--     local inst = CreateEntity()
--     inst.entity:AddTransform()
    
--     --等位置确定后再执行
--     inst:DoTaskInTime(0, function()

--         local pos = inst:GetPosition()
--         --local point = SpawnPrefab('myth_rhino_point')
--         --inst.point = point
--         --point.Transform:SetPosition(pos:Get())
--         local r1 = SpawnPrefab('rhino3_red')
--         local r2 = SpawnPrefab('rhino3_blue')
--         local r3 = SpawnPrefab('rhino3_yellow')
--         r1.Transform:SetPosition(pos:Get())
--         r2.Transform:SetPosition(pos:Get())
--         r3.Transform:SetPosition(pos:Get())
        
--         --point:Register({r1,r2,r3})
--     end)

--     return inst
-- end


-- local function point()
--     local inst = CreateEntity()
--     inst.entity:AddTransform()

--     inst.tracks = {}

--     inst.Register = function(inst, data)
--         for _, v in ipairs(data) do
--             inst.tracks[v] = true
--             v.pointer = inst
--             inst:ListenForEvent('onremove', function()
--                 inst.tracks[v] = nil
--                 if GetTableSize(inst.tracks) == 0 then
--                     inst:Remove()
--                 end
--             end, v)
--             inst:ListenForEvent('death', function()
--                 for k in pairs(inst.tracks)do
--                     if k:IsValid() and not k.components.health:IsDead() then
--                         k:SetStage((k.stage or 1) + 1)
--                     end
--                 end
--             end, v)
--         end
--     end

--     inst.OnSave = function (inst,data)
--         for k in pairs(inst.tracks)do
--             table.insert(data, k.GUID)
--         end
--         return data
--     end 

--     inst.OnLoadPostPass = function(inst, ents, data)
--         if data then
--             for _,v in pairs(data)do
--                 if ents[v] and ents[v].entity then
--                     inst:Register({ents[v].entity})
--                 end
--             end
--         end
--     end

--     inst:DoPeriodicTask(10, function()
--         local pos_total = Point(0,0,0)
--         local num = 0
--         for k in pairs(inst.tracks)do
--             if k:IsValid() then
--                 pos_total = pos_total + k:GetPosition()
--                 num = num +1
--             else
--                 inst.tracks[k] = nil
--             end
--         end
--         if num > 0 then
--             inst.Transform:SetPosition((pos_total/num):Get())
--         end
--     end)

--     return inst
-- end

--[[
local ent = CreateEntity()
ent.entity:AddTransform()
ent.entity:AddAnimState()
ent.entity:SetParent(ThePlayer.entity)
ent.AnimState:SetBank('wilson')
ThePlayer.AnimState:SetMultColour(0,0,0,0)
SetSkinsOnAnim( ent.AnimState, "wurt", "wurt_shadow", {})
ThePlayer.AnimState = ent.AnimState
ent:ListenForEvent('animover', function()ThePlayer:PushEvent('animover') end)
--]]

local function come()
    local inst = CreateEntity()
    inst.entity:AddTransform()
    inst.SetTarget = function(inst, desk)
        if not (desk and desk:IsValid()) then
            return
        end
        local ori = desk:GetPosition()
        local dist = 30
        for i = 1, 1000 do
            local angle = 2*PI*math.random()
            local offset = Vector3(math.sin(angle)*dist, 0, math.cos(angle)*dist)
            local pos = offset + ori
            if not inst.GetIsOnLand or inst:GetIsOnLand(pos:Get()) then
                for _,v in ipairs({'yellow', 'blue', 'red'})do
                    local a = SpawnPrefab('rhino3_'..v)
                    a.Transform:SetPosition(pos:Get())
                    a.desk = desk
                    desk.rhinos[a] = true
                end
                print('spawn succeeded!')
                return
            end
        end
    end
    inst.persists = false

    return inst
end

return Prefab('rhino3_red', RhinoRed, assets), Prefab('rhino3_yellow', RhinoYellow), Prefab('rhino3_blue', RhinoBlue),
-- Prefab('myth_rhino_spawner', sp), 
-- Prefab('myth_rhino_point', point),
Prefab('myth_rhino_come', come)
