local assets = {
	Asset("ANIM", 'anim/dst_myth/deer_fire_burst_myth.zip'),
	Asset("ANIM", 'anim/dst_myth/deer_fire_charge_myth.zip'),
	Asset("ANIM", 'anim/dst_myth/deer_fire_circle_myth.zip'),
	Asset("ANIM", 'anim/dst_myth/deer_fire_flakes_myth.zip'),
	Asset("ANIM", 'anim/dst_myth/deer_ice_burst_myth.zip'),
	Asset("ANIM", 'anim/dst_myth/deer_ice_charge_myth.zip'),
	Asset("ANIM", 'anim/dst_myth/deer_ice_circle_myth.zip'),
	Asset("ANIM", 'anim/dst_myth/deer_ice_flakes_myth.zip'),
   	Asset("ANIM", "anim/dst_myth/antlion_sinkhole_myth.zip"),
    Asset("ANIM", "anim/dst_myth/sinkhole_spawn_fx_myth.zip"),
}

local makefxat = MK_MOD_UTIL.MakeFxAt --(pos, bank, build, animname, loop, fn)
local pos_0 = Point(0, 0, 0)
local makeonground = function(inst)
	inst.AnimState:SetSortOrder(3)
    inst.AnimState:SetOrientation(ANIM_ORIENTATION.OnGround)
    inst.AnimState:SetLayer(LAYER_BACKGROUND)
end
--bloom?

local function common()
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local snd = inst.entity:AddSoundEmitter()

	inst:AddTag("NOCLICK")
	inst:AddTag("FX")
	inst:AddTag('mythTag_rhinospell')
	inst.persists = false

	return inst
end

local function OnUpdateFade(inst)
    local k
    if inst._fade <= inst._fadeframes then
        inst._fade = math.min(inst._fade + inst._fadeinspeed, inst._fadeframes)
        k = inst._fade / inst._fadeframes
    else
        inst._fade = math.min(inst._fade + inst._fadeoutspeed, inst._fadeframes * 2 + 1)
        k = (inst._fadeframes * 2 + 1 - inst._fade) / inst._fadeframes
    end

    inst.Light:SetIntensity(inst._fadeintensity * k)
    inst.Light:SetRadius(inst._faderadius * k)
    inst.Light:SetFalloff(1 - (1 - inst._fadefalloff) * k)

    inst.Light:Enable(inst._fade > 0 and inst._fade <= inst._fadeframes * 2)

    if inst._fade == inst._fadeframes or inst._fade > inst._fadeframes * 2 then
        inst._fadetask:Cancel()
        inst._fadetask = nil
    end
end

local RANDOM_SEGS = 8
local SEG_ANGLE = 360 / RANDOM_SEGS
local ANGLE_VARIANCE = SEG_ANGLE * 2 / 3
local function GetRandomAngle(inst)
    if inst.angles == nil then
        inst.angles = {}
        local offset = math.random() * 360
        for i = 0, RANDOM_SEGS - 1 do
            table.insert(inst.angles, offset + i * SEG_ANGLE)
        end
    end
    local rnd = math.random()
    rnd = rnd * rnd
    local angle = table.remove(inst.angles, math.max(1, math.ceil(rnd * rnd * RANDOM_SEGS)))
    table.insert(inst.angles, angle)
    return (angle + math.random() * ANGLE_VARIANCE) * DEGREES
end

local function DoBurst(inst, name, x, z, minr, maxr)
    local fx = makefxat(pos_0, 'deer_'..name..'_flakes','deer_'..name..'_flakes_myth', 'pre')
    fx:ListenForEvent('animover', fx.Remove)
    local theta = GetRandomAngle(inst)
    local rad = GetRandomMinMax(minr, maxr)
    fx.Transform:SetPosition(x + rad * math.cos(theta), 0, z + rad * math.sin(theta))
end

local ICE_CIRCLE_RADIUS = 3
local NOTAGS = { "playerghost", "INLIMBO", 'FX' }

local function OnUpdateIceCircle(inst, x, z)
    inst._rad = inst._rad * .98 + ICE_CIRCLE_RADIUS * .02

    for i, v in ipairs(TheSim:FindEntities(x, 0, z, inst._rad, nil, NOTAGS, { "locomotor", "freezable", "fire", "smolder",'rhino3_blue' })) do
        if v:IsValid() and not (v.components.health ~= nil and v.components.health:IsDead()) then            
            --减速?
            if v.components.burnable ~= nil and v.components.fueled == nil then
                v.components.burnable:Extinguish()
            end
            if v:HasTag('mythTag_rhino') and not v.prefab == 'rhino3_blue' then
            	v:PushEvent('mythEvent_attackedbyspell')
            end
            if v.components.freezable ~= nil then
                if not v.components.freezable:IsFrozen() then
                	v.components.freezable:AddColdness(.07, 3)
                end
            end
            if v.components.temperature ~= nil then
                local newtemp = math.max(v.components.temperature.mintemp, -2)
                if newtemp < v.components.temperature:GetCurrent() then
                    v.components.temperature:SetTemperature(newtemp)
                end
            end
        end
    end
end

local function OnInitIceCircle(inst)
    local x, y, z = inst.Transform:GetWorldPosition()
    inst._rad = .25
    inst.task = inst:DoPeriodicTask(0, OnUpdateIceCircle, nil, x, z)
end


local FIRE_CIRCLE_RADIUS = 3.6
local function OnUpdateFireCircle(inst, x, z)
    inst._rad = inst._rad * .9 + FIRE_CIRCLE_RADIUS * .1
    inst.components.propagator.propagaterange = inst._rad
    inst.components.propagator.damagerange = inst._rad

    if inst._rad > 1 then
        inst.burstdelay = (inst.burstdelay or 2) - 1
        if inst.burstdelay < 0 then
            inst.burstdelay = math.random(1, 2)
            DoBurst(inst, 'fire', x, z, math.max(1, inst._rad - 1.5), inst._rad - .25)
        end
    end

    inst._track1 = inst._track2 or {}
    inst._track2 = {}

    local y --dummy
    for i, v in ipairs(TheSim:FindEntities(x, 0, z, inst._rad, nil, NOTAGS, { "_health", "canlight", "freezable" })) do
        if v:IsValid() and not (v.components.health ~= nil and v.components.health:IsDead()) then
            if v:HasTag('mythTag_rhino') and not v.prefab == 'rhino3_red' then
        		v:PushEvent('mythEvent_attackedbyspell')
        	end
            if v.components.freezable ~= nil then
                if v.components.freezable:IsFrozen() then
                    v.components.freezable:Unfreeze()
                elseif v.components.freezable.coldness > 0 then
                    v.components.freezable:AddColdness(-.1)
                end
            end
            if v.components.burnable ~= nil and
                v.components.fueled == nil and
                v.components.health ~= nil then
                if not v.components.burnable:IsBurning() then
                    inst._track2[v] = (inst._track1[v] or 0) + 1
                    if inst._track2[v] > 30 then
                        v.components.burnable:Ignite(true, inst)
                    end
                end
            end
            if v.components.temperature ~= nil then
                local newtemp = math.min(v.components.temperature:GetMax(), 72)
                if newtemp > v.components.temperature:GetCurrent() then
                    v.components.temperature:SetTemperature(newtemp)
                end
            end
        end
    end
end

local function OnInitFireCircle(inst)
    local x, y, z = inst.Transform:GetWorldPosition()
    inst._rad = .25
    inst.task = inst:DoPeriodicTask(0, OnUpdateFireCircle, nil, x, z)
    OnUpdateFireCircle(inst, x, z)
end

local function deer_fire_circle_master_postinit(inst)
    inst.task = inst:DoTaskInTime(0, OnInitFireCircle)

    inst:AddComponent("propagator")
    inst.components.propagator.damages = true
    inst.components.propagator.propagaterange = .25
    inst.components.propagator.damagerange = .25
    inst.components.propagator:StartSpreading()
end

local function ice()
	local inst = common()
	local ground = makefxat(pos_0, 'deer_ice_circle', 'deer_ice_circle_myth', nil, nil, makeonground)
	local spawnfx = makefxat(pos_0, 'deer_ice_burst', 'deer_ice_burst_myth') --loop
	inst:AddChild(ground)
	inst:AddChild(spawnfx)

	local light = inst.entity:AddLight()
	inst._fade = 0
	inst._fadeframes = 30
	inst._faderadius = 3
	inst._fadefalloff = 0.9
	inst._fadeintensity = 0.8
	inst._fadeinspeed = 1
    inst._fadeoutspeed = 2
	light:SetColour(60/255, 120/255, 255/255)
    light:SetRadius(inst._faderadius)
    light:SetFalloff(inst._fadefalloff)
    light:SetIntensity(inst._fadeintensity)
    light:Enable(false)

    ground.AnimState:PlayAnimation('pre')
    spawnfx.AnimState:PlayAnimation('loop', true)

    inst._fadetask = inst:DoPeriodicTask(0, OnUpdateFade)
    inst:DoTaskInTime(0, OnInitIceCircle)
    

    inst:DoTaskInTime(10, function()
    	inst._fadeframes = 0
    	inst._faderadius = 0
    	inst._fadetask = inst:DoPeriodicTask(0, OnUpdateFade)
    	spawnfx:AddComponent('colourtweener')
    	spawnfx.components.colourtweener:StartTween( {1,1,1,0}, 0.2, spawnfx.Remove)
    	ground:AddComponent('colourtweener')
    	ground.components.colourtweener:StartTween( {1,1,1,0}, 1,  ground.Remove)
    end)

    inst:DoTaskInTime(11.033, inst.Remove)

    return inst
end

local function fire()
	local inst = common()
	local ground = makefxat(pos_0, 'deer_fire_circle', 'deer_fire_circle_myth', nil, nil, makeonground)
	local spawnfx = makefxat(pos_0, 'deer_fire_burst', 'deer_fire_burst_myth') --loop
	inst:AddChild(ground)
	inst:AddChild(spawnfx)

	local light = inst.entity:AddLight()
	inst._fade = 0
	inst._fadeframes = 30
	inst._faderadius = 3
	inst._fadefalloff = 0.9
	inst._fadeintensity = 0.8
	inst._fadeinspeed = 1
    inst._fadeoutspeed = 2
	light:SetColour(220/255, 100/255, 0/255)
    light:SetRadius(inst._faderadius)
    light:SetFalloff(inst._fadefalloff)
    light:SetIntensity(inst._fadeintensity)
    light:Enable(false)

    ground.AnimState:PlayAnimation('pre')
    spawnfx.AnimState:PlayAnimation('loop', true)

    inst._fadetask = inst:DoPeriodicTask(0, OnUpdateFade)
    inst.task = inst:DoTaskInTime(0, OnInitFireCircle)

    inst:AddComponent("propagator")
    inst.components.propagator.damages = true
    inst.components.propagator.propagaterange = .25
    inst.components.propagator.damagerange = .25
    inst.components.propagator:StartSpreading()
    
    inst:DoTaskInTime(6, function()
    	inst.burstdelay = 100
    end)

    inst:DoTaskInTime(9.4, function()
    	inst._fadeframes = 0
    	inst._faderadius = 0
    	inst._fadetask = inst:DoPeriodicTask(0, OnUpdateFade)
    	spawnfx:AddComponent('colourtweener')
    	spawnfx.components.colourtweener:StartTween( {1,1,1,0}, 0.2, spawnfx.Remove)
    	ground:AddComponent('colourtweener')
    	ground.components.colourtweener:StartTween( {1,1,1,0}, 1,  ground.Remove)
    end)

    inst:DoTaskInTime(10.5, inst.Remove)

    return inst
end

local NUM_CRACKING_STAGES = 3
local COLLAPSE_STAGE_DURATION = 1

local function UpdateOverrideSymbols(inst, state)
    if state == NUM_CRACKING_STAGES then
        inst.AnimState:ClearOverrideSymbol("cracks1")
        if inst.components.unevenground ~= nil then
            inst.components.unevenground:Enable()
        end
    else
        inst.AnimState:OverrideSymbol("cracks1", "antlion_sinkhole_myth", "cracks_pre"..tostring(state))
        if inst.components.unevenground ~= nil then
            inst.components.unevenground:Disable()
        end
    end
end

local function SpawnFx(inst, stage, scale)
    local theta = math.random() * PI * 2
    local num = 7
    local radius = 1.6
    local dtheta = 2 * PI / num
    local x, y, z = inst.Transform:GetWorldPosition()
    local fx = makefxat(Point(x, 0, z), 'sinkhole_spawn_fx', 'sinkhole_spawn_fx_myth', 'idle'..math.random(3))
    fx:ListenForEvent('animover', fx.Remove)
    for i = 1, num do
        local dust = makefxat(Point(0, 0, 0), 'sinkhole_spawn_fx', 'sinkhole_spawn_fx_myth', 'idle'..math.random(3))
    	dust:ListenForEvent('animover', dust.Remove)
        dust.Transform:SetPosition(x + math.cos(theta) * radius * (1 + math.random() * .1), 0, z - math.sin(theta) * radius * (1 + math.random() * .1))
        local s = scale + math.random() * .2
        dust.Transform:SetScale(i % 2 == 0 and -s or s, s, s)
        theta = theta + dtheta
    end
    --inst.SoundEmitter:PlaySoundWithParams("dontstarve/creatures/together/antlion/sfx/ground_break", { size = math.pow(stage / NUM_CRACKING_STAGES, 2) })
end

local COLLAPSIBLE_WORK_ACTIONS =
{
    CHOP = true,
    DIG = true,
    HAMMER = true,
    MINE = true,
}
local NON_COLLAPSIBLE_TAGS = { "flying", "bird", "ghost", "playerghost", "FX", "NOCLICK", "DECOR", "INLIMBO" }

local function SmallLaunch(inst, launcher, basespeed)
    local hp = inst:GetPosition()
    local pt = launcher:GetPosition()
    local vel = (hp - pt):GetNormalized()
    local speed = basespeed * .5 + math.random()
    local angle = math.atan2(vel.z, vel.x) + (math.random() * 20 - 10) * DEGREES
    inst.Physics:Teleport(hp.x, .1, hp.z)
    inst.Physics:SetVel(math.cos(angle) * speed, 3 * speed + math.random(), math.sin(angle) * speed)
end

local function donextcollapse(inst)
    inst.collapsestage = inst.collapsestage + 1
    local isfinalstage = inst.collapsestage >= NUM_CRACKING_STAGES

    if isfinalstage then
        inst.collapsetask:Cancel()
        inst.collapsetask = nil
        inst:RemoveTag("scarytoprey")
        local player = GetClosestInstWithTag("player", inst, 30)
    	if player then
        	player.components.playercontroller:ShakeCamera(inst, "FULL", 0.1, 0.03, 0.5, 30)
    	end
        --ShakeAllCameras(CAMERASHAKE.FULL, COLLAPSE_STAGE_DURATION, .03, .15, inst, TUNING.ANTLION_SINKHOLE.RADIUS*6)
    else
    	local player = GetClosestInstWithTag("player", inst, 30)
    	if player then
        	player.components.playercontroller:ShakeCamera(inst, "FULL", 0.1, 0.015, 0.5, 30)
    	end
        --ShakeAllCameras(CAMERASHAKE.FULL, COLLAPSE_STAGE_DURATION, .015, .15, inst, TUNING.ANTLION_SINKHOLE.RADIUS*4)
    end

    UpdateOverrideSymbols(inst, inst.collapsestage)

    SpawnFx(inst, inst.collapsestage, .8)

    local x, y, z = inst.Transform:GetWorldPosition()
    local ents = TheSim:FindEntities(x, 0, z, 2.5, nil, NON_COLLAPSIBLE_TAGS)
    for i, v in ipairs(ents) do
        if v:IsValid() then
            local isworkable = false
            if v.components.workable ~= nil then
                local work_action = v.components.workable:GetWorkAction()
                isworkable = (
                    (work_action == nil and v:HasTag("NPC_workable")) or
                    (v.components.workable.workable and work_action ~= nil and COLLAPSIBLE_WORK_ACTIONS[work_action.id])
                )
            end
            if isworkable then
                if isfinalstage then
                    v.components.workable:Destroy(inst)
                    if v:IsValid() and v:HasTag("stump") then
                        v:Remove()
                    end
                else
                    v.components.workable:WorkedBy(inst, 1)
                end
            elseif v.components.pickable ~= nil
                and v.components.pickable:CanBePicked()
                and not v:HasTag("intense") then
                local num = v.components.pickable.numtoharvest or 1
                local product = v.components.pickable.product
                local x1, y1, z1 = v.Transform:GetWorldPosition()
                v.components.pickable:Pick(inst) -- only calling this to trigger callbacks on the object
                if product ~= nil and num > 0 then
                    for i = 1, num do
                        SpawnPrefab(product).Transform:SetPosition(x1, 0, z1)
                    end
                end
            elseif v.components.combat ~= nil
                and v.components.health ~= nil
                and not v.components.health:IsDead() then
                if v:HasTag('mythTag_rhino') and not v.prefab == 'rhino3_yellow' then
                	v:PushEvent('mythEvent_attackedbyspell')
                end
                if isfinalstage and v.components.locomotor == nil and not v:HasTag('pugalisk') then
                    v.components.health:Kill()
                elseif v.components.combat:CanBeAttacked() then
                    if v.prefab ~= 'rhino3_yellow' then
                    	v.components.combat:GetAttacked(inst, 20)
                    end
                    
                end
            end
        end
    end
    local totoss = TheSim:FindEntities(x, 0, z, 2.5, nil, { "locomotor", "INLIMBO" })
    for i, v in ipairs(totoss) do
    	if v.components.inventoryitem then
	        if v.components.mine ~= nil then
	            v.components.mine:Deactivate()
	        end
	        if not v.components.inventoryitem.nobounce and v.Physics ~= nil and v.Physics:IsActive() then
	            SmallLaunch(v, inst, 1.5)
	        end
	    end
    end
end

local function onstartcollapse(inst)
    inst.collapsestage = 0

    inst:AddTag("scarytoprey")

    inst.collapsetask = inst:DoPeriodicTask(COLLAPSE_STAGE_DURATION, donextcollapse)
    donextcollapse(inst)
end

local function updateslowdowners(inst)

    local ground = GetWorld()
    local x,y,z = inst.Transform:GetWorldPosition() 
    local slowdowns = TheSim:FindEntities(x,y,z, 2.5, {"locomotor"})
    local tempSlowedObjects = {}

    for i=#slowdowns,1,-1 do
        if not slowdowns[i].sg or not slowdowns[i].sg:HasStateTag("moving") then
            table.remove(slowdowns,i)
        end            
    end

    for i,slowinst in ipairs(slowdowns)do
        if not slowinst.slowing_objects then
            slowinst.slowing_objects  = {}
        end

        slowinst.slowing_objects[inst] = true                    

        tempSlowedObjects[slowinst] = true
    end

    for i,slowedinst in pairs( inst.slowed_objects ) do
        if not tempSlowedObjects[i] then
            i.slowing_objects[inst] = nil
        end       
    end

    inst.slowed_objects = tempSlowedObjects

    inst:DoTaskInTime(2/30,function(inst) updateslowdowners(inst) end)
end

local function onRemove(inst)
    for i,slowedinst in pairs( inst.slowed_objects ) do
        i.slowing_objects[inst] = nil      
    end
end

local function fn()
    local inst = CreateEntity()
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()

    inst.AnimState:SetBank("sinkhole")
    inst.AnimState:SetBuild("antlion_sinkhole_myth")
    inst.AnimState:PlayAnimation("idle")
    inst.AnimState:SetOrientation(ANIM_ORIENTATION.OnGround)
    inst.AnimState:SetLayer(LAYER_BACKGROUND)
    inst.AnimState:SetSortOrder(2)

    inst.Transform:SetEightFaced()

    inst:AddTag("NOCLICK")

    --inst:AddComponent("unevenground")
    --inst.components.unevenground.radius = 3

    inst:AddComponent('colourtweener')
    inst:DoTaskInTime(20, function()inst.components.colourtweener:StartTween({1,1,1,0}, 1, inst.Remove) end)

    inst:ListenForEvent("startcollapse", onstartcollapse)

    inst.slowed_objects = {}
    inst.OnRemoveEntity = onRemove
    inst.slowdowners = {}
    inst:AddTag("locomotor_slowdown")
    inst:AddTag('mythTag_rhinospell')
    inst:DoTaskInTime(0.1, updateslowdowners)

    return inst
end

function dbc() c_spawn'myth_rhino_spell_hole' end
return Prefab('myth_rhino_spell_ice', ice, assets),
	Prefab('myth_rhino_spell_fire', fire),
 	Prefab("myth_rhino_spell_hole", fn)
