local assets = {
	Asset("ANIM", 'anim/myth_rhino_desk.zip'),
}

local function _isvalid(desk, inst)
	if not (inst and inst:IsValid()) then
		return 
	end
	if not desk.peach and inst.prefab == 'peach' then
		return true
	end
	if not desk.light_on and inst.components.fuel and inst.components.fuel.fueltype == 'BURNABLE' then
		return true
	end
	--return inst.prefab == 'peach' or (inst.components.fuel and inst.components.fuel.fueltype == "BURNABLE")
end

local L = MK_MOD_LANGUAGE_SETTING

STRINGS.NAMES.MYTH_RHINO_DESK = L and "Shabby altar" or '破旧的供桌' 
STRINGS.CHARACTERS.GENERIC.DESCRIBE.MYTH_RHINO_DESK = L and "Should I offer a devotional tribute?" or "我是不是该虔诚地献上贡品？"
STRINGS.CHARACTERS.MONKEY_KING.DESCRIBE.MYTH_RHINO_DESK = '这石头供桌什么来头？'
STRINGS.CHARACTERS.NEZA.DESCRIBE.MYTH_RHINO_DESK = '这桌子是供奉哪位仙长的？'
STRINGS.CHARACTERS.WHITE_BONE.MYTH_RHINO_DESK = '好呀，这倒也是一计~'

local function LightOn(inst)
	if inst.light_on then return end

	inst.light_on = true
	if not inst.components.timer:TimerExists('light') then
		inst.components.timer:StartTimer('light', 180)
	end
	for i = 1, 4 do
		inst.AnimState:OverrideSymbol('fire'..i, "myth_rhino_desk", "fire4")
		local light = CreateEntity()
		light.persists = false
		light:AddTag('FX')
		light:AddTag('fx')
		local trans = light.entity:AddTransform()
		trans:SetScale(0.3,0.3,0.3)
		local anim = light.entity:AddAnimState()
		anim:SetBank('campfire_fire')
		anim:SetBuild('campfire_fire')
		anim:PlayAnimation('level1', true)
		anim:SetMultColour(1, 0.6, 1, 1)
		anim:SetLightOverride(1)
		anim:SetBloomEffectHandle( "shaders/anim.ksh" )
    	anim:SetRayTestOnBB(true)
    	anim:SetFinalOffset(-1)

		light.entity:AddLight()
		light.Light:SetRadius(1)
		light.Light:Enable(true)
		light.Light:SetIntensity(.8)
		light.Light:SetFalloff(.5)
		light.Light:SetColour(200/255, 126/255, 30/255)

		local f = light.entity:AddFollower()
		f:FollowSymbol(inst.GUID, 'fire'..i, 0, 0, 0)
		inst.lightfx[light] = true
	end
end

local function LightOff(inst)
	inst.light_on = false
	for i = 1, 4 do
		inst.AnimState:OverrideSymbol('fire'..i, 'a', 'b')
	end
	for k in pairs(inst.lightfx)do
		if k:IsValid() then
			k:Remove()
		end
	end
	inst.lightfx = {}
end

local function AddPeach(inst)
	if not inst.peach then
		inst.peach = true
		inst.AnimState:OverrideSymbol('swap1', 'myth_rhino_desk', 'swap1')
		if not inst.components.timer:TimerExists('peach') then
			inst.components.timer:StartTimer('peach', 600)
		end
	end
end

local function RemovePeach(inst)
	inst.peach = false
	inst.AnimState:OverrideSymbol('swap1', 'a', 'b')
end

local function RhinoFn(inst)
	print('call -> RhinoFn')
	if not inst.peach or not inst.light_on then return end 
	if not GetClock():IsNight() then return end
	if GetSeasonManager().IsSpring and not GetSeasonManager():IsSpring() then return  end
	--检查是否有绑定的实体
	local has = false
	for k in pairs(inst.rhinos)do
		if not k:IsValid() or k.components.health:IsDead() then
			inst.rhinos[k] = nil
		else
			has = true
		end
	end
	if has then return end
	print('no inst')

	if inst.components.timer:TimerExists('cd') then
		return
	else
		print('spawn!')
		inst.components.timer:StartTimer('cd', 1*480)
		local rhino = SpawnPrefab('myth_rhino_come')
		rhino:SetTarget(inst)
	end
end

local function GetItem(inst, item)
	if _isvalid(inst, item) then
		if item.components.stackable then
			item = item.components.stackable:Get()
		end
		item:Remove()
		if item.prefab == 'peach' then
			AddPeach(inst)
		else
			LightOn(inst)
		end
		if inst.light_on and inst.peach then
			RhinoFn(inst)
		end
		return true
	end
	return false
end

local function OnSave(inst, data)
	data.light_on = inst.light_on
	data.peach = inst.peach
end

local function OnLoad(inst, data)
	if not data then return end
	if data.peach then AddPeach(inst) end
	if data.light_on then LightOn(inst) end
end

local function OnDay(inst)
	LightOff(inst)
	RemovePeach(inst)
end

local function OnTimerDone(inst, data)
	if data and data.name == 'peach' then
		RemovePeach(inst)
	end
	if data and data.name == 'light' then
		LightOff(inst)
	end
end

local function fn()
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	--
	MakeObstaclePhysics(inst, 1.2)
	trans:SetScale(1.2,1.2,1.2)

	anim:SetBank('myth_rhino_desk')
	anim:SetBuild('myth_rhino_desk')
	anim:PlayAnimation('idle')

	inst:AddComponent('inspectable')

	inst:AddComponent('timer')

	inst.lightfx = {}
	inst.rhinos = {}
	inst.peach = false
	inst.light_on = false

	inst.OnSave = OnSave
	inst.OnLoad = OnLoad

	LightOff(inst)
	RemovePeach(inst)

	inst:ListenForEvent('onremove', LightOff)

	inst:ListenForEvent('timerdone', OnTimerDone)

	inst:ListenForEvent('nighttime', function() RhinoFn(inst) end, GetWorld())

	inst:ListenForEvent('daytime', function()OnDay(inst) end, GetWorld())

	inst.GetItem = GetItem

	return inst
end

return Prefab('myth_rhino_desk', fn, assets)
