
local MakePlayerCharacter = require "prefabs/player_common"
local easing = require "easing"

local assets = {
    Asset("SCRIPT", "scripts/prefabs/player_common.lua"),
    Asset("ANIM", "anim/neza.zip"),

	Asset("ANIM", "anim/dst_myth/player_superjump_myth.zip"),
	Asset("ANIM", "anim/nz36.zip"),

	Asset("SOUNDPACKAGE", "sound/mythwords_bgm.fev"),
    Asset("SOUND", "sound/mythwords_bgm.fsb"),
}
if softresolvefilepath('anim/walani.zip') and softresolvefilepath('anim/walani_paddle.zip') then
	table.insert(assets,Asset("ANIM", "anim/walani.zip"))
	table.insert(assets,Asset("ANIM", "anim/walani_paddle.zip"))
end
if softresolvefilepath('anim/player_wormwood.zip') then
	table.insert(assets,Asset("ANIM", "anim/player_wormwood.zip"))
end

local prefabs = {
	"mk_jgb",
	"mk_striker",
	"peach",
	"mk_striker_fx"
}

local start_inv = {
	'nz_lance',
	'nz_wheel',
	'nz_damask',
	'nz_ring',
}

local function HealFromWater(inst)
	if not inst.components.moisture then return end
	local moisture = inst.components.moisture.moisture
	local heal_per_minute = 0
	if moisture >= 95 then
		heal_per_minute = 60
	elseif moisture >= 75 then
		heal_per_minute = 30
	elseif moisture >= 50 then
		heal_per_minute = 15
	end
	if heal_per_minute > 0 and not inst.components.health:IsDead() then
		inst.components.health:DoDelta(heal_per_minute/60,true)
	end
end

local function OnKill(inst,data)
	local victim = data.victim
	if victim and not victim:HasTag('monster') and not victim:HasTag('hostile') and not victim:HasTag('wall') then
		inst.components.sanity:DoDelta(-10)
	end
	if victim and victim:HasTag('monster') and math.random()< 0.25 then
		inst.components.sanity:DoDelta(15)
	end
	if victim and victim:HasTag('epic') then
		inst.components.sanity:DoDelta(30)
	end
end

local function sanityfn(inst)
    local x, y, z = inst.Transform:GetWorldPosition() 
    local delta = 0
    local sm = GetSeasonManager()
    local is_raining = (sm.IsHeavyRaining and sm:IsHeavyRaining()) or (sm.IsHailing and sm:IsHailing())
    local raindelta = is_raining and -10/60 or 0
	local moisture_delta = easing.inSine(inst.components.moisture.moisture, 0, TUNING.MOISTURE_SANITY_PENALTY_MAX, inst.components.moisture.moistureclamp.max)
	return -moisture_delta + raindelta
end

local function IsLilyPad(food)
	return food and food.prefab == 'lotus_flower' or food.prefab == 'lotus_flower_cooked'
end

local function gethungermultfn(com,food,value)
	if IsLilyPad(food) then
		return 0
	else
		return 1
	end
end

local fn = function(inst) 
	TUNING.WET_ITEM_DAPPERNESS = 0 --在此心疼一波班花
	inst.MiniMapEntity:SetIcon( "neza.tex" )
	
	inst:AddTag("neza")
	inst.soundsname = "sfx"
	inst.talker_path_override = 'Nezha/'

	inst.AnimState:SetBuild('neza')
	inst.AnimState:OverrideSymbol('nz_head','nz36','nz_head')
	inst.AnimState:OverrideSymbol('nz_hand','nz36','nz_hand')

	--inst.talker_path_override = "dontstarve_DLC002/characters/"

	inst.components.health:SetMaxHealth(150)
	inst.components.hunger:SetMax(150)
	inst.components.sanity:SetMax(150)
	
	inst.components.sanity.custom_rate_fn = sanityfn
	inst.components.sanity.neg_aura_mult = .5

	--inst.components.eater:SetOnEatFn(oneat)
	inst.components.eater.gethungermultfn = gethungermultfn
	inst.components.eater.getsanitymultfn = gethungermultfn
	inst.components.eater.gethealthmultfn = gethungermultfn

	inst.components.inventory:GuaranteeItems(start_inv)

	inst:ListenForEvent('killed',OnKill)

	inst:DoPeriodicTask(1, HealFromWater)
end
function ppp()
local x,y,z = GetPlayer():GetPosition():Get()
for _,v in ipairs(TheSim:FindEntities(x,y,z,4))do
	print('===========')
	print(v)
	print(v.prefab)
end
end

return MakePlayerCharacter("neza", prefabs, assets,fn, start_inv)

