--乾坤圈
local assets=
{
    Asset("ANIM", "anim/nz_ring.zip"),
}

local function OnEquip(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "nz_ring", "swap_item")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal")
end

local function OnHitOwner(inst)
    inst.components.floatable:SetAnimationFromPosition()
    if inst.shadow_task then
        inst.shadow_task:Cancel()
        inst.shadow_task = nil
    end
end

local function OnUnequip(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 
end

local function OnPutInInventory(inst)
    inst.hitcount = 0
    inst.hittargets = {}
    if inst.shadow_task then
        inst.shadow_task:Cancel()
        inst.shadow_task = nil
    end
end

local function OnThrown(inst, owner, target)
    if target ~= owner then
        owner.SoundEmitter:PlaySound("dontstarve/wilson/boomerang_throw")
    end
    inst.AnimState:PlayAnimation("flying", true)
    if not inst.shadow_task then
        inst.shadow_task = inst:DoPeriodicTask(0, function()
            local fx = SpawnPrefab('nz_ring_shadow')
            fx.Transform:SetPosition(inst:GetPosition():Get())
        end)
    end
end

local function FindTarget(inst) --获取附近目标
    if inst.hitcount > 10 then return nil end

    local radius = 6
    local x, y, z = inst:GetPosition():Get()
    local ents = TheSim:FindEntities(x, 0, z, 6, {'combat'}, {'FX', 'NOCLICK', 'INLIMBO', 'DECOR', 'hiding', 'player'})
    for k, v in pairs(ents)do
        if not inst.hittargets[v] then
            if v.components.health and not v.components.health:IsDead() then
                return v
            end
        end
    end
end

local function OnCaught(inst, catcher)
    if catcher then
        if catcher.components.inventory then
            if inst.components.equippable and not catcher.components.inventory:GetEquippedItem(inst.components.equippable.equipslot) then
                catcher.components.inventory:Equip(inst)
            else
                catcher.components.inventory:GiveItem(inst)
            end
            catcher:PushEvent("catch")
        end
    end
end

local function ReturnToOwner(inst, owner)
    if owner then
        owner.SoundEmitter:PlaySound("dontstarve/wilson/boomerang_return")
        inst.components.projectile:Throw(owner, owner)
    end
end

local function Bounce(inst, owner, target)
    inst.components.projectile:Throw(owner, target)
end

local function OnHit(inst, owner, target)
    inst.hittargets[target or 1] = true
    inst.hitcount = inst.hitcount + 1

    if owner == target then  --击中玩家
        OnHitOwner(inst) 
    else
        local target = FindTarget(inst)
        if target then
            Bounce(inst, owner, target) --攻击下一个目标
        else
            ReturnToOwner(inst, owner)  --返回玩家
        end
    end
    local impactfx = SpawnPrefab("impact")
    if impactfx then
        local follower = impactfx.entity:AddFollower()
        follower:FollowSymbol(target.GUID, target.components.combat.hiteffectsymbol, 0, 0, 0 )
        impactfx:FacePoint(inst:GetPosition())
    end
end

local function getdamagefn(inst)
    if inst.hitcount > 5 then
        return 25
    else
        return math.max(Remap(inst.hitcount, 0, 5, 50, 25))
    end
end


local function fn(Sim)
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    MakeInventoryPhysics(inst)
    RemovePhysicsColliders(inst)
    MakeInventoryFloatable(inst, 'idle_water', 'idle')

    inst:AddTag('irreplaceable')

    inst.hitcount = 0
    inst.hittargets = {}

    inst.entity:AddMiniMapEntity()
    inst.MiniMapEntity:SetIcon("nz_ring.tex")
    
    anim:SetBank("nz_ring")
    anim:SetBuild("nz_ring")
    anim:PlayAnimation("idle")
    anim:SetRayTestOnBB(true)
    
    inst:AddTag('irreplaceable')
    inst:AddTag("sharp")
    inst:AddTag("pointy")
    inst:AddTag('projectile')
    inst:AddTag('thrown')
    
    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(50)
    inst.components.weapon:SetRange(12, 14)
    inst.components.weapon.getdamagefn = getdamagefn

    inst:AddComponent("inspectable")
    
    inst:AddComponent("projectile")
    inst.components.projectile:SetSpeed(20) --飞行速度
    inst.components.projectile:SetCanCatch(true)
    inst.components.projectile:SetOnThrownFn(OnThrown)
    inst.components.projectile:SetOnHitFn(OnHit)
    inst.components.projectile:SetOnMissFn(ReturnToOwner)
    inst.components.projectile:SetOnCaughtFn(OnCaught)
    inst.components.projectile:SetLaunchOffset(Vector3(0, 0.2, 0))

    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.atlasname = "images/monkey_king_item.xml"
    inst:ListenForEvent('onputininventory', OnPutInInventory)
    
    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(OnEquip)
    inst.components.equippable:SetOnUnequip(OnUnequip)

    return inst
end

local function shadow(Sim)
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()

    anim:SetBank('nz_ring')
    anim:SetBuild('nz_ring')
    anim:SetMultColour(1,1,1,0.3)
    anim:SetPercent('flying', math.random())

    inst:AddComponent('colourtweener')
    inst.components.colourtweener:StartTween({1,1,1,0}, 0.4, inst.Remove)

    inst:AddTag('FX')
    inst:AddTag('NOCLICK')
    inst.persists = false

    return inst
end

return Prefab('nz_ring_shadow', shadow, assets), Prefab("nz_ring", fn, assets)
