local peach_assets =
{
    Asset("ANIM", "anim/peach_sapling.zip"),
}

local peach_prefabs =
{
   "peachtree_short"
}

local function growtree(inst)
    local tree = SpawnPrefab(inst.growprefab)
    if tree then
        tree.Transform:SetPosition(inst.Transform:GetWorldPosition())
        tree:growfromseed()
        inst:Remove()
    end
end

local function stopgrowing(inst)
    if inst.grow_task then
        inst.grow_task:Cancel()
        inst.grow_time = nil
    end
end

local function startgrowing(inst)
    if not inst.grow_task then
        inst.grow_task = inst:DoTaskInTime(480*6,growtree)
        inst.grow_time = GetTime() + 480*6
    end
end

local function ontimerdone(inst, data)
    if data.name == "grow" then
        growtree(inst)
    end
end

local function digup(inst, digger)
    inst.components.lootdropper:DropLoot()
    inst:Remove()
end

local function sapling_fn(build, anim, growprefab, tag, fireproof, overrideloot)
    local function fn()
        local inst = CreateEntity()

        inst.entity:AddTransform()
        inst.entity:AddAnimState()
        inst.entity:AddSoundEmitter()  

        inst.AnimState:SetBank(build)
        inst.AnimState:SetBuild(build)
        inst.AnimState:PlayAnimation(anim)

        inst:AddTag(tag)

        inst.growprefab = growprefab
        inst.StartGrowing = startgrowing

        inst:AddComponent("inspectable")

        inst:AddComponent("lootdropper")
        inst.components.lootdropper:SetLoot(overrideloot or {"twigs"})

        inst:AddComponent("workable")
        inst.components.workable:SetWorkAction(ACTIONS.DIG)
        inst.components.workable:SetOnFinishCallback(digup)
        inst.components.workable:SetWorkLeft(1)

        if not fireproof then
            MakeSmallBurnable(inst, TUNING.SMALL_BURNTIME)
            inst:ListenForEvent("onignite", stopgrowing)
            inst:ListenForEvent("onextinguish", startgrowing)
            MakeSmallPropagator(inst)

            --MakeHauntableIgnite(inst)
        else
            --MakeHauntableWork(inst)
        end

        --for DEBUG
        --[[
        inst.entity:AddLabel()
        inst.Label:SetFontSize(20)
        inst.Label:SetFont(DEFAULTFONT)
        inst.Label:SetPos(0,0,0)
        inst.Label:SetColour(.5,1,.5)
        inst.Label:Enable(true)
        --inst.Label:SetText(num)
        
        inst:DoPeriodicTask(1,function()
            local num = math.max(0, (inst.grow_time or 0 ) - GetTime())
            num = num and math.floor(num) or "ERROR!"
            inst.Label:SetText(tostring(num))
        end)
        --]]

        inst:DoPeriodicTask(1,function()
            if inst.grow_time and GetTime()>inst.grow_time then
                growtree(inst)
            end
        end)

        inst.OnSave = function(inst,data)
            if inst.grow_time then
                data.time = inst.grow_time - GetTime()
            end
        end

        inst.OnLoad = function(inst,data)
            if data.time then
                inst.grow_time = data.time
                inst.grow_task = inst:DoTaskInTime(inst.grow_time, growtree)
            end
        end

        inst.OnLongUpdate = function(inst,dt)
            if inst.grow_time and inst.grow_task then
                inst.grow_task:Cancel()
                local time = math.max(inst.grow_time - GetTime()-dt, 0)
                inst.grow_task = inst:DoTaskInTime(time, growtree)
                inst.grow_time = GetTime()+time
            end
        end


        return inst
    end
    return fn
end

return Prefab("peach_sapling", sapling_fn("peach_sapling", "plant", "peachtree_short", "peachtree"), peach_assets, peach_prefabs)
