local assets =
{
	Asset("ANIM", "anim/tree_peach_tall.zip"),
	Asset("ANIM", "anim/tree_peach_normal.zip"),
	Asset("ANIM", "anim/tree_peach_short.zip"),
	
	Asset("ANIM", "anim/tree_jungle_build.zip"),
	
	Asset("ANIM", "anim/tree_jungle_normal.zip"),
	Asset("ANIM", "anim/tree_jungle_short.zip"),
	Asset("ANIM", "anim/tree_jungle_tall.zip"),
	
    Asset("SOUND", "sound/forest.fsb"),
}
local function DebugTime(inst)
	do return end
	--for DEBUG
        ---[[
        inst.entity:AddLabel()
        inst.Label:SetFontSize(30)
        inst.Label:SetFont(DEFAULTFONT)
        inst.Label:SetPos(0,0,0)
        inst.Label:SetColour(.5,1,.5)
        inst.Label:Enable(true)
        --inst.Label:SetText(num)
        
        inst:DoPeriodicTask(1,function()
        	if inst:HasTag("burnt")then
        		inst.Label:Enable(false)
        		return
        	end
            local num = inst.components.growable.task and 
            inst.components.growable.task:NextTime() - GetTime()
            --GetTimeLeft('grow') 
            num = num and math.floor(num) or "ERROR"
            inst.Label:SetText(tostring(num))
        end)
        --]]
end

local prefabs =
{
	"log",
	"peach",
	"charcoal",
	--"peach_chop",
	--"peach_fall",
}

local builds =
{
	normal = {
		file="tree_jungle_build",
		prefab_name="peachtree",
		normal_loot = {"log", "log"},
		short_loot = {"log"},
		tall_loot = {"log", "log", "log", "peach", "peach" , "peach"},		
	}	
}

local function makeanims(stage)
	return {
		idle="idle_"..stage,
		sway1="sway1_loop_"..stage,
		sway2="sway2_loop_"..stage,
		chop="chop_"..stage,
		fallleft="fallleft_"..stage,
		fallright="fallright_"..stage,
		stump="stump_"..stage,
		burning="burning_loop_"..stage,
		burnt="burnt_"..stage,
		chop_burnt="chop_burnt_"..stage,
		idle_chop_burnt="idle_chop_burnt_"..stage,
		blown1="blown_loop_"..stage.."1",
		blown2="blown_loop_"..stage.."2",
		blown_pre="blown_pre_"..stage,
		blown_pst="blown_pst_"..stage
	}
end

local short_anims = makeanims("short")
local tall_anims = makeanims("tall")
local normal_anims = makeanims("normal")
local old_anims =
{
	idle="idle_old",
	sway1="idle_old",
	sway2="idle_old",
	chop="chop_old",
	fallleft="chop_old",
	fallright="chop_old",
	stump="stump_old",
	burning="idle_olds",
	burnt="burnt_tall",
	chop_burnt="chop_burnt_tall",
	idle_chop_burnt="idle_chop_burnt_tall",
	blown="blown_loop",
	blown_pre="blown_pre",
	blown_pst="blown_pst"
}

local function dig_up_stump(inst, chopper)
	inst:Remove()
	inst.components.lootdropper:SpawnLootPrefab("log")
end

local function chop_down_burnt_tree(inst, chopper)
	inst:RemoveComponent("workable")
	inst.SoundEmitter:PlaySound("dontstarve/forest/treeCrumble")
    if not (chopper ~= nil and chopper:HasTag("playerghost")) then
        inst.SoundEmitter:PlaySound("dontstarve/wilson/use_axe_tree")
    end
	inst.AnimState:PlayAnimation(inst.anims.chop_burnt)
	RemovePhysicsColliders(inst)
	inst:ListenForEvent("animover", inst.Remove)
	inst.components.lootdropper:SpawnLootPrefab("charcoal")
	inst.components.lootdropper:DropLoot()
	if inst.pineconetask then
		inst.pineconetask:Cancel()
		inst.pineconetask = nil
	end
end

local function GetBuild(inst)
	local build = builds[inst.build]
	if build == nil then
		return builds["normal"]
	end
	return build
end

local function OnBurnt(inst, imm)
    local function changes()
        if inst.components.burnable ~= nil then
            inst.components.burnable:Extinguish()
        end
        inst:RemoveComponent("burnable")
        inst:RemoveComponent("propagator")
        inst:RemoveComponent("growable")
		inst:RemoveComponent("pickable")
        --inst:RemoveComponent("hauntable")
		
        inst:RemoveTag("shelter")
		
		inst.components.lootdropper:SetLoot({})
		
        --MakeHauntableWork(inst)
        if inst.components.workable then
            inst.components.workable:SetWorkLeft(1)
            inst.components.workable:SetOnWorkCallback(nil)
            inst.components.workable:SetOnFinishCallback(chop_down_burnt_tree)
        end
    end

    if imm then
        changes()
    else
        inst:DoTaskInTime(.5, changes)
    end
    inst.AnimState:PlayAnimation(inst.anims.burnt, true)

    inst.AnimState:SetRayTestOnBB(true)
    inst:AddTag("burnt")
	
    inst.MiniMapEntity:SetIcon("evergreen_burnt.png")
end

local function PushSway(inst)
	if math.random() > .5 then
		inst.AnimState:PushAnimation(inst.anims.sway1, true)
	else
		inst.AnimState:PushAnimation(inst.anims.sway2, true)
	end
end

local function Sway(inst)
	if math.random() > .5 then
		inst.AnimState:PlayAnimation(inst.anims.sway1, true)
	else
		inst.AnimState:PlayAnimation(inst.anims.sway2, true)
	end
	inst.AnimState:SetTime(math.random()*2)
end

local function SetShort(inst)
	inst.AnimState:OverrideSymbol("leaves", "tree_peach_short", "leaves")
	inst.anims = short_anims
	if inst.components.pickable then
		inst.components.pickable.canbepicked = false
	end

	if inst.components.workable then
		inst.components.workable:SetWorkLeft(TUNING.EVERGREEN_CHOPS_SMALL)
	end
	
	inst:AddTag("shelter")
	
	inst.components.lootdropper:SetLoot(GetBuild(inst).short_loot)
	
	Sway(inst)
end

local function GrowShort(inst)
	inst.AnimState:PlayAnimation("grow_tall_to_short")
	inst.SoundEmitter:PlaySound("dontstarve/forest/treeGrowFromWilt")
	PushSway(inst)
end

local function SetNormal(inst)
	inst.AnimState:OverrideSymbol("leaves", "tree_peach_normal", "leaves")
	inst.anims = normal_anims
	
	if inst.components.pickable then
		inst.components.pickable.canbepicked = false
	end
    if inst.components.workable then
        inst.components.workable:SetWorkLeft(TUNING.EVERGREEN_CHOPS_NORMAL)
    end

	inst.components.lootdropper:SetLoot(GetBuild(inst).normal_loot)

	inst:AddTag("shelter")
	Sway(inst)
end

local function GrowNormal(inst)
	inst.AnimState:PlayAnimation("grow_short_to_normal")
	inst.SoundEmitter:PlaySound("dontstarve/forest/treeGrow")
	PushSway(inst)
end

local function SetTall(inst)
	inst.AnimState:OverrideSymbol("leaves", "tree_peach_tall", "leaves")

	inst.anims = tall_anims
	if inst.components.workable then
        inst.components.workable:SetWorkLeft(TUNING.EVERGREEN_CHOPS_TALL)
	end
	if inst.components.pickable then
		inst.components.pickable.canbepicked = true
	end
	inst:AddTag("shelter")
	inst.components.lootdropper:SetLoot(GetBuild(inst).tall_loot)

	Sway(inst)
end

local function GrowTall(inst)
	inst.AnimState:PlayAnimation("grow_normal_to_tall")
	inst.SoundEmitter:PlaySound("dontstarve/forest/treeGrow")
	PushSway(inst)
end

local function inspect_tree(inst)
	if inst:HasTag("burnt") then
		return "BURNT"
	elseif inst:HasTag("stump") then
		return "CHOPPED"
	elseif inst:HasTag("fire") then
		return "FIRE"		
	else 
	return "GENERIC"
	end
end

--local growth_stages =
--{
--	{name="short", time = function() return TUNING.UMBRELLA_PERISHTIME  end, fn = function(inst) SetShort(inst) end,  growfn = function(inst) GrowShort(inst) end },
--	{name="normal", time = function() return TUNING.UMBRELLA_PERISHTIME  end , fn = function(inst) SetNormal(inst) end, growfn = function(inst) GrowNormal(inst) end },
--	{name="tall", time = function() return TUNING.UMBRELLA_PERISHTIME  end , fn = function(inst) SetTall(inst) end, growfn = function(inst) GrowTall(inst) end },
--}

local growth_stages =
{
	{name="short", time = function() return 6* 480 end, fn = function(inst) SetShort(inst) end,  growfn = function(inst) GrowShort(inst) end },
	{name="normal", time = function() return 6* 480  end , fn = function(inst) SetNormal(inst) end, growfn = function(inst) GrowNormal(inst) end },
	{name="tall", time = function() return 6* 480  end , fn = function(inst) SetTall(inst) end, growfn = function(inst) GrowTall(inst) end },
}

local function chop_tree(inst, chopper, chops)
    if not (chopper ~= nil and chopper:HasTag("playerghost")) then
        inst.SoundEmitter:PlaySound(
            chopper ~= nil and chopper:HasTag("beaver") and
            "dontstarve/characters/woodie/beaver_chop_tree" or
            "dontstarve/wilson/use_axe_tree"
        )
    end

	local fx = SpawnPrefab("jungle_chop")
	local x, y, z= inst.Transform:GetWorldPosition()
	fx.Transform:SetPosition(x,y + 2 + math.random()*2,z)

	inst.AnimState:PlayAnimation(inst.anims.chop)
	inst.AnimState:PushAnimation(inst.anims.sway1, true)
end

local function chop_down_tree(inst, chopper)
    inst:RemoveComponent("burnable")
    MakeSmallBurnable(inst)
    inst:RemoveComponent("propagator")
    MakeSmallPropagator(inst)
    inst:RemoveComponent("workable")
    inst:RemoveTag("shelter")
    --inst:RemoveComponent("hauntable")
	inst:RemoveComponent("pickable")
    --MakeHauntableIgnite(inst)

    RemovePhysicsColliders(inst)
	
	inst.SoundEmitter:PlaySound("dontstarve/forest/treefall")
    local pt = inst:GetPosition()

    local he_right = true

    if chopper then
        local hispos = chopper:GetPosition()
        he_right = (hispos - pt):Dot(TheCamera:GetRightVec()) > 0
    else
        if math.random() > 0.5 then
            he_right = false
        end
    end

    if he_right then
        inst.AnimState:PlayAnimation(inst.anims.fallleft)
        inst.components.lootdropper:DropLoot(pt - TheCamera:GetRightVec())
    else
        inst.AnimState:PlayAnimation(inst.anims.fallright)
        inst.components.lootdropper:DropLoot(pt + TheCamera:GetRightVec())
    end

	local fx = SpawnPrefab("jungle_fall")
	local x, y, z= inst.Transform:GetWorldPosition()
	fx.Transform:SetPosition(x,y + 2 + math.random()*2,z)
	--[[
	inst:DoTaskInTime(.4, function()
    ShakeAllCameras(CAMERASHAKE.FULL, .25, .03,
        inst.components.growable ~= nil and
        inst.components.growable.stage > 2 and .5 or .25,
        inst, 6)
	end)
	--]]
	inst:DoTaskInTime(.4, function() 
		local sz = (inst.components.growable and inst.components.growable.stage > 2) and .5 or .25
		GetPlayer().components.playercontroller:ShakeCamera(inst, "FULL", 0.25, 0.03, sz, 6)
    end)

	RemovePhysicsColliders(inst)
	inst.AnimState:PushAnimation(inst.anims.stump)

	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.DIG)
	inst.components.workable:SetOnFinishCallback(dig_up_stump)
	inst.components.workable:SetWorkLeft(1)

	inst:AddTag("stump")
	if inst.components.growable then
		inst.components.growable:StopGrowing()
	end
	inst.MiniMapEntity:SetIcon("evergreen_stump.png")
end


local function tree_burnt(inst)
	OnBurnt(inst)
end

local function handler_growfromseed (inst)
	inst.components.growable:SetStage(1)
	inst.AnimState:PlayAnimation("grow_seed_to_short")
	inst.SoundEmitter:PlaySound("dontstarve/forest/treeGrow")
	PushSway(inst)
end

local function onsave(inst, data)
    if inst:HasTag("burnt") or (inst.components.burnable ~= nil and inst.components.burnable:IsBurning()) then
        data.burnt = true
    end

	if inst:HasTag("stump") then
		data.stump = true
	end

	if inst.build ~= "normal" then
		data.build = inst.build
	end
end

local function onload(inst, data)
	if data then
		if not data.build or builds[data.build] == nil then
			inst.build = "normal"
		else
			inst.build = data.build
		end
        if data.stump then
			inst:RemoveComponent("burnable")
			MakeSmallBurnable(inst)
			inst:RemoveComponent("propagator")
			MakeSmallPropagator(inst)
			inst:RemoveComponent("workable")
			inst:RemoveTag("shelter")
			--inst:RemoveComponent("hauntable")
			--MakeHauntableIgnite(inst)
			inst:RemoveTag("pickable")
			inst:RemoveComponent("pickable")

			RemovePhysicsColliders(inst)

			inst:AddTag("stump")
			if inst.components.growable ~= nil then
				inst.components.growable:StopGrowing()
			end

			inst.MiniMapEntity:SetIcon("evergreen_stump.png")

			inst:AddComponent("workable")
			inst.components.workable:SetWorkAction(ACTIONS.DIG)
			inst.components.workable:SetOnFinishCallback(dig_up_stump)
			inst.components.workable:SetWorkLeft(1)
            inst.AnimState:PlayAnimation(inst.anims.stump)
			
            if data.burnt or inst:HasTag("burnt") then
                DefaultBurntFn(inst)
            end
        elseif data.burnt and not inst:HasTag("burnt") then
            OnBurnt(inst, true)
        end
	end
end

local function OnEntitySleep(inst)
   local doBurnt = inst.components.burnable ~= nil and inst.components.burnable:IsBurning()
    if doBurnt and inst:HasTag("stump") then
        DefaultBurntFn(inst)
    else
        inst:RemoveComponent("burnable")
        inst:RemoveComponent("propagator")
        inst:RemoveComponent("inspectable")
        if doBurnt then
            inst:RemoveComponent("growable")
			inst:RemoveComponent("pickable")
            inst:AddTag("burnt")
        end
    end
end

local function OnEntityWake(inst)
   if inst:HasTag("burnt") then
        tree_burnt(inst)
    else
        local isstump = inst:HasTag("stump")

        if not (inst.components.burnable ~= nil and inst.components.burnable:IsBurning()) then
            if inst.components.burnable == nil then
                if isstump then
                    MakeSmallBurnable(inst)
                else
                    MakeLargeBurnable(inst, TUNING.TREE_BURN_TIME)
                    inst.components.burnable:SetFXLevel(5)
                    inst.components.burnable:SetOnBurntFn(tree_burnt)
                end
            end

            if inst.components.propagator == nil then
                if isstump then
                    MakeSmallPropagator(inst)
                else
                    MakeMediumPropagator(inst)
                end
            end
        end
    end

    if inst.components.inspectable == nil then
        inst:AddComponent("inspectable")
        inst.components.inspectable.getstatus = inspect_tree
    end
end

local function onpicked(inst)
	if inst.components.growable then
		inst.components.growable:DoPickGrowth()
	end
end

local function makefn(build, stage, data)

	local function fn(Sim)

		local inst = CreateEntity()
		local trans = inst.entity:AddTransform()
		local anim = inst.entity:AddAnimState()

		local sound = inst.entity:AddSoundEmitter()
        
		
		MakeObstaclePhysics(inst, .25)

		local minimap = inst.entity:AddMiniMapEntity()
		
		minimap:SetIcon("peachtree.tex")

		minimap:SetPriority(-1)

		inst:AddTag("tree")

		inst:AddTag("shelter")		

		inst.build = build
		anim:SetBuild(GetBuild(inst).file)
		anim:SetBank("jungletree")
        inst:SetPrefabName(GetBuild(inst).prefab_name)
        inst:AddTag(GetBuild(inst).prefab_name)
		
		--MakeSnowCoveredPristine(inst)
		
		local color = 0.5 + math.random() * 0.5
		anim:SetMultColour(color, color, color, 1)

		-------------------
		MakeLargeBurnable(inst, TUNING.TREE_BURN_TIME)
		inst.components.burnable:SetFXLevel(5)
		inst.components.burnable:SetOnBurntFn(tree_burnt)
		MakeMediumPropagator(inst)

		-------------------
		inst:AddComponent("inspectable")
		inst.components.inspectable.getstatus = inspect_tree

		-------------------
		inst:AddComponent("workable")
		inst.components.workable:SetWorkAction(ACTIONS.CHOP)
		inst.components.workable:SetOnWorkCallback(chop_tree)
		inst.components.workable:SetOnFinishCallback(chop_down_tree)

		-------------------
		inst:AddComponent("lootdropper")
		---------------------
		inst:AddComponent("pickable")
		inst.components.pickable.picksound  = "dontstarve/wilson/harvest_berries"
		inst.components.pickable:SetUp("peach")
		--if stage == 3 then
		--	inst.components.pickable.canbepicked = true
		--else
			inst.components.pickable.canbepicked = false
		--end
		inst.components.pickable.jostlepick = true
		inst.components.pickable.droppicked = true
		inst.components.pickable.dropheight = 6.5
		inst.components.pickable.OnSave = function(self,data)
			return {}
		end
		inst.components.pickable.OnLoad = function(self,data)
			return
		end
		inst.components.pickable.Pick = function( self, picker)
			if self.canbepicked  then
			local loot = nil
			if picker ~= nil and picker.components.inventory ~= nil and self.product ~= nil then
				if self.droppicked and self.inst.components.lootdropper ~= nil then
					local num = math.random(1,3)
					local pt = self.inst:GetPosition()
					pt.y = pt.y + (self.dropheight or 0)
					for i = 1, num do
						--self.inst.components.lootdropper:SpawnLootPrefab(self.product, pt)
						local product = math.random()<0.01 and 'bigpeach' or 'peach'
						self.inst.components.lootdropper:SpawnLootPrefab(product, pt)
					end
				end
			end
			if self.onpickedfn ~= nil then
				self.onpickedfn(self.inst, picker, loot)
			end
			self.canbepicked = false
				self.inst:PushEvent("picked", { picker = picker, loot = loot, plant = self.inst })
			end
		end
		inst.components.pickable.onpickedfn = onpicked
		
		inst:AddComponent("growable")
		inst.components.growable.stages = growth_stages
		inst.components.growable:SetStage(stage == 0 and math.random(1, 3) or stage)
		inst.components.growable.springgrowth = true
		local old_StartGrowing = inst.components.growable.StartGrowing
		inst.components.growable.StartGrowing = function(self,time)
			if #self.stages == 0 or  self.stage >= #self.stages then
				return
			end
			old_StartGrowing(self,time)
		end
		inst.components.growable:StartGrowing()
		inst.components.growable.DoPickGrowth = function(self)
			local stage = 1
			self:SetStage(stage)
			if self.stages[stage] ~= nil and self.stages[stage].growfn ~= nil then
				self.stages[stage].growfn(self.inst)
			end
			if self.stage < #self.stages  then
				self:StartGrowing()
			end		
		end
		inst.components.growable.OnSave = function(self)
			local data =
			{
				stage = self.stage or nil,
				time = (self.pausedremaining ~= nil and math.floor(self.pausedremaining))
					or (self.targettime ~= nil and math.floor(self.targettime - GetTime()))
					or nil,
			}
			return next(data) ~= nil and data or nil
		end		
		if inst.components.growable.stage == 3 then
			inst.components.pickable.canbepicked = true
		end

		inst.growfromseed = handler_growfromseed
		
		inst.OnSave = onsave
		inst.OnLoad = onload
		if data =="stump"  then
            RemovePhysicsColliders(inst)
            inst:AddTag("stump")
            inst:RemoveTag("shelter")

            inst:RemoveComponent("burnable")
            MakeSmallBurnable(inst)
            inst:RemoveComponent("workable")
            inst:RemoveComponent("propagator")
            MakeSmallPropagator(inst)
            inst:RemoveComponent("growable")
			inst:RemoveComponent("pickable")
            inst:AddComponent("workable")
            inst.components.workable:SetWorkAction(ACTIONS.DIG)
            inst.components.workable:SetOnFinishCallback(dig_up_stump)
            inst.components.workable:SetWorkLeft(1)
            inst.AnimState:PlayAnimation(inst.anims.stump)
			inst.MiniMapEntity:SetIcon("evergreen_stump.png")
		else
            inst.AnimState:SetTime(math.random() * 2)
            if data == "burnt" then 
                OnBurnt(inst)
            end
		end
		inst.OnEntitySleep = OnEntitySleep
		inst.OnEntityWake = OnEntityWake

		--DebugTime(inst) --@--

		return inst
	end
	return fn
end

local function tree(name, build, stage, data)
	return Prefab(name, makefn(build, stage, data), assets, prefabs)
end
--DebugSpawn"peachtree_tall"
--DebugSpawn"peachtree_normal"
return tree("peachtree", "normal", 0),
		tree("peachtree_normal", "normal", 2),
		tree("peachtree_tall", "normal", 3),
		tree("peachtree_short", "normal", 1),
		tree("peachtree_burnt", "normal", 0, "burnt"),
		tree("peachtree_stump", "normal", 0, "stump")
