local assets = {
	Asset("ANIM", 'anim/wb_heart.zip'),
	Asset("ANIM", 'anim/wb_tinymeat.zip'),
}

STRINGS.NAMES.WB_PREDECESSOR = '骨架'
STRINGS.RECIPE_DESC.WB_PREDECESSOR = '建造一副合适的身躯'

STRINGS.NAMES.WB_HEART = '阴森之心'
STRINGS.RECIPE_DESC.WB_HEART = '灵魂出壳'

STRINGS.NAMES.WB_TINYMEAT = '碎肉粒'
STRINGS.NAMES.WB_TINYMEAT_COOKED = '风味肉松'

STRINGS.NAMES.WB_BODY = '没有灵魂的尸体'
STRINGS.CHARACTERS.GENERIC.DESCRIBE.WB_BODY = '吾曾附身于此'
 
local function heart()
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	local snd = inst.entity:AddSoundEmitter()

	MakeInventoryPhysics(inst)
	MakeInventoryFloatable(inst, 'idle', 'idle')

	anim:SetBank('wb_heart')
	anim:SetBuild('wb_heart')
	anim:PlayAnimation('idle', true)

	inst:AddComponent('inventoryitem')
	inst.components.inventoryitem.atlasname = 'images/monkey_king_item.xml'
	inst:AddComponent('inspectable')

	inst:AddTag('wb_heart')
	inst:AddTag('meat')

	inst:AddComponent('edible')
	inst.components.edible.oneaten = function(inst, eater)
		if eater and eater.components.health and not eater.components.health:IsDead() then
			if eater.prefab ~= 'white_bone' then
				eater.components.health:DoDelta(-1000)
			else
				eater.components.sanity:DoDelta(100)
				eater:PushEvent('wbEvent_eatheart')
			end
		end
	end


	return inst
end

local function onhammered(inst)
	inst.components.inventory:DropEverything()
	inst.components.lootdropper:DropLoot()
	inst:Remove()
end

local function onkilled(inst)
	onhammered(inst)
end

local function body()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	MakeObstaclePhysics(inst, 0.25)

	inst.Transform:SetFourFaced()
	anim:SetBank('wilson')
	anim:SetBuild('white_bone')
	anim:PlayAnimation('death')
	anim:SetTime(10)
	inst:AddComponent('inventory')

	inst:AddComponent('inspectable')

	inst:AddComponent('lootdropper')
	inst.components.lootdropper:SetLoot({'boneshard', 'boneshard', 'boneshard'})

	inst:AddComponent('wb_bone')

	inst.FromPlayer = function(inst, player)
		for i, v in pairs(player.components.inventory.itemslots)do
			inst.components.inventory:GiveItem(v, i)
		end
		inst.Transform:SetRotation(player.Transform:GetRotation())
		inst.Transform:SetPosition(player:GetPosition():Get())
		inst.components.wb_bone.value = player.components.health.maxhealth
	end

	inst.ToPlayer = function(inst, player)
		for i, v in pairs(inst.components.inventory.itemslots)do
			player.components.inventory:GiveItem(v, i, nil, true)
		end
	end

	inst.OnSave = function(inst, data)data.h = inst.components.wb_bone.value end
	inst.OnLoad = function(inst, data)
		if data.h then inst.components.wb_bone.value = data.h end
	end

	inst.onkilled = onkilled

	return inst
end


local function common()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
    
    MakeBlowInHurricane(inst, .1, .25)
	MakeInventoryFloatable(inst, 'idle', 'idle')
    
    inst:AddTag("meat")

    inst:AddComponent("edible")
    inst.components.edible.ismeat = true    
    inst.components.edible.foodtype = "MEAT"
    inst.components.edible.foodstate = "RAW"
    inst.components.edible.forcequickeat = true
    
    inst:AddComponent("stackable")
	inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM

    inst:AddComponent("bait")
    
    inst:AddComponent("inspectable")
    
    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.atlasname = 'images/monkey_king_item.xml'

	inst:AddComponent("perishable")
	inst.components.perishable:SetPerishTime(TUNING.PERISH_TWO_DAY)
	inst.components.perishable:StartPerishing()
	inst.components.perishable.onperishreplacement = "spoiled_food"

    inst:AddComponent("appeasement")
    inst.components.appeasement.appeasementvalue = TUNING.APPEASEMENT_TINY

    return inst
end

local function tinymeat()
	local inst = common()
	inst.AnimState:SetBank('wb_tinymeat')
	inst.AnimState:SetBuild('wb_tinymeat')
	inst.AnimState:PlayAnimation('idle')
	inst.components.edible.healthvalue = 0
    inst.components.edible.hungervalue = 3
    inst.components.edible.sanityvalue = 0

    inst:AddComponent('cookable')
    inst.components.cookable.product = "wb_tinymeat_cooked"

    return inst
end

local function tinymeat_cooked()
	local inst = common()
	inst.AnimState:SetBank('wb_tinymeat')
	inst.AnimState:SetBuild('wb_tinymeat')
	inst.AnimState:PlayAnimation('idle_cooked')
	MakeInventoryFloatable(inst, 'idle_cooked', 'idle_cooked')

	inst.components.edible.healthvalue = 0.5
    inst.components.edible.hungervalue = 3
    inst.components.edible.sanityvalue = 0

    inst.components.edible.foodstate = 'COOKED'

    inst.components.perishable:SetPerishTime(TUNING.PERISH_FAST)

    return inst
end

local function shadow()
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()

	trans:SetScale(1.2,1.2,1)
	trans:SetFourFaced()

	anim:SetBank('wilson')
	anim:SetBuild('white_bone')

	inst:SetStateGraph('SGwilson')
	local states = {
		'idle', 'run_start', 'run', 'run_stop', 'frozen', 'death', 'attack', 
		'chop', 'eat', 'hit', 
	}
	local states2 = {
		walk_start = 'run_start',
		walk_stop = 'run_stop',
		walk = 'run',
	}

	inst.Bind = function(inst, pig)
		inst.entity:SetParent(pig.entity)
		inst:ListenForEvent('newstate', function(_, data)
			local new = data and data.statename
			if new then
				for _, v in ipairs(states)do
					if v == new then
						inst.sg:GoToState(new)
						return
					end
				end
				for k, v in pairs(states2)do
					if k == new then
						inst.sg:GoToState(v)
						return 
					end
				end
			end
		end)
	end

	return inst
end

return Prefab('wb_heart', heart, assets),
		--Prefab('wb_tinymeat', tinymeat),
		--Prefab('wb_tinymeat_cooked', tinymeat),
		Prefab('wb_body', body),
		MakePlacer('wb_predecessor_placer', 'skeleton', 'skeletons', 'idle3'),
		Prefab('wb_shadow')

