local assets = {
	Asset('ANIM', 'anim/wb_skeleton.zip'),
}

require "stategraphs/SGwb_skeleton"

local L = MK_MOD_LANGUAGE_SETTING
STRINGS.NAMES.WB_SKELETON = L and 'Living skeleton' or '复活的骨架'
--STRINGS.CHARACTERS.GENERIC.DESCRIBE.WB_STAFF = L and "A magic staff." or '巴啦啦能量, 沙卡拉卡,... 咳咳...'

local function EquipWeapon(inst)
	local weapon = SpawnPrefab(inst.weaponprefab or 'spear')
	weapon.components.weapon:SetDamage(34)

	weapon.persists = false
	if weapon.components.finiteuses then
		weapon:RemoveComponent('finiteuses')
	end
	weapon:ListenForEvent('ondropped', weapon.Remove)

	inst.components.inventory:Equip(weapon)
end

local function OnSave(inst, data)
	data.weaponprefab = inst.weaponprefab
end

local function OnLoad(inst, data)
	inst.weaponprefab = data.weaponprefab
end

local function SpawnBySpell(inst)
	inst.sg:GoToState('spawn')
	MK_MOD_UTIL.MakeFxAt(inst:GetPosition(), 'small_puff', 'smoke_puff_small', 'puff', nil, 
		function(fx)
			fx.AnimState:SetMultColour(150/255, 30/255, 200/255, 1)
			fx.entity:AddSoundEmitter()
			fx.SoundEmitter:PlaySound('dontstarve/common/deathpoof')
		end)
end

local function OnAttacked(inst, data)
	if inst.components.health:IsDead() then return end
	if data and data.attacker and data.attacker ~= GetPlayer() then
		inst.components.combat:SuggestTarget(data.attacker)
	end
end

local function BecomeSkeleton(inst)
	SpawnPrefab('small_puff').Transform:SetPosition(inst:GetPosition():Get())
	SpawnPrefab('skeleton').Transform:SetPosition(inst:GetPosition():Get())
end

local function ShouldAcceptItem(inst, item)
    if inst.components.health:IsDead() then
        return false
    end
    --需要继续调
    if item.components.equippable and item.components.equippable.equipslot ~= EQUIPSLOTS.HANDS then
        return true
    end
end

local function OnGetItemFromPlayer(inst, giver, item)
    if item.components.equippable then
    	local slot = item.components.equippable.equipslot
        local current = inst.components.inventory:GetEquippedItem(slot)
        if current then
            inst.components.inventory:DropItem(current)
        end
        inst.components.inventory:Equip(item)
        --inst.AnimState:Show("hat")
    end
end

local fn = function()
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	local snd = inst.entity:AddSoundEmitter()
	local shadow = inst.entity:AddDynamicShadow()
	shadow:SetSize(0.8,0.4)
	MakeCharacterPhysics(inst, 20, .2)

	anim:SetBank('wilson')
	anim:SetBuild('wb_skeleton')
	local c = GetRandomMinMax(0.7, 1)
	anim:SetMultColour(c,c,c, 1)
	anim:Hide('ARM_normal')

	trans:SetScale(0.95,0.95,1)
	trans:SetFourFaced()

	inst:AddComponent('combat')
	inst.components.combat:SetAttackPeriod(0.5)
	inst.components.combat.notags = {'wb_skeleton', 'white_bone'}

	inst:AddComponent('inventory')
	inst:AddComponent('health')
	inst.components.health:SetMaxHealth(75)
	inst.components.health.canheal = false

	inst.DoFix = function(inst, value) inst.components.health:DoDelta(value) end
	
	inst:AddComponent('locomotor')
	inst:AddComponent('follower')
	inst:AddComponent('inspectable')
	inst:AddComponent('timer')
	inst:AddComponent('trader')
	inst.components.trader:Enable()
	inst.components.trader:SetAcceptTest(ShouldAcceptItem)
	inst.components.trader.onaccept = OnGetItemFromPlayer

	inst:AddComponent('lootdropper')
	if math.random() < 0.25 then
		inst.components.lootdropper:SetLoot({'boneshard'})
	else
		--inst.components.lootdropper:SetLoot({'boneash'})
	end

	inst:SetStateGraph('SGwb_skeleton')
	inst:SetBrain(require 'brains/wb_skeleton_brain')

	inst:AddTag('wb_skeleton')
	inst:AddTag('monster')
	inst:AddTag('character')

	inst:DoTaskInTime(0, EquipWeapon)

	inst.OnSave = OnSave
	inst.OnLoad = OnLoad
	inst.SpawnBySpell = SpawnBySpell

	inst:ListenForEvent('attacked', OnAttacked)

	inst:ListenForEvent('loseloyalty', BecomeSkeleton)

	return inst
end

return Prefab('wb_skeleton', fn, assets)
