local MakePlayerCharacter = require "prefabs/player_common"
local easing = require "easing"
require 'stategraphs/SGwhite_bone_ghost'

local assets = {
    Asset("ANIM", "anim/white_bone.zip"),
    Asset("ANIM", "anim/white_bone_beautiful.zip"),
    Asset("ANIM", "anim/ghost_white_bone_build.zip"),
    Asset("ANIM", "anim/dst_myth/myth_ghostbank.zip"),
    Asset("ANIM", 'anim/white_bone_changefx.zip'),
}

local start_inv = {
	'boneshard', 'boneshard', 'boneshard', 'boneshard',
	'nightmarefuel', 'nightmarefuel', 'nightmarefuel', 'nightmarefuel',
	'livinglog', 'livinglog',
}

local function ghostHUD(enable)
	if GetPlayer().HUD then
		local HUD = GetPlayer().HUD
		local ctrl = HUD.controls
		local widgets = {
			HUD.vig,
			ctrl.inv,
			--ctrl.containerroot, --这玩意儿居然是TMI的基座...
			ctrl.containerroot_side,
			ctrl.crafttabs,
			ctrl.status.stomach,
			ctrl.status.heart,
			ctrl.status.wb_button,
			ctrl.status.moisturemeter,
			ctrl.mapcontrols,--?
		}
		for _, v in pairs(widgets)do
			if type(v) == 'table' and v.Hide and enable then
				v:Hide()
			end
			if type(v) == 'table' and v.Show and not enable then
				v:Show()
			end
		end
	end
end

local function SpawnBoneOnDeath(inst)
	local pos = inst:GetPosition()
	local maxhealth = inst.components.health.maxhealth
	local drops = { SpawnPrefab('nightmarefuel')}
	if maxhealth < 30 then
		table.insert(drops, SpawnPrefab('ash')) --?
	elseif maxhealth < 75 then
		table.insert(drops, SpawnPrefab('boneshard'))
	elseif maxhealth < 150 then
		table.insert(drops, SpawnPrefab('boneshard'))
		table.insert(drops, SpawnPrefab('boneshard'))
	elseif maxhealth < 250 then
		table.insert(drops, SpawnPrefab('houndstooth'))
		table.insert(drops, SpawnPrefab('boneshard'))
		table.insert(drops, SpawnPrefab('boneshard'))
	else
		table.insert(drops, SpawnPrefab('houndstooth'))
		table.insert(drops, SpawnPrefab('boneshard'))
		table.insert(drops, SpawnPrefab('boneshard'))
		table.insert(drops, SpawnPrefab('boneshard'))
	end
	local function _FlingItem(loot)
        loot.Transform:SetPosition(pos:Get())
        if loot.Physics ~= nil then
            local angle = math.random() * 2 * PI
            local speed = math.random() * 2
            local radius = 1
            local sinangle = math.sin(angle)
            local cosangle = math.cos(angle)
            loot.Physics:SetVel(speed * cosangle, GetRandomWithVariance(8, 4), speed * -sinangle)
			loot.Transform:SetPosition(
                pos.x + cosangle * radius,
                pos.y,
                pos.z - sinangle * radius
            )
        end
    end
	for _,v in ipairs(drops)do
		_FlingItem(v)
	end
end

local function SpawnMyBody(inst)
	--生成一个带有物品的“尸体”
	local body = SpawnPrefab('wb_body')
	body:FromPlayer(inst)
end

local function IsWorkable(inst)
	local work_action = inst.components.workable and inst.components.workable.workleft
	  	and inst.components.workable.workleft > 0 and inst.components.workable.action
	return work_action ~= nil and (
		work_action == ACTIONS.CHOP or
        work_action == ACTIONS.HAMMER or
        work_action == ACTIONS.MINE or   
        work_action == ACTIONS.DIG or
        work_action == ACTIONS.BLANK
    )
end
	
local function IsValidGhostActionTarget(inst)
	return inst and (
		inst.components.inventoryitem or
		inst.components.health or
		inst.components.wb_bone or
		inst.components.perishable or
		inst.components.teleporter or --虫洞
		IsWorkable(inst)
	)
end

local function GhostActionButton(inst)
	if not inst:CanHaunt() then return end
	if inst.sg:HasStateTag('haunt') then return end

	local action_target = FindEntity(inst, 6, IsValidGhostActionTarget)
	if not action_target then
		return BufferedAction(inst, inst, ACTIONS.WB_HAUNT, nil, inst:GetPosition())
	else
		return BufferedAction(inst, action_target, ACTIONS.WB_HAUNT)
	end
end

local function LeftClickPicker(inst, target_ent, pos)
	if IsValidGhostActionTarget(target_ent) and inst:CanHaunt() then
		return inst.components.playeractionpicker:SortActionList({ACTIONS.WB_HAUNT}, target_ent)
	end
end

local function RightClickPicker(inst, target_ent, pos)
	return {}
end

local function ChangeToGhost(inst)
	inst._isghost = true
	inst._isbeauty = false
	inst.Light:Enable(true)
	inst:AddTag('mythTag_ignorearmor')
	inst:AddTag('wb_ghost')
	inst:AddTag('hostile') --比较逗的一个标签
	inst.AnimState:SetBuild('ghost_white_bone_build')
	inst.AnimState:SetBank('ghost')
	inst:SetStateGraph("SGwhite_bone_ghost")

	--移速加成
	inst.components.locomotor:AddSpeedModifier_Additive('wb_ghost', 1)

	inst.components.playercontroller.actionbuttonoverride = GhostActionButton
	inst.components.playeractionpicker.leftclickoverride = LeftClickPicker
	inst.components.playeractionpicker.rightclickoverride = RightClickPicker
	--理智值
	--inst.components.sanity:DoDelta(50) --见sg
	--饥饿值
	inst.components.hunger:DoDelta(100, true)
	inst.components.hunger:Pause()
	--无敌状态
	inst.components.health:SetPercent(1)
	inst.components.health.redirect = function() return end
	inst.components.health:SetInvincible(true)
	inst.components.health.absorb = 1
	--免疫潮湿(API)
	inst.components.moisture:DoDelta(-200)
	--免疫过冷过热(health组件)
	--自由物理
	MK_MOD_UTIL.SetPhysicsCollision(inst, {'ground',})
	inst.components.locomotor:EnableGroundSpeedMultiplier(false)
	--毒林
	if inst.components.tiletracker then
		inst.components.tiletracker:Stop()
	end
	--隐藏部分ui
	if inst == GetPlayer() then
		ghostHUD(true)
	end
end

local function ChangeTreeToLeif(inst)
    local days_survived = GetClock().numcycles
    local rate = 0.20
    if math.random() < rate and inst.components.growable and inst.components.growable.stage <= 3 
    	and inst:HasTag('tree') and not inst:HasTag('burnt') and not inst:HasTag('stump') and not inst.noleif then

    	local stage = inst.components.growable.stage
    	local treeprefab = inst.prefab or 'evergreen_normal'
    	local prefab = treeprefab:find('_sparse') and 'leif_sparse' or treeprefab:find('palmtree') and 'treeguard' or 'leif'
    	local scale = stage == 1 and 0.7 or stage == 2 and 1 or stage == 3 and 1.25 or 1
    	inst.noleif = true
    	local leif = SpawnPrefab(prefab)
    	if leif then
    		leif.AnimState:SetMultColour(inst.AnimState:GetMultColour())

			leif.components.locomotor.walkspeed = leif.components.locomotor.walkspeed*scale
			leif.components.combat.defaultdamage = leif.components.combat.defaultdamage*scale
			leif.components.health.maxhealth = leif.components.health.maxhealth*scale
			leif.components.health.currenthealth = leif.components.health.currenthealth*scale
			leif.components.combat.hitrange = leif.components.combat.hitrange*scale
			leif.components.combat.attackrange = leif.components.combat.attackrange*scale
			
			leif.Transform:SetScale(scale,scale,scale) 
			leif.sg:GoToState("spawn")
			leif.Transform:SetPosition(inst.Transform:GetWorldPosition())
			inst:Remove()
		end
    end
end

local HAUNT_CD = 2
local function CanHaunt(inst)
	return GetTime() - inst._lasthaunttime > HAUNT_CD
end

local function DoHaunt(inst, pos)
	if not inst:CanHaunt() then
		return
	else
		inst._lasthaunttime = GetTime()
	end
	local work = 3
	local damage = 34
	local r = 4

	local ents = TheSim:FindEntities(pos.x, 0, pos.z, r, nil, {'FX', 'NOCLICK', 'INLIMBO', 'notarget', 'player', 'wb_skeleton'})
	local validents = {}
	for _,v in pairs(ents)do
		if IsValidGhostActionTarget(v) then
			table.insert(validents, v)
		end
	end
	if #validents == 1 then
		local ent = validents[1]
		if ent.components.wb_bone then
			ent.components.wb_bone:Spirited(inst)
			return
		end
		if ent.components.teleporter then
			ent.components.teleporter:Activate(inst)
			return
		end
		if ent.components.door and ACTIONS.USEDOOR then
			if ACTIONS.USEDOOR.fn({target = ent, doer = inst}) then
				return
			end
			--ent.components.door:Activate(inst)
			--return
		end
	end
	for _,v in pairs(validents)do
		if v.components.combat and v.components.health and not v.components.health:IsDead() then
			v.components.combat:GetAttacked(inst, damage)
		end
		if IsWorkable(v) and not v:HasTag('wall') then
			v.components.workable:WorkedBy(inst, work)
		end
		if v.components.perishable then
			v.components.perishable:LongUpdate(1000)
		end
		if v.prefab == 'evergreen_sparse' or v.prefab == 'evergreen' then
			ChangeTreeToLeif(v)
		end
		if v.components.werebeast and not v.components.werebeast:IsInWereState() and not v.components.health:IsDead() then
			if math.random() < 0.5 then
				v.components.werebeast:SetWere(60)
			end
		end
		if v.components.inventoryitem and v.Physics and v.Physics:IsActive() then
        	local x, y, z = v.Transform:GetWorldPosition()
        	local x1, y1, z1 = pos:Get()
        	local vx, vz = x - x1, z - z1
        	local spd = math.sqrt(vx * vx + vz * vz)
        	local angle =
            	spd > 0 and
            	math.atan2(vz / spd, vx / spd) + (math.random() * 20 - 10) * DEGREES or
            	math.random() * 2 * PI
        	spd = 5 + math.random() * 2
        	v.Physics:Teleport(x, .1, z)
        	v.Physics:SetVel(math.cos(angle) * spd, 10, math.sin(angle) * spd)
        end
    end
end

local function ChangeToHuman(inst)
	inst._isghost = false
	inst._isbeauty = false
	inst:RemoveTag('mythTag_ignorearmor')
	inst:RemoveTag('wb_ghost')
	inst:RemoveTag('hostile')
	inst.Light:Enable(false)
	inst.AnimState:SetBank('wilson')
	inst.AnimState:SetBuild('white_bone')
	inst:SetStateGraph('SGwilson')

	inst.components.health.redirect = nil
	inst.components.health.absorb = .4 --默认为白骨状态
	inst.components.health:SetInvincible(false)
	inst.components.health:SetPercent(1) --生命上限的设定在sg
	inst.components.health:DoDelta(0)

	inst.components.hunger:Resume()

	MK_MOD_UTIL.SetPhysicsCollision(inst, {'world', 'obs', 'char', 'wave'})
	inst.components.locomotor:EnableGroundSpeedMultiplier(true)
	inst.components.locomotor:RemoveSpeedModifier_Additive('wb_ghost')

	inst.components.playercontroller.actionbuttonoverride = nil
	inst.components.playeractionpicker.leftclickoverride = nil
	inst.components.playeractionpicker.rightclickoverride = nil

	if inst.components.tiletracker then
		inst.components.tiletracker:Start()
	end

	ghostHUD(false)

	inst:ChangeToMonster(true)
end


local function DoFix(inst, value)
	local hurtvalue = inst.components.health.maxhealth - inst.components.health.currenthealth
	local _maxhealth = inst.components.health.maxhealth
	if hurtvalue >= value then
		inst.components.health:DoDelta(value, false, 'wb_bonefix')
	else
		local left = value - hurtvalue
		local healthbonus = math.min(0.5*0.5, (50/ _maxhealth)^2) * left
		inst.components.health:SetMaxHealth(_maxhealth + healthbonus)
		inst.components.health:DoDelta(0, false) --刷新ui
		if inst.HUD then
			--inst.SoundEmitter:PlaySound("dontstarve/characters/wx78/levelup")
			--要不要加一个音效?
			inst.HUD.controls.status.heart:PulseGreen()
			inst.HUD.controls.status.heart:ScaleTo(1.3,1,.7)
		end
	end
end

local function ChangeToBeauty(inst)
	if inst._isghost then return end
	inst._isbeauty = true
	inst.AnimState:SetBuild('white_bone_beautiful')
	inst:RemoveTag('monster')
	inst.components.locomotor:AddSpeedModifier_Additive('wb_beauty', -1.5)
	inst.components.health.absorb = 0

	--清除栏位
	local inv = inst.components.inventory
	if inv then
		for k, v in pairs(inv.equipslots)do
			if v and v.prefab == 'wb_bonelock' then
				v.components.equippable.myth_unequipable = nil
				v:Remove()
			end
		end
	end
end

local function ChangeToMonster(inst, load)
	if inst._isghost then return end
	inst._isbeauty = false
	inst.AnimState:SetBuild('white_bone')
	inst:AddTag('monster')
	if inst.components.locomotor.RemoveSpeedModifier_Additive then
		inst.components.locomotor:RemoveSpeedModifier_Additive('wb_beauty')
	end
	inst.components.health.absorb = .4

	--锁死栏位
	local inv = inst.components.inventory
	if inv then
		for k, slot in pairs(EQUIPSLOTS)do
			if k ~= 'HANDS' then
				print('equip on -> ', slot)
				local item = SpawnPrefab('wb_bonelock')
				item.components.equippable.equipslot = slot
				inv:Equip(item)
			end
		end
	end

	if not load then
		local pos = inst:GetPosition()
		--特效
		SpawnPrefab('lightning').Transform:SetPosition(pos:Get())
		local fx = SpawnPrefab('sanity_raise')
		fx.Transform:SetPosition(pos:Get())
		fx.AnimState:SetTime(0.25)
		fx.AnimState:SetDeltaTimeMultiplier(1.5)
		--范围震慑
		local ents = TheSim:FindEntities(pos.x, pos.y, pos.z, 20, nil, {"FX",'INLIMBO'})
		for _, v in pairs(ents)do
			if v.components.health and not v.components.health:IsDead() then
				if v:HasTag('bird') then
					v:PushEvent('flyaway')
				end
				if v.components.combat then
					v.components.combat:SuggestTarget(inst)
				end
			end
		end
	end
end

local function TriggerDisguise(inst)
	if inst._isghost then return end
	if not inst._isbeauty then inst:PushEvent('wbEvent_changetobeauty') end
	if inst._isbeauty then inst:ChangeToMonster() end
end

local function onsave(inst, data)
	data._maxhealth = inst.components.health.maxhealth
	data._isghost = inst._isghost
	data._isbeauty = inst._isbeauty
end

local function onpreload(inst, data)
	inst._isghost = data._isghost
	inst._isbeauty = data._isbeauty
end

local function onload(inst, data)
	if inst._isghost then
		inst:ChangeToGhost()
	end
	if inst._isbeauty then
		inst:ChangeToBeauty()
	else
		inst:ChangeToMonster(true)
	end
	inst.components.health.maxhealth = data._maxhealth or 150
	inst.components.health.currenthealth = data.health and data.health.health or data._maxhealth or 150
	inst.components.health:DoDelta(0)
end

local fn = function(inst) 
	inst._isghost = false
	inst._isbeauty = false
	inst.OnSave = onsave 
	inst.OnPreLoad = onpreload 
	inst.OnLoad = onload 

	inst.MiniMapEntity:SetIcon( "white_bone.tex" )
	
	inst:AddTag("white_bone")
	inst:AddTag("monster")
	inst:AddTag("insomniac")--失眠
	inst.soundsname = "sfx"
	inst.talker_path_override = 'WhiteBone/'

	inst.AnimState:SetBuild('white_bone')

	inst.components.health:SetMaxHealth(150)
	inst.components.hunger:SetMax(150)
	inst.components.sanity:SetMax(250)
	inst.components.sanity.night_drain_mult = 0 --不受黑暗的影响

	inst.components.locomotor.runspeed = 7.5
	
	inst.SpawnBoneOnDeath = SpawnBoneOnDeath
	inst.SpawnMyBody = SpawnMyBody
	inst.ChangeToGhost = ChangeToGhost
	inst.ChangeToHuman = ChangeToHuman
	inst.ChangeToBeauty = ChangeToBeauty
	inst.ChangeToMonster = ChangeToMonster
	inst.TriggerDisguise = TriggerDisguise
	inst.DoHaunt = DoHaunt
	inst.CanHaunt = CanHaunt
	inst.DoFix = DoFix

	inst:AddComponent('wb_sanity')
	inst:AddComponent('wb_fog')

	inst._lasthaunttime = 0

	--awsl
	if inst.components.poisonable then
		inst.components.poisonable:SetBlockAll(true)
	end
	if inst.components.grogginess then
		inst.components.grogginess.hasoverheatinggear = function() return false end
	end
	if inst.components.hayfever then
		inst.components.hayfever.hayfeverimune = true   
	end
	inst.components.temperature.hurtrate = 0.3 --wilson 1.25

	--死亡逻辑修改
	local old_setval = inst.components.health.SetVal
	inst.components.health.SetVal = function(self, val, cause, ...)
		if inst._isghost then return end 
		if val <= 0 then
			if self:Myth_DoYamarajaBuff() then
		 		return
		 	else
		 		--inst:PushEvent('healthdelta', {newpercent = 0, oldpercent = 0, overtime = true}) --操纵ui
		 		inst:PushEvent('wbEvent_changetoghost', {cause = cause})
		 		return
		 	end
		end
		old_setval(self, val, ...)
	end
	--无视回血
	--[[
	local old_delta = inst.components.health.DoDelta
	inst.components.health.DoDelta = function(self, delta, ot, cause, ...)
		if delta > 0 and cause ~= 'wb_bonefix' then
			return
		else
			return old_delta(self, delta, ot, cause, ...)
		end
	end
	--]]
	--吃肉恢复理智
	local old_eat = inst.components.eater.Eat
	inst.components.eater.Eat = function(self, food, ...)
		if self:CanEat(food) and food.components.edible then
			if food.components.stackable then
				food = food.components.stackable:Get()
			end
			local edible = food.components.edible
			if food:HasTag('meat') or food:HasTag("smallmeat") then
				local israw = food.components.cookable ~= nil
				if israw and edible.hungervalue > 0 then
					edible.hungervalue = edible.hungervalue* 1.2
				end
				local sanity = (israw and 0.5 or 0.0)* edible.hungervalue
				local health = (israw and 0.5 or 0.0)* edible.hungervalue
				edible.healthvalue = israw and math.max(edible.healthvalue, health) or 0
				edible.sanityvalue = math.max(sanity, edible.sanityvalue)
			else
				edible.sanityvalue = math.min(edible.sanityvalue, 0)
				edible.healthvalue = math.min(edible.healthvalue, 0)
				local hunger_mult = (edible.foodtype == 'VEGGIE' or edible.foodtype == 'SEEDS') and 0.2 or 0.8
				if edible.hungervalue > 0 then
					edible.hungervalue = edible.hungervalue* hunger_mult
				end
			end
		end
		return old_eat(self, food, ...)
	end
	--鬼魂状态下持续掉san
	local old_up = inst.components.sanity.OnUpdate
	inst.components.sanity.OnUpdate = function(self, dt, ...)
		if self.inst._isghost then
			self.rate = -10/60
			self:DoDelta(self.rate*dt, true)
		else
			return old_up(self, dt, ...)
		end
	end

	--受攻击扣除生命上限
	--使周围中立生物对攻击者产生仇恨
	inst:ListenForEvent('attacked', function(_, data)
		local pos = inst:GetPosition()
		if inst._isghost or inst.sg:HasStateTag('wb_2ghost') then return end
		--[[
		if data and data.damage and data.damage > 0 then
			local health = inst.components.health
			local max = health.maxhealth
			local current = health.currenthealth
			local scale = 0.5
			local dec = math.min(scale* data.damage, max - 30)
			if dec > 0 and current > 0 then
				health.maxhealth = max - dec
				health:DoDelta(0)
				if dec > 20 and math.random() < 0.33 then
					local item = SpawnPrefab('boneshard')
					local angle = math.random()*2*PI
					item.Physics:Teleport(pos.x, 0.05, pos.z)
					item.Physics:SetVel(math.cos(angle) * 5, 5, math.sin(angle) * 5)
				end
			end
		end
		--]]
		local notags = {'FX', 'INLIMBO', 'hostile', 'monster', 'player', 'notarget', 'werepig', 'guard',}
		if inst._isbeauty and data and data.attacker and data.attacker:IsValid() then
			for _,v in pairs(TheSim:FindEntities(pos.x, 0, pos.z, 20, nil, notags))do
				if v.components.combat and
					v.components.health and not v.components.health:IsDead() then
					v.components.combat:SuggestTarget(data.attacker)
				end
			end
		end
	end)
	local old_app = inst.components.inventory.ApplyDamage
	inst.components.inventory.ApplyDamage = function(self,damage, attacker, weapon,...)
		if attacker and (attacker:HasTag("shadow") or attacker:HasTag("shadowminion") or attacker:HasTag("shadowchesspiece") ) then
			if inst.components.hunger then
				inst.components.hunger:DoDelta(-damage)
			end
			return 0
		end
		return old_app(self,damage, attacker, weapon,...)
	end
	--噩梦燃料
	table.insert(inst.components.eater.foodprefs, "WB_NIGHTMAREFUEL")
	table.insert(inst.components.eater.ablefoods, "WB_NIGHTMAREFUEL")
	--发动攻击破除伪装
	--self.inst:PushEvent("onattackother", {target = targ, weapon = weapon, projectile = projectile, stimuli = stimuli})
	inst:ListenForEvent('onattackother', function(_, data)
		if inst._isghost or not inst._isbeauty then return end
		local weapon = data.weapon
		local proj = data.projectile
		if not proj then 
			--多加点代码吧
			inst:ChangeToMonster()
		end
	end)

	inst:ListenForEvent('wbEvent_changetoghost', function(inst) 
		inst.components.combat:SetTarget(nil)
		ghostHUD(false) --在此处就隐藏界面
	end)

	local key = KEY_H
	TheInput:AddKeyDownHandler(key, function()
    	if not IsPaused() then
    		if inst.TriggerDisguiseUI then
    			inst:TriggerDisguiseUI()
    		end
    	end 
    end)
end

local function _w()
	local pt = GetPlayer():GetPosition()
	local ground = GetWorld()
	local x, y = ground.Map:GetTileCoordsAtPoint(pt.x, pt.y, pt.z)
	local original_tile_type = ground.Map:GetTileAtPoint(pt.x, pt.y, pt.z)
	print('ori:',x,y,original_tile_type)

	local basetile = GROUND.OCEAN_SHALLOW
	ground.Map:SetTile( x, y, basetile )
	ground.Map:RebuildLayer( original_tile_type, x, y )
	ground.Map:RebuildLayer( basetile, x, y )
end

return MakePlayerCharacter("white_bone", nil, assets,fn, start_inv)
