
local function CreateGhostFireFx(pos,s)
    s = s or 2
    local fx = SpawnPrefab('mk_halloween_firepuff_cold_'..math.random(3))
    fx.Transform:SetScale(s,s,s)
    fx.Transform:SetPosition(pos:Get())
end

local L = MK_MOD_LANGUAGE_SETTING
local boss_str1 = L and {'Your death has arrived...','Reincarnation and be a good person...',"Don't resist...",} or {'你大限已至...','轮回有命...','莫要再反抗了...',}
local boss_str2 = L and 'You think you can bribe me with just a little money?' or '就凭这点银两还想收买本王!'
local boss_str3 = L and 'No way!' or '不知好歹!'
local boss_str4 = L and {'Let go of this time',"You won't have a second chance."} or {'算你走运','你不会有下次了'}

local wc_str = L and {"Don't resist...","Go with us!",'come on!',"You can't refuse to die!!"} or {'别反抗了!','跟我们走吧!','阎王要你三更死，谁敢留你到五更?','来吧来吧!'}

local getstr = function(str)
    if type(str) == 'table' then
        return str[math.random(#str)]
    end
    return str
end

local say = function(inst,str)
    inst.components.talker:Say(getstr(str))
end

local states=
{
    State{

        name = "idle", tags = {"idle",},

        onenter = function(inst)
            if not inst.AnimState:IsCurrentAnimation('anim')then
                inst.AnimState:PlayAnimation("anim", true)
            end
            inst.sg:SetTimeout(math.random(8,15))
        end,

        ontimeout = function(inst)
            inst.sg:GoToState('hit')
        end,
    },

    State{

        name = "hit", tags = {"busy",},

        onenter = function(inst)
            local function DoTeleport(inst)
                if GetWorld().Map then
                    local max_tries = 10
                    for k = 1,max_tries do
                        local pos = Vector3(inst.Transform:GetWorldPosition())
                        local offset = 20
                        pos.x = pos.x + (math.random(2*offset)-offset)          
                        pos.z = pos.z + (math.random(2*offset)-offset)
                        if GetWorld().Map:GetTileAtPoint(pos:Get()) ~= GROUND.IMPASSABLE 
                            and not GetWorld().Map:IsWater(GetVisualTileType(pos:Get())) then
                            inst.Transform:SetPosition(pos:Get())
                            inst.sg:GoToState("appear")
                            break
                        end
                    end
                end
            end
            CreateGhostFireFx(inst:GetPosition())
            inst.components.colourtweener:StartTween({0,0,0,0},.25,DoTeleport)
            for k,_ in pairs(inst.lanterns)do
                k.components.colourtweener:StartTween({0,0,0,0},.25,function()k.Light:Enable(false)end)
            end
            --inst.AnimState:PlayAnimation("hit")       
        end,
    },    

    State{
        name = "appear", tags = {"busy",},

        onenter = function(inst)
            inst.components.health:SetInvincible(true)
        end,
        onexit = function(inst)
            inst.components.health:SetInvincible(false)
        end,
        timeline = {
            TimeEvent(FRAMES,function(inst)
                CreateGhostFireFx(inst:GetPosition())
                inst.components.colourtweener:StartTween({1,1,1,1},.25,function()inst.sg:GoToState('idle')end)
                for k in pairs(inst.lanterns)do
                    k.components.colourtweener:StartTween({1,1,1,1},.25,function()k.Light:Enable(true)end)
                end
            end)
        },

    },

    State{
        name = "death", tags = {"busy"},
        
        onenter = function(inst)
            local hasbook = function()
                for k,v in pairs(Ents)do
                    if v.prefab == 'yamaraja_book' and v:IsValid() then
                        return true
                    end
                end
                return GetPlayer().components.yamarajabuff.hasbuff
            end
            if not hasbook() then
                inst.components.lootdropper:AddLoot('yamaraja_book')
            end
            inst.components.colourtweener:StartTween({0,0,0,0},1)
            for k in pairs(inst.lanterns)do
                k.components.colourtweener:StartTween({0,0,0,0},.75)
            end
            CreateGhostFireFx(inst:GetPosition())
            RemovePhysicsColliders(inst) 
            inst.components.lootdropper:DropLoot(inst:GetPosition())  
            inst:KillAllMinions()
            say(inst,boss_str4)
        end,
        
    },
}

local event_handlers=
{
    EventHandler("attacked", function(inst) 
        if not inst.components.health:IsDead() then 
            inst.sg:GoToState("hit") 
        end 
    end),
    EventHandler("death", function(inst) inst.sg:GoToState("death") end),
}
local sg = StateGraph("yamaraja_boss", states, event_handlers, "idle", {})

local assets=
{
    Asset("ANIM", "anim/yamaraja_boss.zip"),
    Asset("SCRIPT", "anim/yamaraja.zip"),
    Asset("SOUND", "sound/maxwell.fsb"),
}

local function OnAttacked(inst,data)
    if data.attacker then
        inst.components.combat:SetTarget(data.attacker)
    end
end

local function GetPhase()
    return GetWorld():IsCave() and 'night' or GetClock():GetPhase()
end

local function SummonWC(inst)
    if not inst.ismerge 
        or inst.components.health:IsDead()
        or inst.components.leader:CountFollowers('yamaraja_wc') > 0
        or GetPhase() == 'day' then
        return
    end

    for _,v in pairs({'black_wc','white_wc'})do
        local wc = SpawnPrefab(v)
        local angle = 6.283*math.random()
        local r = math.random(3,5)
        local pos = inst:GetPosition()+Vector3(r*math.cos(angle),0,r*math.sin(angle))

        wc.Transform:SetPosition(pos:Get())
        wc.components.follower:SetLeader(inst)
        wc.components.knownlocations:RememberLocation('home', inst:GetPosition())
        inst.components.leader:AddFollower(wc)
        inst.miniontable[wc] = true

        CreateGhostFireFx(pos,1)
    end
end

local function MergeFn(inst) --从地底钻出
    if inst.inlimbo then
        inst:ReturnToScene()
    end
    inst.sg:GoToState('appear')
    inst.ismerge = true
    inst:SetSummonRate(2)
end

local function SummonGhost(inst) --召唤鬼魂
    local max = 20
    local r1,r2 = 3,7
    if inst.components.health:IsDead()
        or inst.components.leader:CountFollowers('yamaraja_ghost') > max
        or GetPhase() == 'day' then
        return
    end
    local ghost = SpawnPrefab('yamaraja_ghost')
    local angle = 6.283*math.random()
    local radius = math.random(r1,r2)
    local pos = Point(math.cos(angle)*radius,0,math.sin(angle)*radius)+inst:GetPosition()
    ghost.Transform:SetPosition(pos:Get())

    inst.components.leader:AddFollower(ghost)
    inst.miniontable[ghost] = true
    inst:ListenForEvent('death',function(i,data)
        if data and data.cause~='nopower'then
            inst.ghostcount = inst.ghostcount+1
            if inst.ghostcount >= inst.ghostcountmax then
                if not inst.ismerge then
                    inst:MergeFn()
                end
            end
        end
    end, ghost)
end

local function SetSummonRate(inst,rate)
    if inst.summon_task then
        inst.summon_task:Cancel()
        inst.summon_task = nil
    end
    inst._summonrate = rate
    if not rate then return end
    inst.summon_task = inst:DoPeriodicTask(rate, function()inst:SummonGhost()end)
end

local function EscapeFn(inst) ---逃跑(消失了)
    if GetPlayer().components.yamarajabuff.boss == inst then
        GetPlayer().components.yamarajabuff.boss = nil
    end
    CreateGhostFireFx(inst:GetPosition())
    inst:KillAllMinions()
    inst:Remove()
end

local function NewSpawnFn(inst, afterspawn)  --由组件刷出
    inst:RemoveFromScene()
    inst.ismerge = false
    local pos = GetPlayer():GetPosition()
    local theta = math.random() * 6.283
    for radius = 10,0,-5 do
        local offset = FindGroundOffset(pos, theta, radius,10)
        if offset then
            inst.Transform:SetPosition((pos+offset):Get())
            inst:SetSummonRate(5)
            --if inst.ismerge then
            --    inst.sg:GoToState('appear')
            --    inst:SetSummonRate(2)
            --else
            --    inst:SetSummonRate(5)
            --end
            return
        end
    end
    inst:Remove()
end

local function SpawnGhostLantern(inst)
    for i = 1,4 do
        local light = SpawnPrefab('yamaraja_light')
        inst.lanterns[light] = true
        inst:AddChild(light)
        --light.entity:SetParent(inst.entity)
        light.angle = i*PI/2
    end
end

local function SpawnMist(inst)
    local pos = inst:GetPosition()
    if not GetClosestInstWithTag('yamaraja_mist',inst,10) then
        local mist = SpawnPrefab('yamaraja_mist')
        mist.Transform:SetPosition(pos:Get())
        inst.mist[mist] = true
    end
end

local function IgnorePlayer(inst)
    if GetPlayer().components.yamarajabuff.boss == inst then
        GetPlayer().components.yamarajabuff:AddColdDown(20*480)
    end
    inst:EscapeFn()
end

local function OnGetItemFromPlayer(inst, giver, item)
    local num = item.components.stackable and item.components.stackable:StackSize() or 1
    --[[
    if inst.goldaccept == 0 and num >= 20 then
        say(inst,boss_str4)
        IgnorePlayer(inst)
    elseif inst.goldaccept > 0 and inst.goldaccept + num >= 20 then
        say(inst,boss_str4)
        IgnorePlayer(inst)
    --]]
    if inst.goldaccept + num >= 19 then
        say(inst,boss_str4)
        IgnorePlayer(inst)
    elseif inst.goldaccept + num < 19 then
        say(inst,boss_str2)
    end
    inst.goldaccept = inst.goldaccept + num
end

local function KillAllMinions(inst)
    for k,_ in pairs(inst.miniontable)do
        if k:IsValid() and not k.components.health:IsDead() then
            print(k.prefab)
            if k.prefab == 'yamaraja_ghost' then
                k.components.health:Kill('nopower')
            end
            if k.prefab:sub(-3) == '_wc' then
                k.sg:GoToState('leave')
            end
        end
    end
    for k,_ in pairs(inst.mist)do
        if k:IsValid() then
            k:Remove()
        end
    end
end

local function OnRefuseItem(inst, item)
    say(inst,boss_str3)
end

local function ListenPlayerHealth(inst, data)
    if data then
        if data.newpercent > 0.2 then
            inst:EscapeFn()
        end
    end
end

local function fn()
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    local snd = inst.entity:AddSoundEmitter()
    MakeObstaclePhysics(inst, .3)

    anim:SetBank("yamaraja_boss")
    anim:SetBuild("yamaraja_boss")
    --trans:SetScale(.4,.4,.5)
    
    anim:PlayAnimation("anim",true)
    inst.mist = {}

    inst.ismerge = false
    inst.goldaccept = 0
    inst.ghostcount = 0
    inst.ghostcountmax = 10

    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(1000)
    
    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(0)
    inst.components.combat.notags = {"yamaraja"}

    inst:AddComponent("leader")

    inst:AddComponent("sanityaura")
    inst.components.sanityaura.aurafn = function() return inst.ismerge and -TUNING.SANITYAURA_LARGE or 0 end

    inst:AddComponent("talker")
    inst.components.talker.ontalk = function() 
        inst.SoundEmitter:PlaySound('dontstarve/maxwell/talk_LP_world4','TALK') 
        inst:DoTaskInTime(2.5,function()inst.SoundEmitter:KillSound('TALK')end)
    end
    inst.components.talker.fontsize = 35
    inst.components.talker.font = TALKINGFONT
    inst.components.talker.colour = Vector3(10/255, 100/255, 230/255)
    inst.components.talker.offset = Vector3(0,-700,0)

    inst:AddComponent("inspectable")

    inst:AddComponent("colourtweener")

    inst:AddComponent("lootdropper")
    for i = 1,math.random(10,15)do
        inst.components.lootdropper:AddLoot('goldnugget')
    end

    inst:AddComponent("trader")
    inst.components.trader:SetAcceptTest(function(inst,item)return item and item.prefab == 'goldnugget' end)
    inst.components.trader.onaccept = OnGetItemFromPlayer
    inst.components.trader.onrefuse = OnRefuseItem
    function inst.components.trader:AcceptGift( giver, item )
        if not self.enabled then
            return false
        end
        if self:CanAccept(item, giver) then
            item.components.inventoryitem:RemoveFromOwner()

            if self.inst.components.inventory then
                self.inst.components.inventory:GiveItem(item)
            elseif self.deleteitemonaccept then
                item:Remove()
            end

            if self.onaccept then
                self.onaccept(self.inst, giver, item)
            end

            self.inst:PushEvent("trade", {giver = giver, item = item})

            return true
        end

        local frozen = false
        if  self.inst.components.freezable and self.inst.components.freezable:IsFrozen() then
            frozen = true        
        end

        if self.onrefuse and not frozen then
            self.onrefuse(self.inst, giver, item)
        end
    end

    inst:AddTag("epic")
    inst:AddTag("hostile")
    inst:AddTag("yamaraja")

    inst:ListenForEvent("attacked", OnAttacked)
    inst:ListenForEvent('daytime', function()EscapeFn(inst)end, GetWorld())
    inst:ListenForEvent("death", function(inst,data)
        if GetPlayer().components.yamarajabuff.boss == inst then
            GetPlayer().components.yamarajabuff:OnBossDefeated()
        end
    end)

    inst.MergeFn = MergeFn
    inst.SummonGhost = SummonGhost
    inst.EscapeFn = EscapeFn
    inst.SetSummonRate = SetSummonRate
    inst.NewSpawnFn = NewSpawnFn
    inst.KillAllMinions = KillAllMinions
    inst.SummonWC = SummonWC

    inst.sg = StateGraphInstance(sg, inst)
    SGManager:AddInstance(inst.sg)
    inst.sg:GoToState('idle')

    inst.summon_wc_task = inst:DoPeriodicTask(1,SummonWC)

    inst.miniontable = {}
    inst.lanterns = {}
    inst:DoTaskInTime(0,function()
        for k,_ in pairs(inst.components.leader.followers)do
            if k:IsValid()then
                inst.miniontable[k]=true
            end
        end
    end)
    SpawnGhostLantern(inst)
    inst:DoPeriodicTask(20,function()if inst.ismerge then say(inst,boss_str1)end end)

    inst:ListenForEvent('healthdelta',function(_, data)ListenPlayerHealth(inst, data)end, GetPlayer())

    inst:ListenForEvent('death', function() GetPlayer().components.yamarajabuff:OnBossDefeated()end)

    inst.OnSave = function(inst,data)
        data.ismerge = inst.ismerge
        data._summonrate = inst._summonrate
    end

    inst.OnLoad = function(inst,data)
        inst.ismerge = data.ismerge
        if not inst.ismerge then
            inst:RemoveFromScene()
        end
        if data._summonrate then
            inst:SetSummonRate(data._summonrate)
        end
    end

    return inst
end

return Prefab( "yamaraja_boss", fn, assets)
