local L = MK_MOD_LANGUAGE_SETTING

STRINGS.NAMES.YAMARAJA_GHOST = L and "Resentment soul" or '怨魂' 
STRINGS.CHARACTERS.GENERIC.DESCRIBE.YAMARAJA_GHOST = L and "These ghosts exude the sorrow of infiltration!" or "这些鬼魂散发着渗人的怨气!" 
STRINGS.CHARACTERS.MONKEY_KING.DESCRIBE.YAMARAJA_GHOST = L and "These ghosts exude the sorrow of infiltration!" or "小鬼! 莫要再上前一步!" 

STRINGS.NAMES.BLACK_WC = L and "Black impermanence" or '黑无常' 
STRINGS.CHARACTERS.GENERIC.DESCRIBE.BLACK_WC = L and "Don't catch me!" or "别抓我走!" 
STRINGS.CHARACTERS.MONKEY_KING.DESCRIBE.BLACK_WC = L and "Don't catch me!" or "范无救,滚开!" 

STRINGS.NAMES.WHITE_WC = L and "White impermanence" or '白无常' 
STRINGS.CHARACTERS.GENERIC.DESCRIBE.WHITE_WC = L and "Don't catch me!" or "别抓我走!" 
STRINGS.CHARACTERS.MONKEY_KING.DESCRIBE.WHITE_WC = L and "Don't catch me!" or "谢必安,给你孙爷爷滚开!" 

STRINGS.NAMES.YAMARAJA_BOSS = L and "Yama Raja" or '阎王' 
STRINGS.CHARACTERS.GENERIC.DESCRIBE.YAMARAJA_BOSS = L and "He is the god of hell from the East?!" or "来自东方的死神?!" 
STRINGS.CHARACTERS.MONKEY_KING.DESCRIBE.YAMARAJA_BOSS = L and "He is the god of hell from the East?!" or "呵,老阎王你倒是现身了." 

STRINGS.NAMES.YAMARAJA_BOOK = L and "Book of death" or '生死簿' 
STRINGS.CHARACTERS.GENERIC.DESCRIBE.YAMARAJA_BOOK = L and "Cross my name off." or "把我的名字勾掉。" 
STRINGS.CHARACTERS.MONKEY_KING.DESCRIBE.YAMARAJA_BOOK = L and "Cross my name off." or "嘿嘿待我把这生死一笔勾销!" 

local function CreateGhostFireFx(pos)
    local fx = SpawnPrefab('mk_halloween_firepuff_cold_'..math.random(3))
    fx.Transform:SetScale(2,2,2)
    fx.Transform:SetPosition(pos:Get())
end

local function SayWarningLine(inst)
    --inst.components.talker:Say("别皮了")
end

local MOVE_SPEED = 6

local assets=
{
    Asset("ANIM", "anim/ds_pig_basic.zip"),
    Asset("ANIM", "anim/ds_pig_actions.zip"),
    Asset("ANIM", "anim/ds_pig_attacks.zip"),
    Asset("ANIM", "anim/pig_build.zip"),
    Asset("ANIM", "anim/pigspotted_build.zip"),
    Asset("ANIM", "anim/pig_guard_build.zip"),
    Asset("ANIM", "anim/werepig_build.zip"),
    Asset("ANIM", "anim/werepig_basic.zip"),
    Asset("ANIM", "anim/werepig_actions.zip"),
    Asset("SOUND", "sound/pig.fsb"),

    Asset("ANIM", "anim/black_wc_build.zip"),
    Asset("ANIM", "anim/white_wc_build.zip"),

    Asset("ANIM", "anim/wc_hat_symbol.zip"),

    Asset("ANIM", "anim/yamaraja_light.zip"),
}

local function ontalk(inst, script)
    inst.SoundEmitter:PlaySound("dontstarve/pig/grunt")
end

local function CalcSanityAura(inst, observer) 
    return -TUNING.SANITYAURA_LARGE
end

local function OnAttacked(inst, data)
    local attacker = data.attacker
    inst:ClearBufferedAction()

    inst.components.combat:SetTarget(attacker)
end

local function NormalRetargetFn(inst)
    return FindEntity(inst, TUNING.PIG_TARGET_DIST,
        function(guy)
            return guy:HasTag("player") and guy.components.health and not guy.components.health:IsDead() and inst.components.combat:CanTarget(guy)
        end)
end
local function NormalKeepTargetFn(inst, target)
    return inst.components.combat:CanTarget(target)
end

local function HideSymbol(inst)
    for _,v in pairs({'foot','ear','leg','pig_foot','pig_ear','pig_leg'})do
        inst.AnimState:OverrideSymbol(v,'aa','bb')
    end
end

local function common()
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    local sound = inst.entity:AddSoundEmitter()
    local shadow = inst.entity:AddDynamicShadow()
    shadow:SetSize( 1.5, .75 )
    inst.Transform:SetFourFaced()

    inst.entity:AddLightWatcher()
    
    inst:AddComponent("talker")
    inst.components.talker.ontalk = ontalk
    inst.components.talker.fontsize = 35
    inst.components.talker.font = TALKINGFONT
    --inst.components.talker.colour = Vector3(133/255, 140/255, 167/255)
    inst.components.talker.offset = Vector3(0,-400,0)
    inst.components.talker:StopIgnoringAll()

    MakeCharacterPhysics(inst, 50, .5)
    MakePoisonableCharacter(inst)
    
    inst:AddComponent("locomotor") -- locomotor must be constructed before the stategraph
    inst.components.locomotor.runspeed = MOVE_SPEED 
    inst.components.locomotor.walkspeed = MOVE_SPEED

    inst:AddTag("yamaraja")
    inst:AddTag("yamaraja_wc")
    inst:AddTag("scarytoprey")
    anim:SetBank("pigman")
    anim:PlayAnimation("idle_loop")
    --anim:Hide("hat")

    ------------------------------------------
    inst:AddComponent("combat")
    inst.components.combat.hiteffectsymbol = "pig_torso"
    inst.components.combat:SetDefaultDamage(34)
    inst.components.combat:SetAttackPeriod(1)
    inst.components.combat:SetKeepTargetFunction(NormalKeepTargetFn)
    inst.components.combat:SetRetargetFunction(3, NormalRetargetFn)
    inst:ListenForEvent('doattack',function()
        inst.components.combat:SetAttackPeriod(TUNING.WEREPIG_ATTACK_PERIOD*GetRandomMinMax(0.5,1.5))
    end)

    MakeMediumBurnableCharacter(inst, "pig_torso")
    
    ------------------------------------------
    inst:AddComponent("follower")
    --inst.components.follower.maxfollowtime = TUNING.PIG_LOYALTY_MAXTIME
    ------------------------------------------
    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(300)
    ------------------------------------------

    inst:AddComponent("sanityaura")
    inst.components.sanityaura.aurafn = CalcSanityAura

    ------------------------------------------
    MakeMediumFreezableCharacter(inst, "pig_torso")
    
    ------------------------------------------

    inst:AddComponent("inspectable")

    inst:AddComponent('knownlocations')
    ------------------------------------------
    
    inst:ListenForEvent("attacked", OnAttacked) 

    inst:SetStateGraph('SGyamaraja_wc')
    inst:SetBrain(require "brains/frogbrain")

    return inst
end

local function fn1()
    local inst = common()
    inst.AnimState:SetBuild('black_wc_build')
    HideSymbol(inst)
    inst.AnimState:OverrideSymbol('swap_hat',"wc_hat_symbol",'black')
    return inst
end

local function fn2()
    local inst = common()
    inst.AnimState:SetBuild('white_wc_build')
    HideSymbol(inst)
    inst.AnimState:OverrideSymbol('swap_hat',"wc_hat_symbol",'white')
    return inst
end

local function ghostfn()
    local inst = SpawnPrefab('ghost')
    inst.prefab = 'yamaraja_ghost'
    inst:AddTag("yamaraja")

    inst.components.health:SetMaxHealth(101)

    inst.components.aura.auratestfn = function(i,dude) return not dude:HasTag('yamaraja') end
    inst:DoTaskInTime(GetRandomMinMax(20,40),function()inst.components.health:Kill('nopower') end)

    if inst.components.follower then
        inst:RemoveComponent("follower")
    end
    inst:AddComponent("follower")

    inst:DoTaskInTime(math.random(1,3),function()
        if inst.brain then
            if not inst.brain.followtarget then
                inst.brain.followtarget = GetPlayer()
            end
        end
    end)

    inst:ListenForEvent("daycomplete",function()inst.components.health:Kill('nopower')end, GetWorld() )

    return inst
end

local function lanternfn()
    local inst = CreateEntity()
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    local light = inst.entity:AddLight()
    light:SetFalloff(0.7)
    light:SetIntensity(.5)
    light:SetRadius(2)
    light:SetColour(0/255, 50/255, 209/255)
    light:Enable(false)
    inst.AnimState:SetBloomEffectHandle( "shaders/anim.ksh" )
    inst.AnimState:SetLightOverride(1)
    inst:Hide()

    inst.AnimState:SetBuild('yamaraja_light')
    inst.AnimState:SetBank('yamaraja_light')
    inst.AnimState:PlayAnimation('anim1',true)

    inst:AddComponent('colourtweener')

    inst.angle = 0
    local speed = .7*DEGREES
    local r = 3
    local s = 4
    inst.Transform:SetScale(s,s,s)
    inst:DoTaskInTime(0,function()inst:Show()end)
    inst:DoPeriodicTask(0,function()
        inst.angle = inst.angle+speed
        local pos = Point(math.cos(inst.angle)*r, 0, math.sin(inst.angle)*r)
        inst.Transform:SetPosition(pos:Get())
    end)
    return inst
end


return Prefab( "black_wc", fn1, assets),Prefab("white_wc", fn2, assets),
Prefab("yamaraja_ghost", ghostfn),
Prefab('yamaraja_light',lanternfn)
