require("stategraphs/commonstates")

--mod from DST lavaarena mod

local function ShakeAllCameras(type, p1, p2, p3, inst, SHAKE_DIST)
	local player = GetClosestInstWithTag("player", inst, SHAKE_DIST)
    if player then
        player.components.playercontroller:ShakeCamera(inst, type, p1, p2, p3, SHAKE_DIST)
    end
end

local function ShakeIfClose(inst)
    ShakeAllCameras("FULL", .2, .01, .1, inst, 8)
end

local function FootShake(inst)
    ShakeAllCameras("FULL", .2, .01, .1, inst, 8)
end

local function ShakePound(inst)
	inst.SoundEmitter:PlaySound("dontstarve/creatures/deerclops/bodyfall_dirt")
    ShakeAllCameras("FULL", 1.2, .03, .7, inst, 30)
end

local function ShakeRoar(inst)
    ShakeAllCameras("FULL", 0.8, .03, .5, inst, 30)
end

local actionhandlers = 
{
	ActionHandler(ACTIONS.ATTACK, "attack"),
}

local NOTAGS = {"mythTag_rhino", "notarget", "INLIMBO", 'FX'}

local function DoFrontAoe(inst)
local posx, posy, posz = inst.Transform:GetWorldPosition()
local angle = -inst.Transform:GetRotation() * DEGREES
local offset = 3
local targetpos = {x = posx + (offset * math.cos(angle)), y = 0, z = posz + (offset * math.sin(angle))} 
local ents = TheSim:FindEntities(targetpos.x, 0, targetpos.z, 1.5, nil, NOTAGS)
	for _,ent in ipairs(ents) do
		if ent ~= inst and inst.components.combat:IsValidTarget(ent) and ent.components.health and ent ~= inst.components.combat.target then
			inst:PushEvent("onareaattackother", { target = ent--[[, weapon = self.inst, stimuli = self.stimuli]] })
			ent.components.combat:GetAttacked(inst, 50)
			--print('front aoe on -> ', ent)
		end
	end
end

local function DoDashingAOE(inst)
	if inst.sg:HasStateTag("dashing") then
		local pos = inst:GetPosition()
		local angle = -inst.Transform:GetRotation() * DEGREES
		local offset = 2
		local targetpos = {x = pos.x + (offset * math.cos(angle)), y = 0, z = pos.z + (offset * math.sin(angle))} 
		local chargedents = {}
		local ents = TheSim:FindEntities(targetpos.x, 0, targetpos.z, 3, nil, NOTAGS)
		for i, v in ipairs(ents) do
			if v:IsValid() and v.components.health and not v.components.health:IsDead() and v.components.combat then
				if inst.dash_hit_targets[v] and GetTime() - inst.dash_hit_targets[v] <= 1.5 then
					--不造成伤害
					--print('no dmg!')
				else
					v.components.combat:GetAttacked(inst, 75)
					print('dmg! -> ', v)
					inst.dash_hit_targets[v] = GetTime()
					if inst.components.combat.target == v then
						inst.dash_hit_counter = (inst.dash_hit_counter or 0) + 1
					end
				end
			end
		end
	end
end

local function _WorldCollision()
	return rawget(_G, 'GetWorldCollision') and GetWorldCollision() or COLLISION.WORLD
end

local function _WaterCollision()
	return rawget(_G, 'GetWaterCollision') and GetWaterCollision() or COLLISION.WORLD
end

local function SetDashPhysics(inst)
	inst.Physics:ClearCollisionMask()
	inst.Physics:CollidesWith(_WorldCollision())
    inst.Physics:CollidesWith(_WaterCollision())
    inst.Physics:CollidesWith(COLLISION.OBSTACLES)
    inst.Physics:Teleport(inst.Transform:GetWorldPosition())
end

local function SetNormalPhysics(inst)
	ChangeToCharacterPhysics(inst)
end

local events=
{
	EventHandler("dospell", function(inst)
		if not inst.components.health:IsDead() and not inst.sg:HasStateTag("frozen")
			and (inst.sg:HasStateTag("hit") or not inst.sg:HasStateTag("busy") or inst.sg:HasStateTag("canattack")) then
			inst.sg:GoToState("spell_pre")
		end
	end),

	EventHandler("attacked", function(inst, data) 		
		if not inst.sg:HasStateTag('death') and not (inst.sg:HasStateTag('busy') or inst.sg:HasStateTag('attack') or inst.sg:HasStateTag('dashing')) then
			inst.sg:GoToState("hit")
		end 
	end),
    EventHandler("death", function(inst, data) inst.sg:GoToState("death") end),
    EventHandler("doattack", function(inst, data) 
		if not inst.components.health:IsDead() and (inst.sg:HasStateTag("hit") or not inst.sg:HasStateTag("busy") or inst.sg:HasStateTag("canattack")) then 
			inst.sg:GoToState("attack", data.target) 
		end 
	end),
    CommonHandlers.OnSleep(),
    CommonHandlers.OnLocomote(true, false),
    CommonHandlers.OnFreeze(),
}


local states=
{

    State{
        name = "idle",
        tags = {"idle", "canrotate"},
        onenter = function(inst, playanim)
        	if not inst.components.combat.target then
        		inst.sg:SetTimeout(1.5+math.random())
        	end
			inst.Transform:SetSixFaced()
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("idle_loop", true)
        end,

        ontimeout = function(inst)
        	if not inst.components.combat.target then
        		inst.sg:GoToState('pose')
        	end
        end,
    },
	
    State{
        name = "attack", 
        tags = {"attack", "busy"},
		
		onenter = function(inst, target)
        local target = inst.components.combat.target 
		if target and target:IsValid() then
            inst:ForceFacePoint(target:GetPosition())
            inst.sg.statemem.attacktarget = target
        end
        inst.components.combat:SetTarget(target)
		
		inst.Transform:SetSixFaced()
        
        inst.components.combat:StartAttack()
        inst.components.locomotor:Stop()
        inst.Physics:Stop()
		inst.SoundEmitter:PlaySound("dontstarve/forge2/rhino_drill/attack_2")
        inst.AnimState:PlayAnimation("attack")
	end,
	
    	timeline={
			--TimeEvent(1*FRAMES, function(inst)inst.SoundEmitter:PlaySound("dontstarve_DLC003/creatures/enemy/spidermonkey/swipe") end),
            TimeEvent(13*FRAMES, function(inst) 
				inst.components.combat:DoAttack(inst.components.combat.target)
				DoFrontAoe(inst)
			end),
   		},

        events={
            EventHandler("animover", function(inst) inst.sg:GoToState("idle")  end),
        },
	
    },

    --冲刺前的吼叫
    State{
		name = "dash_taunt",
        tags = {"busy",},

        onenter = function(inst, force)			
            inst.components.locomotor:Stop()
			inst.Transform:SetSixFaced()
            inst.AnimState:PlayAnimation("taunt")
        end,

		timeline=
        {
			TimeEvent(10*FRAMES, function(inst) inst.SoundEmitter:PlaySound("dontstarve/forge2/rhino_drill/taunt") end),
        },

        events=
        {
			EventHandler("animover", function(inst) inst.sg:GoToState("dash_pre") end),
        },
    },
	
	--冲刺起步(run_start)
	State{
        name = "dash_pre",
        tags = {"running", "attack", 'dashing'},
		
		onenter = function(inst, target)
			inst.dash_loop_counter = 0
			inst.dash_hit_counter = 0
			inst.dash_hit_targets = {}
			inst.last_dash_time = GetTime()
			inst.components.locomotor:RunForward()
			inst.AnimState:PlayAnimation('attack2_pre')
			inst.brain:Stop()

			local target = inst.components.combat.target
			if target and target:IsValid() then
				inst:ForceFacePoint(target:GetPosition())
				inst.sg.statemem.target = target
			end
		end,
	
		onexit = function(inst)
			SetNormalPhysics(inst)
			if inst.brain ~= nil then
				inst.brain:Start()
			end
	    end,

        events=
        {
            EventHandler("animover", function(inst) 
				inst.sg:GoToState("dash_loop", inst.sg.statemem.target)  
			end),
        },
	
    },
	--冲刺阶段
	State{
        name = "dash_loop",
        tags = {"running", "attack", "dashing"},
		
		onenter = function(inst, target)
			inst.dash_loop_counter = ( inst.dash_loop_counter or 0 ) + 1
			
			if inst.brain ~= nil then
				inst.brain:Stop()
			end

			SetDashPhysics(inst)
			inst.Transform:SetEightFaced()

			if target and target:IsValid() then
				inst.sg.statemem.target = target
			elseif inst.components.combat.target and inst.components.combat.target:IsValid() then
				inst.sg.statemem.target = inst.components.combat.target
			else
				inst.sg:GoToState('run_stop') return;
			end
			
			if inst.dash_dmg_task == nil then
				inst.dash_dmg_task = inst:DoPeriodicTask(0.15, DoDashingAOE)
			end

			inst.components.locomotor:RunForward()
			inst.Physics:SetMotorVelOverride(inst.components.locomotor.runspeed, 0, 0)
        	inst.AnimState:PlayAnimation("attack2_loop")
        
		end,
	
		onupdate = function(inst)
			local target = inst.sg.statemem.target
			if not (target and target:IsValid() and not target.components.health:IsDead()) then
				inst.sg:GoToState('run_stop')
			else
				local angle = inst.Transform:GetRotation() - inst:GetAngleToPoint(target:GetPosition())

				if angle > 180 then 
					angle = 360 - angle 
				elseif angle < -180 then 
					angle = 360 + angle 
				end
					
				local rot = 0
				local maxrott = 6
					
				if angle >= maxrott then
					rot = maxrott
				elseif angle <= -maxrott then
					rot = -maxrott
				else
					rot = angle
				end
				inst.Transform:SetRotation(inst.Transform:GetRotation() - rot)
				inst.Physics:SetMotorVelOverride(inst.components.locomotor.runspeed, 0, 0)
				--inst.Physics:SetMotorVelOverride(7,0,0)
			end	
		end,
		
		onexit = function(inst)
			SetNormalPhysics(inst)
			if inst.dash_dmg_task then
				inst.dash_dmg_task:Cancel()
				inst.dash_dmg_task = nil
			end
			if inst.brain ~= nil then
				inst.brain:Start()
			end
			inst.Physics:ClearMotorVelOverride()
	    end,
		
	    timeline=
	    {
			TimeEvent(9*FRAMES, PlayFootstep), TimeEvent(9*FRAMES, FootShake),
			TimeEvent(18*FRAMES, PlayFootstep), TimeEvent(18*FRAMES, FootShake),
	    },

        events=
        {
            EventHandler("animover", function(inst) 
				if inst.dash_loop_counter >= 30 then --最长冲刺距离
					inst.sg:GoToState("run_stop")
				elseif inst.dash_hit_counter >= 2 then
					inst.sg:GoToState('run_stop')
				elseif inst.components.burnable and inst.components.burnable:IsBurning() then
					inst.sg:GoToState('run')
				else
					inst.sg:GoToState('dash_loop', inst.sg.statemem.target)
				end	
			end),
        },
	
    },
	
	State{
		name = "hit",
        tags = {"busy", "hit"},

        onenter = function(inst)
			inst.Transform:SetSixFaced()
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("hit")
			inst.sg.mem.last_hit_time = GetTime()
			inst.SoundEmitter:PlaySound("dontstarve/forge2/rhino_drill/hit")
			inst.rams = 0
			inst.cheer_loop = 0
        end,
		
		onexit = function(inst)
			if inst.brain ~= nil then
				inst.brain:Start()
			end		
		end,

        events=
        {
			EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),
        },
    },
	
	State{
		name = "pose",
        tags = { "posing" , "idle", 'canattack'},

        onenter = function(inst)
        	inst.components.health:StartRegen(5, 0.5)
			inst.Physics:Stop()
			inst.Transform:SetSixFaced()
			inst.Transform:SetRotation(math.random(360))
			inst.AnimState:PlayAnimation("pose_pre", false)
			inst.AnimState:PushAnimation("pose_loop", true)
        end,

		timeline=
        {
			--TimeEvent(15*FRAMES, function(inst) inst.SoundEmitter:PlaySound("dontstarve/forge2/rhino_drill/cheer")end),
        },

        onexit = function(inst)
        	inst.components.health:StopRegen()
        end,
    },
	
	State{
		name = "taunt",
        tags = {"busy", "canattack"},

        onenter = function(inst, force)			
            inst.Physics:Stop()
			inst.Transform:SetSixFaced()
            inst.AnimState:PlayAnimation("taunt")
        end,

		timeline=
        {
			TimeEvent(10*FRAMES, function(inst) inst.SoundEmitter:PlaySound("dontstarve/forge2/rhino_drill/taunt") end),
        },

        events=
        {
			EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),
        },
    },
	
	State{
		name = "spell_pre",
        tags = {"busy", "spelling", "nosleep"},

        onenter = function(inst)
			inst.Transform:SetSixFaced()
            inst.Physics:Stop()
			inst.AnimState:PlayAnimation("cheer_pre")
        end,

        events=
        {
			EventHandler("animover", function(inst) inst.sg:GoToState("spell_loop") end),
        },
    },
	
	State{
		name = "spell_loop",
        tags = {"busy", "spelling", "nosleep"},

        onenter = function(inst, cb)
        	local target = inst.components.combat.target
        	if target and target:IsValid() and not target:GetIsOnWater() then
        		if inst:CanDoSpell() then
					local spell = SpawnPrefab(inst.spellprefab)
					spell.Transform:SetPosition(target:GetPosition():Get())
					spell:PushEvent('startcollapse')
					inst.last_spell_time = GetTime()
				else
					inst.last_spell_time = GetTime() - 5
				end
			end
            inst.Physics:Stop()		
			inst.AnimState:PlayAnimation("cheer_loop")
			inst.SoundEmitter:PlaySound("dontstarve/forge2/rhino_drill/cheer")
        end,

        events=
        {
			EventHandler("animover", function(inst) inst.sg:GoToState("spell_pst") end),
        },
    },
	
	State{
		name = "spell_pst",
        tags = {"busy", "nosleep"},

        onenter = function(inst, cb)
            inst.Physics:Stop()
			inst.AnimState:PlayAnimation("cheer_post")
        end,

        events=
        {
			EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),
        },
    },
	
	State{
        name = "death",
        tags = {"busy", "nointerrupt", 'death'},

        onenter = function(inst, isdead)
        	
			inst.components.health:SetInvincible(true) --防止二次死亡
			inst.Transform:SetSixFaced()
			inst.AnimState:PlayAnimation("death")

			if inst.components.propagator then
				inst.components.propagator:StopSpreading()
			end
			
			inst.SoundEmitter:PlaySound("dontstarve/forge2/rhino_drill/death")
            inst.Physics:Stop()
			RemovePhysicsColliders(inst)
            --inst.components.lootdropper:DropLoot()
        end,
		
		timeline=
        {
        	TimeEvent(60*FRAMES, function(inst) inst.AnimState:PlayAnimation('death_finalfinal') end),
        	--和第二个动画同步
			--TimeEvent(37*FRAMES, function(inst) inst.SoundEmitter:PlaySound("dontstarve/forge2/rhino_drill/death_final_final") end),
			TimeEvent(80*FRAMES, function(inst) inst.components.lootdropper:DropLoot() end),
        },
    },
	
	State {
        name = "sleep",
        tags = { "sleeping", "busy" }, --add tag "busy" if you hate sliding

        onenter = function(inst)
			inst.Transform:SetSixFaced()
            inst.components.locomotor:StopMoving()
            inst.AnimState:PlayAnimation("sleep_pre")
			inst.SoundEmitter:PlaySound("dontstarve/forge2/rhino_drill/sleep_pre")

        end,

        timeline = 
		{
			TimeEvent(38*FRAMES, function(inst) 
				ShakeIfClose(inst) 
				inst.SoundEmitter:PlaySound("dontstarve/creatures/lava_arena/trails/bodyfall") 
			end),
		},

        events =
        {
            EventHandler("animover", function(inst) inst.sg:GoToState("sleeping") end ),
            EventHandler("onwakeup", function(inst) inst.sg:GoToState("wake") end),
        },
    },

    State
    {
        name = "sleeping",
        tags = { "sleeping", "busy" },
		
		onenter = function(inst)
			inst.components.locomotor:StopMoving()
			inst.SoundEmitter:PlaySound("dontstarve/forge2/rhino_drill/sleep_in")
			inst.AnimState:PlayAnimation("sleep_loop")
		end,
			
		timeline=
        {
			TimeEvent(24*FRAMES, function(inst) inst.SoundEmitter:PlaySound("dontstarve/forge2/rhino_drill/sleep_out") end),
        },

        events =
        {
            EventHandler("animover", function(inst) inst.sg:GoToState("sleeping") end ),
			EventHandler("attacked", function(inst) inst.sg:GoToState("wake") end ),
			EventHandler("onwakeup", function(inst) inst.sg:GoToState("wake") end),
        },
    },

    State
    {
        name = "wake",
        tags = { "busy", "waking", "nosleep" },

        onenter = function(inst)
            if inst.components.locomotor ~= nil then
                inst.components.locomotor:StopMoving()
            end
            inst.AnimState:PlayAnimation("sleep_pst")
            if inst.components.sleeper ~= nil and inst.components.sleeper:IsAsleep() then
                inst.components.sleeper:WakeUp()
            end
        end,

        events =
        {
            EventHandler("animover", function(inst) 
				if inst.sg.mem.wants_to_cheer then
					--print("DEBUG: Waking up to cheer!")
					inst.sg:GoToState("cheer_pre")
				else
					inst.sg:GoToState("idle")
				end
			end),
        },
    },
	
	State
    {
        name = "run_start",
        tags = { "moving", "running", "canrotate" },

        onenter = function(inst)
        	inst.Transform:SetSixFaced()
        	if inst.components.combat.target and inst.components.combat.target:IsValid()
        		and inst:CanDash()
        		and inst:GetDistanceSqToInst(inst.components.combat.target) > 3*3 then
        		inst.sg:GoToState('dash_taunt')
        	else
				inst.components.locomotor:RunForward()
				inst.AnimState:PlayAnimation("run_pre")
			end
			
        end,
		
        events =
        {
            EventHandler("animqueueover", function(inst) inst.sg:GoToState("run") end ),
        },
		
		onexit = function(inst)
			inst.Transform:SetSixFaced()
        end,
    },
	
	State
    {
        name = "run",
        tags = { "moving", "running", "canrotate" },

        onenter = function(inst)
			inst.Transform:SetEightFaced()
			inst.components.locomotor:RunForward()
			inst.AnimState:PlayAnimation("run_loop")
        end,
		
		timeline = {

			TimeEvent(9*FRAMES, PlayFootstep),
			TimeEvent(9*FRAMES, FootShake),
			TimeEvent(18*FRAMES, PlayFootstep),
			TimeEvent(18*FRAMES, FootShake),
		},
		
        events= {
			EventHandler("animqueueover", function(inst) 
				inst.sg:GoToState("run")  
			end),
		},
			
		onexit = function(inst)
			inst.Transform:SetSixFaced()
        end,	

    },
	
	State
    {
        name = "run_stop",
        tags = { "idle" },

        onenter = function(inst)
        	inst.Transform:SetSixFaced()
            inst.components.locomotor:StopMoving()
			inst.AnimState:PlayAnimation("run_pst")           
        end,
		
		timeline = {

			TimeEvent(16*FRAMES, function(inst) inst.SoundEmitter:PlaySound("dontstarve/forge2/rhino_drill/grunt") end),
		},

        events =
        {
            EventHandler("animqueueover", function(inst) inst.sg:GoToState("idle") end ),
        },
    },

    State{
        name = "frozen",
        tags = {"busy", "frozen"},
        
        onenter = function(inst)
        	inst.AnimState:SetTwoFaced()
            if inst.components.locomotor then
                inst.components.locomotor:StopMoving()
            end
            inst.AnimState:PlayAnimation("fossilized")
            inst.AnimState:SetTime(2)
            inst.SoundEmitter:PlaySound("dontstarve/common/freezecreature")
            if not inst.icefx then
            	inst.icefx = MK_MOD_UTIL.MakeFxAt(Point(0,0,0), 'myth_frozen', 'myth_frozen',
            	'rhino3', nil, function(inst) 
            		inst.Transform:SetRotation(0)
            		inst.Transform:SetTwoFaced()
            		inst.AnimState:SetLayer(LAYER_BACKGROUND) 
            		inst.AnimState:SetScale(3.2, 2.8 ,1)
            		inst.AnimState:SetAddColour(1,1,1,0.5)
            	end)
            	inst.icefx.entity:SetParent(inst.entity)
            end
            --
        end,
        
        onexit = function(inst)
        	inst.AnimState:SetSixFaced()
        	if not inst.sg.statemem.thaw then
        		if inst.icefx then inst.icefx:Remove() inst.icefx = nil end
        	end
        end,
        
        events=
        {   
            EventHandler("onthaw", function(inst)inst.sg.statemem.thaw = true; inst.sg:GoToState("thaw") end ),
        },
    },

    State{
        name = "thaw",
        tags = {"busy", "thawing"},
        
        onenter = function(inst) 
            if inst.components.locomotor then
                inst.components.locomotor:StopMoving()
            end

            inst.SoundEmitter:PlaySound("dontstarve/common/freezethaw", "thawing")
            
            if not inst.icefx then
            	inst.icefx = MK_MOD_UTIL.MakeFxAt(Point(0,0,0), 'myth_frozen', 'myth_frozen',
            	'rhino3', nil, function(inst) 
            		inst.Transform:SetRotation(180)
            		inst.Transform:SetTwoFaced()
            		inst.AnimState:SetLayer(LAYER_BACKGROUND) 
            		inst.AnimState:SetScale(3.2, 2.8 ,1)
            		inst.AnimState:SetAddColour(1,1,1,0.5)
            	end)
            	inst.icefx.entity:SetParent(inst.entity)
            end
        end,
        
        onexit = function(inst)
            inst.SoundEmitter:KillSound("thawing")
            if inst.icefx then inst.icefx:Remove() inst.icefx = nil end
        end,

        events =
        {   
            EventHandler("unfreeze", function(inst)
                inst.sg:GoToState("hit")
            end ),
        },
    },
}
    
return StateGraph("rhino3", states, events, "pose", actionhandlers)

