require("stategraphs/commonstates")
local actionhandlers = 
{

}

local events=
{
    EventHandler("death", function(inst) inst.sg:GoToState("death") end),
    EventHandler("doattack", function(inst) if not inst.components.health:IsDead() and not inst.sg:HasStateTag("busy") then inst.sg:GoToState("attack") end end),
    
    EventHandler("locomote", function(inst) 
        if inst.components.health:IsDead() then return end
        if not inst.components.locomotor:WantsToMoveForward() then
            if inst.sg:HasStateTag("moving") then
                inst.sg:GoToState("run_stop")
            end
        else
            if not inst.sg:HasStateTag("moving") then
                inst.sg:GoToState("run_start")
            end
        end
    end),
}

local states=
{
    State{
        name = 'spawn',
        tags = {'idle', 'busy'},
        onenter = function(inst)
            inst.AnimState:PlayAnimation('idle_loop')
        end,

        events = {
            EventHandler('animover', function(inst) inst.sg:GoToState('idle') end),
        }
    },
 
    State{
        
        name = "idle",
        tags = {"idle", "canrotate"},
        onenter = function(inst, playanim)
            inst.Physics:Stop()
            if playanim then
                inst.AnimState:PlayAnimation(playanim)
                inst.AnimState:PushAnimation("idle_loop")
            else
                inst.AnimState:PlayAnimation("idle_loop")
            end
        end,

        events = {
            EventHandler('animover', function(inst)
                if math.random() < 0.25 then
                    inst.sg:GoToState('idle_dance')
                else
                    inst.sg:GoToState("idle")
                end
            end),
        },
    },

    State{
        name = 'idle_dance',
        tags = {'idle', 'canrotate'},
        onenter = function(inst)
            inst.Physics:Stop()
            local animlist = {
                {'emoteXL_pre_dance0', 'emoteXL_loop_dance0', 'emoteXL_pst_dance0', 10, 20},
                --{'emoteXL_pre_dance6', 'emoteXL_loop_dance6', 'emoteXL_pst_dance6', 4, 6}, --火鸡舞
                {'emoteXL_pre_dance7', 'emoteXL_loop_dance7', 'emoteXL_pst_dance7', 2, 3}, --踢踏舞
                --{'emoteXL_pre_dance8', 'emoteXL_loop_dance8', 'emoteXL_pst_dance8', 1, 1}, --机械舞
                {'idle_inaction'},
            }
            local anim = animlist[math.random(#animlist)]
            if #anim == 1 then
                inst.AnimState:PlayAnimation(anim[1])
            else
                inst.AnimState:PlayAnimation(anim[1])
                for i = 1, math.random(anim[4], anim[5]) do
                    inst.AnimState:PushAnimation(anim[2], false)
                end
                inst.AnimState:PushAnimation(anim[3], false)
            end
        end,

        events = {
            EventHandler('animqueueover', function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
                end
            end),
        },
    },
    
    State{
        name = "attack",
        tags = {"attack"},
        
        onenter = function(inst, cb)
            inst.Physics:Stop()
            inst.components.combat:StartAttack()
            inst.AnimState:PlayAnimation("atk")
        end,
        
        timeline=
        {
            --TimeEvent(20*FRAMES, function(inst) inst.SoundEmitter:PlaySound("dontstarve/frog/attack_spit") end),
            --TimeEvent(20*FRAMES, function(inst) inst.SoundEmitter:PlaySound("dontstarve/frog/attack_voice") end),
            TimeEvent(8*FRAMES, function(inst) inst.components.combat:DoAttack() end),
            TimeEvent(15*FRAMES, function(inst) inst.sg:GoToState('idle') end),
        },
        
        events=
        {
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),
        },
    },
    
    State{
        name = "fall",
        tags = {"busy"},
        onenter = function(inst)
            inst.Physics:SetDamping(0)
            inst.Physics:SetMotorVel(0,-20+math.random()*10,0)
            inst.AnimState:PlayAnimation("fall_idle", true)
        end,
        
        onupdate = function(inst)
            local pt = Point(inst.Transform:GetWorldPosition())
            if pt.y < 2 then
                inst.Physics:SetMotorVel(0,0,0)
            end
            
            if pt.y <= .1 then
                pt.y = 0

                -- TODO: 20% of the time, they should explode on impact!

                inst.Physics:Stop()
                inst.Physics:SetDamping(5)
                inst.Physics:Teleport(pt.x,pt.y,pt.z)
                inst.DynamicShadow:Enable(true)
                inst.SoundEmitter:PlaySound("dontstarve/frog/splat")
                inst.sg:GoToState("idle", "jump_pst")
            end
        end,

        onexit = function(inst)
            local pt = inst:GetPosition()
            pt.y = 0
            inst.Transform:SetPosition(pt:Get())
        end,
    },    

    State{
        name = "run_start",
        tags = {"moving", "running", "canrotate"},

        onenter = function(inst)
            inst.components.locomotor:RunForward()
            inst.AnimState:PlayAnimation("run_pre")
        end,

        onupdate = function(inst)
            inst.components.locomotor:RunForward()
        end,

        events=
        {   
            EventHandler("animover", function(inst) inst.sg:GoToState("run") end ),        
        },
    },

    State{
        name = "run",
        tags = {"moving", "running", "canrotate"},

        onenter = function(inst)
            inst.components.locomotor:RunForward()
            inst.AnimState:PlayAnimation("run_loop")
        end,

        onupdate = function(inst)
            inst.components.locomotor:RunForward()
        end,

        events=
        {   
            EventHandler("animover", function(inst) inst.sg:GoToState("run") end ),        
        },
    },

    State{
        name = "run_stop",
        tags = {"canrotate", 'idle'},

        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("run_pst")
        end,

        events=
        {   
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end ),        
        },
    },
    
   
    State{
        name = "death",
        tags = {"busy"},
        
        onenter = function(inst)
            --inst.SoundEmitter:PlaySound("dontstarve/frog/die")
            --骨头碎掉的声音
            inst.AnimState:PlayAnimation("death")
            inst.Physics:Stop()
            RemovePhysicsColliders(inst)            
            inst.components.lootdropper:DropLoot(Vector3(inst.Transform:GetWorldPosition()))            
        end,
    },

}
    
return StateGraph("wb_skeleton", states, events, "idle", actionhandlers)

