local function DoTalkSound(inst)
    if inst.talksoundoverride ~= nil then
        inst.SoundEmitter:PlaySound(inst.talksoundoverride, "talk")
        return true
    elseif not inst:HasTag("mime") then
        inst.SoundEmitter:PlaySound((inst.talker_path_override or "dontstarve/characters/")..(inst.soundsname or inst.prefab).."/ghost_LP", "talk")
        return true
    end
end

local function StopTalkSound(inst, instant)
    if not instant and inst.endghosttalksound ~= nil and inst.SoundEmitter:PlayingSound("talk") then
        inst.SoundEmitter:PlaySound(inst.endghosttalksound)
    end
    inst.SoundEmitter:KillSound("talk")
end

local actionhandlers =
{
    ActionHandler(ACTIONS.WB_HAUNT, "haunt_pre"),
    ActionHandler(ACTIONS.JUMPIN, "jumpin_pre"),
    ActionHandler(ACTIONS.ATTACK, function() end),
}

local events =
{
    EventHandler("locomote", function(inst, data)
        if inst.sg:HasStateTag("busy") then
            return
        end
        local is_moving = inst.sg:HasStateTag("moving")
        local should_move = inst.components.locomotor:WantsToMoveForward()

        if is_moving and not should_move then
            inst.sg:GoToState("idle")
        elseif not is_moving and should_move then
            inst.sg:GoToState("run")
        elseif data.force_idle_state and not (is_moving or should_move or inst.sg:HasStateTag("idle")) then
            inst.sg:GoToState("idle")
        end
    end),

    EventHandler("ontalk", function(inst, data)
        if inst.sg:HasStateTag("idle") then
            inst.sg:GoToState("talk", data.noanim)
        end
    end),

    EventHandler('death', function(inst)
        inst.sg:GoToState('dissipate')
    end),

    EventHandler('wbEvent_changetohuman', function(inst, data)
        print('event!')
        print(data.bone)
        if data and data.bone and not inst.sg:HasStateTag('wb_2human') then
            inst.sg:GoToState('wb_2human')
            --print(123)
        end
    end),
}

local states =
{
    State
    {
        name = "idle",
        tags = { "idle", "canrotate" },
        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.components.locomotor:Clear()
            if not inst.AnimState:IsCurrentAnimation("idle") then
                inst.AnimState:PlayAnimation("idle", true)
            end
        end,
    },

    State
    {
        name = "run",
        tags = { "moving", "running", "canrotate", },

        onenter = function(inst)
            inst.components.locomotor:RunForward()
            if not inst.AnimState:IsCurrentAnimation("idle") then
                inst.AnimState:PlayAnimation("idle", true)
            end
        end,

        onupdate = function(inst)
            inst.components.locomotor:RunForward()
        end,
    },

    State
    {
        name = "haunt_pre",
        tags = { "doing", "busy", "haunt", 'canrevive' },

        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("dissipate")
            inst.SoundEmitter:PlaySound("dontstarve/ghost/ghost_haunt")
        end,

        timeline =
        {
            TimeEvent(15 * FRAMES, function(inst)
                inst:PerformBufferedAction()
            end)
        },

        events =
        {
            EventHandler("animover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("haunt")
                end
            end),

            EventHandler('wbEvent_changetohuman', function(inst, data) --附身到一个骨头上
                print('event!')
                print(data.bone)
                if data and data.bone and data.bone:IsValid() and not inst.sg:HasStateTag('wb_2human') then
                    inst.sg:GoToState('wb_2human', data.bone)
                    --print(123)
                end
            end),
        },
    },

    State
    {
        name = "haunt",
        tags = { "doing", "busy", "haunt"},

        onenter = function(inst, fromdeath)
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("appear")
            if fromdeath == 'death' then
                inst:DoTaskInTime(1, function() inst.components.talker:Say(
                    MK_MOD_LANGUAGE_SETTING and 'Um.. this body is still too fragile' or '呵, 这身体还是太脆弱')
                end)
            elseif fromdeath == 'heart' then
                inst:DoTaskInTime(1, function() inst.components.talker:Say(
                    MK_MOD_LANGUAGE_SETTING and 'No soma, no limit' or '身躯不过是个累赘')
                end)
            end
        end,

        events =
        {
            EventHandler("animover", function(inst)
                if inst.AnimState:AnimDone() and inst.entity:IsVisible() then --hidden if resurrecting
                    inst.sg:GoToState("idle")
                end
            end),
        },
        onexit = function(inst)
            if inst.components.playercontroller then
                inst.components.playercontroller:Enable(true)
            end
        end,
    },

    State
    {
        --真正的死亡
        name = "dissipate",
        tags = { "busy", 'mythSGtag_locked'},

        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.components.locomotor:Clear()
            inst:ClearBufferedAction()
            
            inst.Light:Enable(false)
            inst.AnimState:PlayAnimation("dissipate")

            inst.SoundEmitter:PlaySound(
                inst:HasTag("girl") and
                "dontstarve/ghost/ghost_girl_howl" or
                "dontstarve/ghost/ghost_howl"
            )

        end,

        events =
        {
            EventHandler("animover", function(inst)
                inst:Hide()
                inst.DynamicShadow:Enable(false)
            end),
        },
    },

    State
    {
        name = "talk",
        tags = { "idle", "talking" },

        onenter = function(inst)
            DoTalkSound(inst)
            inst.sg:SetTimeout(1.5 + math.random() * .5)
        end,

        ontimeout = function(inst)
            inst.sg:GoToState("idle")
        end,

        events =
        {
            EventHandler("donetalking", function(inst)
                inst.sg:GoToState("idle")
            end),
        },

        onexit = StopTalkSound,
    },

    State{
        name = "jumpin_pre",
        tags = { "doing", "busy", "canrotate" },

        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("dissipate", false)
            inst.SoundEmitter:PlaySound("dontstarve/ghost/ghost_haunt", nil, nil)
        end,

        events =
        {
            EventHandler("animover", function(inst)
                if inst.AnimState:AnimDone() then
                    if inst.bufferedaction ~= nil then
                        inst:PerformBufferedAction()
                    else
                        inst.sg:GoToState("idle")
                    end
                end
            end),
        },
    },

    State
    {
        name = "jumpin",
        tags = { "doing", "busy", "canrotate" },

        onenter = function(inst, data)
            inst.components.locomotor:Stop()
            --inst.AnimState:PlayAnimation("dissipate")

            inst.sg.statemem.target = data.teleporter
            inst.sg.statemem.teleportarrivestate = "jumpout"

            inst.sg.statemem.target:PushEvent("starttravelsound", inst)
            if inst.sg.statemem.target ~= nil and inst.sg.statemem.target.components.teleporter ~= nil
                and inst.sg.statemem.target.components.teleporter:Activate(inst) then
                inst.sg.statemem.isteleporting = true
                if inst.components.playercontroller ~= nil then
                    inst.components.playercontroller:Enable(false)
                end
                inst:Hide()
            else
                inst.sg:GoToState("jumpout")
            end
        end,

        onexit = function(inst)
            if inst.sg.statemem.isteleporting then
                if inst.components.playercontroller ~= nil then
                    inst.components.playercontroller:Enable(true)
                end
                inst:Show()
            end
        end,
    },

    State
    {
        name = "jumpout",
        tags = { "doing", "busy", "canrotate", "nopredict" },

        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("appear")
        end,

        events =
        {
            EventHandler("animover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
                end
            end),
        },
    },

    State{
        name = "jumpin",
        tags = {"doing", "canrotate"},
        
        onenter = function(inst)
            inst.sg.statemem.action = inst:GetBufferedAction()
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("dissipate")

            --inst:DoTaskInTime(4.7, function(inst) inst.SoundEmitter:PlaySound("dontstarve/movement/bodyfall_dirt", "bodyfall") end )
        end,
        
        timeline =
        {
            -- this is just hacked in here to make the sound play BEFORE the player hits the wormhole
            TimeEvent(19*FRAMES, function(inst)
                if inst.sg.statemem.action and inst.sg.statemem.action.target and inst.sg.statemem.action.target.prefab == "bermudatriangle" then
                    inst.SoundEmitter:PlaySound("dontstarve_DLC002/common/bermudatriangle_travel", "wormhole_travel")
                else
                    inst.SoundEmitter:PlaySound("dontstarve/common/teleportworm/travel", "wormhole_travel")
                end
            end),
        },

        events=
        {
            EventHandler("animover", function(inst)
                inst:PerformBufferedAction()
                inst.sg:GoToState("idle") 
            end ),
        },
    },

    --复活
    State{
        name = 'wb_2human',
        tags = {'busy', 'mythSGtag_locked'},
        onenter = function(inst, bone)
            --来个烟雾?
            MK_MOD_UTIL.MakeFxAt(inst:GetPosition(), 'lavaarena_player_revive_fx', 'wb_revive_myth', 'player_revive')
            --讲道理应该有一个熔炉的音效
            inst.sg.statemem.bone = bone
        end,
        timeline = {
            TimeEvent(2.8, function(inst) 
                SpawnPrefab('collapse_small').Transform:SetPosition(inst:GetPosition():Get())
                local bone = inst.sg.statemem.bone
                if bone and bone:IsValid() and bone.components.wb_bone then
                    inst.sg.statemem.maxhealth = bone.components.wb_bone.value
                    if bone.components.inventory and bone.ToPlayer then
                        bone:ToPlayer(inst)
                    end
                    if bone.components.stackable then
                        bone.components.stackable:Get():Remove()
                    else
                        bone:Remove()
                    end
                end
                inst.sg:SetTimeout(0.1)
            end),
        },
        ontimeout = function(inst)
            inst.sg:RemoveStateTag('mythSGtag_locked')
            inst.components.health:SetMaxHealth(inst.sg.statemem.maxhealth or 150)
            inst.components.sanity:DoDelta(-50, true)
            inst:ChangeToHuman()
            inst.sg:GoToState('wb_getup')
        end,
    },

}

return StateGraph("wb_ghost", states, events, "idle", actionhandlers)
